@@ -6,7 +6,7 @@
     @license: GNU GPL, see COPYING for details.
 """
 
-import os, time, codecs, re
+import os, time, codecs, re, errno
 
 from MoinMoin import caching, config, user, util, wikiutil, error
 from MoinMoin.Page import Page
@@ -23,55 +23,15 @@ import MoinMoin.util.datetime
 ### Javascript code for editor page
 #############################################################################
 
-# This code is internal to allow I18N, else we'd use a .js file;
 # we avoid the "--" operator to make this XHTML happy!
 _countdown_js = """
+%(countdown_script)s
 <script type="text/javascript">
-var timeout_min = %(lock_timeout)s;
-var state = 0; // 0: start; 1: long count; 2: short count; 3: timeout; 4/5: blink
-var counter = 0, step = 1, delay = 1;
-
-function countdown() {
-    // change state if counter is down
-    if (counter <= 1) {
-        state += 1
-        if (state == 1) {
-            counter = timeout_min
-            step = 1
-            delay = 60000
-        }
-        if (state == 2) {
-            counter = 60
-            step = 5
-            delay = step * 1000
-        }
-        if (state == 3 || state == 5) {
-            window.status = "%(lock_expire)s"
-            state = 3
-            counter = 1
-            step = 1
-            delay = 500
-        }
-        if (state == 4) {
-            // blink the above text
-            window.status = " "
-            counter = 1
-            delay = 250
-        }
-    }
-
-    // display changes
-    if (state < 3) {
-        var msg
-        if (state == 1) msg = "%(lock_mins)s"
-        if (state == 2) msg = "%(lock_secs)s"
-        window.status = msg.replace(/#/, counter)
-    }
-    counter -= step
-
-    // Set timer for next update
-    setTimeout("countdown()", delay)
-}
+var countdown_timeout_min = %(lock_timeout)s
+var countdown_lock_expire = "%(lock_expire)s"
+var countdown_lock_mins = "%(lock_mins)s"
+var countdown_lock_secs = "%(lock_secs)s"
+addLoadEvent(countdown)
 </script>
 """
 
@@ -113,10 +73,6 @@ class PageEditor(Page):
         """
         Page.__init__(self, request, page_name, **keywords)
         
-        # already done:
-        #self.request = request
-        #self.cfg = request.cfg
-        
         self._ = request.getText
 
         self.do_revision_backup = keywords.get('do_revision_backup', 1)
@@ -174,11 +130,12 @@ class PageEditor(Page):
         _ = self._
         self.request.http_headers(self.request.nocache)
 
+        raw_body = ''
         msg = None
         conflict_msg = None
         edit_lock_message = None
         preview = kw.get('preview', None)
-        emit_anchor = not kw.get('staytop', 0)
+        staytop = kw.get('staytop', 0)
 
         from MoinMoin.formatter.text_html import Formatter
         self.request.formatter = Formatter(self.request, store_pagelinks=1)
@@ -193,15 +150,20 @@ class PageEditor(Page):
             # the web interface, but catch it just in case...
             msg = _('Cannot edit old revisions!')
         else:
-            # try to acquire edit lock
-            ok, edit_lock_message = self.lock.aquire()
-            if not ok:
-                # failed to get the lock
-                if preview is not None:
-                    edit_lock_message = _('The lock you held timed out, be prepared for editing conflicts!'
-                        ) + "<br>" + edit_lock_message
-                else:
-                    msg = edit_lock_message
+            try:
+                # try to acquire edit lock
+                ok, edit_lock_message = self.lock.acquire()
+                if not ok:
+                    # failed to get the lock
+                    if preview is not None:
+                        edit_lock_message = _('The lock you held timed out. Be prepared for editing conflicts!'
+                            ) + "<br>" + edit_lock_message
+                    else:
+                        msg = edit_lock_message
+            except OSError, err:
+                if err.errno != errno.ENAMETOOLONG:
+                    raise err
+                msg = _("Page name is too long, try shorter name.")
 
         # Did one of the prechecks fail?
         if msg:
@@ -273,21 +235,29 @@ Have a look at the diff of %(difflink)s to see what has been changed.""") % {
             title % {'pagename': self.split_title(self.request),},
             page=self,
             pagename=self.page_name, msg=status,
-            body_onload=self.lock.locktype and 'countdown()' or '', # broken / bug in Mozilla 1.5, when using #preview
             html_head=self.lock.locktype and (
                 _countdown_js % {
+                     'countdown_script': self.request.theme.externalScript('countdown'),
                      'lock_timeout': lock_timeout,
                      'lock_expire': lock_expire,
                      'lock_mins': lock_mins,
                      'lock_secs': lock_secs,
-                    }) or ''
+                    }) or '',
+            editor_mode=1,
         )
         
         self.request.write(self.request.formatter.startContent("content"))
-        
-        # get the text body for the editor field
-        if form.has_key('template'):
-            # "template" parameter contains the name of the template page
+
+        # Get the text body for the editor field.
+        # TODO: what about deleted pages? show the text of the last revision or use the template?
+        if preview is not None:
+            raw_body = self.get_raw_body()
+        elif self.exists():
+            # If the page exists, we get the text from the page.
+            # TODO: maybe warn if template argument was ignored because the page exists?
+            raw_body = self.get_raw_body()
+        elif form.has_key('template'):
+            # If the page does not exists, we try to get the content from the template parameter.
             template_page = wikiutil.unquoteWikiname(form['template'][0])
             if self.request.user.may.read(template_page):
                 raw_body = Page(self.request, template_page).get_raw_body()
@@ -296,42 +266,7 @@ Have a look at the diff of %(difflink)s to see what has been changed.""") % {
                 else:
                     self.request.write(_("[Template %s not found]") % (template_page,), '<br>')
             else:
-                raw_body = ''
                 self.request.write(_("[You may not read %s]") % (template_page,), '<br>')
-        else:
-            raw_body = self.get_raw_body()
-
-        # send text above text area
-        template_param = ''
-        if form.has_key('template'):
-            template_param = '&amp;template=' + form['template'][0]
-        self.request.write('<p>')
-        self.request.write(wikiutil.getSysPage(self.request, 'HelpOnFormatting').link_to(self.request))
-        self.request.write(" | ", wikiutil.getSysPage(self.request, 'InterWiki').link_to(self.request))
-        if preview is not None and emit_anchor:
-            self.request.write(' | <a href="#preview">%s</a>' % _('Skip to preview'))
-        self.request.write(' ')
-        self.request.write(_('[current page size \'\'\'%(size)d\'\'\' bytes]') % {'size': self.size()})
-        self.request.write('</p>')
-        
-        # send form
-        self.request.write('<form id="editor" method="post" action="%s/%s#preview">' % (
-            self.request.getScriptname(),
-            wikiutil.quoteWikinameURL(self.page_name),
-            ))
-
-        # yet another weird workaround for broken IE6 (it expands the text
-        # editor area to the right after you begin to type...). IE sucks...
-        # http://fplanque.net/2003/Articles/iecsstextarea/
-        self.request.write('<fieldset style="border:none;padding:0;">')
-        
-        self.request.write('<p>')
-        self.request.write(unicode(html.INPUT(type="hidden", name="action", value="savepage")))
-
-        # Save backto in a hidden input
-        backto = form.get('backto', [None])[0]
-        if backto:
-            self.request.write(unicode(html.INPUT(type="hidden", name="backto", value=backto)))
 
         # Make backup on previews - but not for new empty pages
         if preview and raw_body:
@@ -341,32 +276,90 @@ Have a look at the diff of %(difflink)s to see what has been changed.""") % {
         if not raw_body:
             raw_body = _('Describe %s here.') % (self.page_name,)
 
+        # send form
+        self.request.write('<form id="editor" method="post" action="%s/%s#preview" onSubmit="flgChange = false;">' % (
+            self.request.getScriptname(),
+            wikiutil.quoteWikinameURL(self.page_name),
+            ))
+
+        # yet another weird workaround for broken IE6 (it expands the text
+        # editor area to the right after you begin to type...). IE sucks...
+        # http://fplanque.net/2003/Articles/iecsstextarea/
+        self.request.write('<fieldset style="border:none;padding:0;">')
+        
+        self.request.write(unicode(html.INPUT(type="hidden", name="action", value="edit")))
+
         # Send revision of the page our edit is based on
         self.request.write('<input type="hidden" name="rev" value="%d">' % (rev,))
 
+        # Save backto in a hidden input
+        backto = form.get('backto', [None])[0]
+        if backto:
+            self.request.write(unicode(html.INPUT(type="hidden", name="backto", value=backto)))
+
+        # button bar
+        button_spellcheck = (SpellCheck and
+            '<input class="button" type="submit" name="button_spellcheck" value="%s" onClick="flgChange = false;">'
+                % _('Check Spelling')) or ''
+
+        save_button_text = _('Save Changes')
+        cancel_button_text = _('Cancel')
+        
+        if self.cfg.page_license_enabled:
+            self.request.write('<p><em>', _(
+"""By hitting '''%(save_button_text)s''' you put your changes under the %(license_link)s.
+If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.""") % {
+                'save_button_text': save_button_text,
+                'cancel_button_text': cancel_button_text,
+                     'license_link': wikiutil.getSysPage(self.request, self.cfg.page_license_page).link_to(self.request),
+            }, '</em></p>')
+
+        self.request.write('''
+<input class="button" type="submit" name="button_save" value="%s" onClick="flgChange = false;">
+<input class="button" type="submit" name="button_preview" value="%s" onClick="flgChange = false;">
+<input id="switch2gui" style="display: none;" class="button" type="submit"     
+    name="button_switch" value="%s">
+%s
+<input class="button" type="submit" name="button_cancel" value="%s">
+<input type="hidden" name="editor" value="text">
+''' % (save_button_text, _('Preview'), _('GUI Mode'), button_spellcheck, cancel_button_text,))
+
         # Add textarea with page text
 
         # TODO: currently self.language is None at this point. We have
         # to do processing instructions parsing earlier, or move page
         # language into meta file.
-        lang = self.language or self.request.cfg.default_lang
+        lang = self.language or self.request.cfg.language_default
+
+        # to prevent moving away from the page without saving it
+        self.request.write(u'''\
+<script type="text/javascript">
+    var flgChange = false;
+    function confirmleaving() {
+        if ( flgChange )
+            return "%s";
+    }
+</script>
+''' % _("Your changes are not saved!"))
 
         self.request.write(
-            u'<textarea id="editor-textarea" name="savetext" lang="%(lang)s" dir="%(dir)s"'
-            u' rows="%(rows)d">' % {
+            u'''\
+<textarea id="editor-textarea" name="savetext" lang="%(lang)s" dir="%(dir)s" rows="%(rows)d"
+          onChange="flgChange = true;" onKeyPress="flgChange = true;">\
+%(text)s\
+</textarea>''' %   {
             'lang': lang,
             'dir': i18n.getDirection(lang),
             'rows': text_rows,
-            }
-            )
-        self.request.write(wikiutil.escape(raw_body))
-        self.request.write(u'</textarea>')
+            'text': wikiutil.escape(raw_body)
+        })
 
-        self.request.write('</p>')
-
-        self.request.write("<p>", _("Optional comment about this change"),
-            '<br><input id="editor-comment" type="text" name="comment" value="%s" maxlength="80"></p>' % (
+        self.request.write("<p>")
+        self.request.write(_("Comment:"),
+            ' <input id="editor-comment" type="text" name="comment" value="%s" maxlength="80"'
+            ' onChange="flgChange = true;" onKeyPress="flgChange = true;">' % (
                 wikiutil.escape(kw.get('comment', ''), 1), ))
+        self.request.write("</p>")
 
         # Category selection
         filter = re.compile(self.cfg.page_category_regex, re.UNICODE).search
@@ -374,53 +367,29 @@ Have a look at the diff of %(difflink)s to see what has been changed.""") % {
         cat_pages.sort()
         cat_pages = [wikiutil.pagelinkmarkup(p) for p in cat_pages]
         cat_pages.insert(0, ('', _('<No addition>', formatted=False)))
-        self.request.write("<p>", _('Make this page belong to category %(category)s') % {
+        self.request.write("<p>")
+        self.request.write(_('Add to: %(category)s') % {
             'category': unicode(util.web.makeSelection('category', cat_pages)),
         })
 
-        # button bar
-        button_spellcheck = (SpellCheck and
-            '<input type="submit" name="button_spellcheck" value="%s">'
-                % _('Check Spelling')) or ''
-
-        save_button_text = _('Save Changes')
-        cancel_button_text = _('Cancel')
-        
-        self.request.write("</p>")
-            
-        if self.cfg.page_license_enabled:
-            self.request.write('<p><em>', _(
-"""By hitting '''%(save_button_text)s''' you put your changes under the %(license_link)s.
-If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.""") % {
-                'save_button_text': save_button_text,
-                'cancel_button_text': cancel_button_text,
-                'license_link': wikiutil.getSysPage(self.request, self.cfg.page_license_page).link_to(self.request),
-            }, '</em></p>')
-
-        self.request.write('''
-<p>
-<input type="submit" name="button_save" value="%s">
-<input type="submit" name="button_preview" value="%s"> %s
-<input type="submit" name="button_cancel" value="%s">
-</p>
-<p>
-''' % (save_button_text, _('Preview'), button_spellcheck, cancel_button_text,))
-
-        if self.cfg.mail_smarthost:
+        if self.cfg.mail_enabled:
             self.request.write('''
-<input type="checkbox" name="trivial" value="1" %(checked)s>
-<label>%(label)s</label> ''' % {
+&nbsp;
+<input type="checkbox" name="trivial" id="chktrivial" value="1" %(checked)s>
+<label for="chktrivial">%(label)s</label> ''' % {
                 'checked': ('', 'checked')[form.get('trivial',['0'])[0] == '1'],
                 'label': _("Trivial change"),
                 })
 
         self.request.write('''
-<input type="checkbox" name="rstrip" value="1" %(checked)s>
-<label>%(label)s</label>
-</p> ''' % {
+&nbsp;
+<input type="checkbox" name="rstrip" id="chkrstrip" value="1" %(checked)s>
+<label for="chkrstrip">%(label)s</label>
+''' % {
             'checked': ('', 'checked')[form.get('rstrip',['0'])[0] == '1'],
             'label': _('Remove trailing whitespace from each line')
             })
+        self.request.write("</p>")
 
         badwords_re = None
         if preview is not None:
@@ -433,22 +402,28 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
         self.request.write("</form>")
         
         # QuickHelp originally by Georg Mischler <schorsch@lightingwiki.com>
-        self.request.write('<hr>\n<dl>' + _(""" Emphasis:: [[Verbatim('')]]''italics''[[Verbatim('')]]; [[Verbatim(''')]]'''bold'''[[Verbatim(''')]]; [[Verbatim(''''')]]'''''bold italics'''''[[Verbatim(''''')]]; [[Verbatim('')]]''mixed ''[[Verbatim(''')]]'''''bold'''[[Verbatim(''')]] and italics''[[Verbatim('')]]; [[Verbatim(----)]] horizontal rule.
+        self.request.write(self.request.formatter.div(1, id="editor-help"))
+        self.request.write(_(""" Emphasis:: [[Verbatim('')]]''italics''[[Verbatim('')]]; [[Verbatim(''')]]'''bold'''[[Verbatim(''')]]; [[Verbatim(''''')]]'''''bold italics'''''[[Verbatim(''''')]]; [[Verbatim('')]]''mixed ''[[Verbatim(''')]]'''''bold'''[[Verbatim(''')]] and italics''[[Verbatim('')]]; [[Verbatim(----)]] horizontal rule.
  Headings:: [[Verbatim(=)]] Title 1 [[Verbatim(=)]]; [[Verbatim(==)]] Title 2 [[Verbatim(==)]]; [[Verbatim(===)]] Title 3 [[Verbatim(===)]];   [[Verbatim(====)]] Title 4 [[Verbatim(====)]]; [[Verbatim(=====)]] Title 5 [[Verbatim(=====)]].
  Lists:: space and one of: * bullets; 1., a., A., i., I. numbered items; 1.#n start numbering at n; space alone indents.
  Links:: [[Verbatim(JoinCapitalizedWords)]]; [[Verbatim(["brackets and double quotes"])]]; url; [url]; [url label].
- Tables:: || cell text |||| cell text spanning 2 columns ||;    no trailing white space allowed after tables or titles.""")
- + '</dl>')
+ Tables:: || cell text |||| cell text spanning 2 columns ||;    no trailing white space allowed after tables or titles.
+
+(!) For more help, see HelpOnEditing or SyntaxReference.
+"""))
+        self.request.write(self.request.formatter.div(0))
 
         if preview is not None:
-            self.send_page(self.request, content_id='preview', content_only=1,
+            if staytop:
+                content_id = 'previewbelow'
+            else:
+                content_id = 'preview'
+            self.send_page(self.request, content_id=content_id, content_only=1,
                            hilite_re=badwords_re)
 
-        self.request.write(self.request.formatter.endContent()) # end content div
-        self.request.theme.emit_custom_html(self.cfg.page_footer1)
-        self.request.theme.emit_custom_html(self.cfg.page_footer2)
-
-
+        self.request.write(self.request.formatter.endContent())
+        wikiutil.send_footer(self.request, self.page_name)
+        
     def sendCancel(self, newtext, rev):
         """
         User clicked on Cancel button. If edit locking is active,
@@ -471,20 +446,27 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
         and keeping the backups, logs and attachments).
         
         @param comment: Comment given by user
+        @rtype: unicode
+        @return: error message
         """
-        # First save a final backup copy of the current page
-        # (recreating the page allows access to the backups again)
+        _ = self._
+        
         try:
-            self.saveText(u"deleted\n", 0, comment=comment or u'')
-        except self.SaveError, msg:
+            # First save a final backup copy of the current page
+            # (recreating the page allows access to the backups again)
+            msg = self.saveText(u"deleted\n", 0, comment=comment or u'')
+            msg = msg.replace(
+                _("Thank you for your changes. Your attention to detail is appreciated."),
+                _('Page "%s" was successfully deleted!') % (self.page_name,))
+            # Then really delete it
+            try:
+                os.remove(self._text_filename())
+            except OSError, err:
+                if err.errno != errno.ENOENT:
+                    raise err
+        except self.SaveError, message:
             # XXX Error handling
-            pass
-        # Then really delete it
-        try:
-            os.remove(self._text_filename())
-        except OSError, er:
-            import errno
-            if er.errno != errno.ENOENT: raise er
+            msg = "SaveError has occured in PageEditor.deletePage. We need locking there."
         
         # reset page object
         self.reset()
@@ -507,6 +489,7 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
             key = formatter_name
             cache = caching.CacheEntry(self.request, arena, key)
             cache.remove()
+        return msg
 
     def _sendNotification(self, comment, emails, email_lang, revisions, trivial):
         """
@@ -568,8 +551,7 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
         @return: message, indicating success or errors.
         """
         _ = self._
-        subscribers = self.getSubscribers(self.request, return_users=1,
-                                          trivial=trivial)
+        subscribers = self.getSubscribers(self.request, return_users=1, trivial=trivial)
         if subscribers:
             # get a list of old revisions, and append a diff
             revisions = self.getRevList()
@@ -591,25 +573,6 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
         # No mail sent, no message.
         return ''
 
-
-    def _user_variable(self):
-        """
-        If user has a profile return the user name from the profile
-        else return the remote address or "<unknown>"
-
-        If the user name contains spaces it is wiki quoted to allow
-        links to the wiki user homepage (if one exists).
-        
-        @rtype: string
-        @return: wiki freelink to user's homepage or remote address
-        """
-        username = self.request.user.name
-        if username and self.cfg.allow_extended_names and \
-                username.count(' ') and Page(self.request, username).exists():
-            username = '["%s"]' % username
-        return user.getUserIdentification(self.request, username)
-
-
     def _expand_variables(self, text):
         """
         Expand @VARIABLE@ in `text`and return the expanded text.
@@ -618,35 +581,35 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
         @rtype: string
         @return: new text of wikipage, variables replaced
         """
-        #!!! TODO: Allow addition of variables via wikiconfig (and/or a text file)
+        # TODO: Allow addition of variables via wikiconfig or a global
+        # wiki dict.
         now = time.strftime("%Y-%m-%dT%H:%M:%SZ", util.datetime.tmtuple())
-        system_vars = {
-            'PAGE': lambda s=self: s.page_name,
-            'TIME': lambda t=now: "[[DateTime(%s)]]" % t,
-            'DATE': lambda t=now: "[[Date(%s)]]" % t,
-            'USERNAME': lambda s=self: s._user_variable(),
-            'USER': lambda s=self: "-- %s" % (s._user_variable(),),
-            'SIG': lambda s=self, t=now: "-- %s [[DateTime(%s)]]"
-                % (s._user_variable(), t,),
+        user = self.request.user
+        signature = user.signature()
+        variables = {
+            'PAGE': self.page_name,
+            'TIME': "[[DateTime(%s)]]" % now,
+            'DATE': "[[Date(%s)]]" % now,
+            'ME': user.name,
+            'USERNAME': signature,
+            'USER': "-- %s" % signature,
+            'SIG': "-- %s [[DateTime(%s)]]" % (signature, now),
         }
-
-        if self.request.user.valid and self.request.user.name:
-            if self.request.user.email:
-                system_vars['MAILTO'] = lambda u=self.request.user: \
-                    "[mailto:%s %s]" % (u.email, u.name)
-            # users can define their own vars via UserHomepage/MyDict
-            uservarspagename = self.request.user.name + "/MyDict"
-            dicts = self.request.dicts
-            if dicts.has_dict(uservarspagename):
-                userdict = dicts.dict(uservarspagename)
-                for key in userdict.keys():
-                    text = text.replace('@%s@' % key, userdict[key])
-                    
+  
+        if user.valid and user.name:
+            if user.email:
+                variables['MAILTO'] = "[[MailTo(%s)]]" % user.email
+            # Users can define their own variables via
+            # UserHomepage/MyDict, which override the default variables.
+            userDictPage = user.name + "/MyDict"
+            if self.request.dicts.has_dict(userDictPage):
+                variables.update(self.request.dicts.dict(userDictPage))
+                      
         # TODO: Use a more stream-lined re.sub algorithm
-        for name, val in system_vars.items():
-            text = text.replace('@%s@' % name, val())
+        for name in variables:
+            text = text.replace('@%s@' % name, variables[name])
         return text
-
+  
     def normalizeText(self, text, **kw):
         """ Normalize text
 
@@ -689,25 +652,20 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
         @return: url of page backup or None
         """
         # Backup only if set to backup and user has a home page.
-        homepage = wikiutil.getHomePage(self.request)
-        if not homepage or not self.do_editor_backup:
+        homepage = wikiutil.getInterwikiHomePage(self.request)
+        if homepage is None or not self.do_editor_backup:
             return None
-        
+        if homepage[0] != 'Self': # user has a remote homepage
+            return None
+
         _ = self._
-        if config.allow_subpages:
-            delimiter = "/"
-        else:
-            delimiter = ""
         backuppage = PageEditor(self.request,
-                                homepage.page_name + delimiter + "MoinEditorBackup",
+                                homepage[1] + "/MoinEditorBackup",
                                 do_revision_backup=0)
-        if self.cfg.acl_enabled:
-            # We need All: at the end to prevent that the original page ACLs
-            # make it possible to see preview saves (that maybe were never
-            # really saved by the author).
-            intro = u"#acl %s:read,write,delete All:\n" % self.request.user.name
-        else:
-            intro = ""
+        # We need All: at the end to prevent that the original page ACLs
+        # make it possible to see preview saves (that maybe were never
+        # really saved by the author).
+        intro = u"#acl %s:read,write,delete All:\n" % self.request.user.name
         pagename = self.page_name
         date = self.request.user.getFormattedDateTime(time.time())
         intro += _('## backup of page "%(pagename)s" submitted %(date)s') % {
@@ -792,61 +750,64 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
             f.close()
             os.chmod(cfn, 0666 & config.umask)
             
-        got_lock= False
+        got_lock = False
         retry = 0
-        while not got_lock and retry < 100:
-            retry += 1
-            try:
-                filesys.rename(cfn, clfn)
-                got_lock = True
-            except OSError, err:
-                got_lock = False
-                if err.errno == 2: # there was no 'current' file
-                    time.sleep(0.1)
-                else:
-                    raise self.CouldNotLock, _("Page could not get locked. Unexpected error (errno=%d).") % err.errno
-        
-        if not got_lock:
-            raise self.CouldNotLock, _("Page could not get locked. Missing 'current' file?")
-        
-        # increment rev number of current(-locked) page
-        f = open(clfn)
-        revstr = f.read()
-        f.close()
-        rev = int(revstr)
-        if not was_deprecated:
-            if self.do_revision_backup or rev == 0:
-                rev += 1
-        revstr = '%08d' % rev
-        f = open(clfn, 'w')
-        f.write(revstr+'\n')
-        f.close()
-        
-        # save to page file
-        pagefile = os.path.join(revdir, revstr)
-        f = codecs.open(pagefile, 'wb', config.charset)
-        # Write the file using text/* mime type
-        f.write(self.encodeTextMimeType(text))
-        f.close()
-        os.chmod(pagefile, 0666 & config.umask)
-        mtime_usecs = wikiutil.timestamp2version(os.path.getmtime(pagefile))
-        # set in-memory content
-        self.set_raw_body(text)
-        
-        # reset page object
-        self.reset()
-        
-        # write the editlog entry
-        # for now simply make 2 logs, better would be some multilog stuff maybe
-        if self.do_revision_backup:
-            # do not globally log edits with no revision backup (like /MoinEditorBackup pages)
-            # if somebody edits a deprecated page, log it in global log, but not local log
-            glog.add(self.request, mtime_usecs, rev, action, self.page_name, None, extra, comment)
-        if not was_deprecated and self.do_revision_backup:
-            # if we did not create a new revision number, do not locally log it
-            llog.add(self.request, mtime_usecs, rev, action, self.page_name, None, extra, comment)
 
-        filesys.rename(clfn, cfn)
+        try:
+                while not got_lock and retry < 100:
+                    retry += 1
+                    try:
+                        filesys.rename(cfn, clfn)
+                        got_lock = True
+                    except OSError, err:
+                        got_lock = False
+                        if err.errno == 2: # there was no 'current' file
+                            time.sleep(0.1)
+                        else:
+                            raise self.CouldNotLock, _("Page could not get locked. Unexpected error (errno=%d).") % err.errno
+                
+                if not got_lock:
+                    raise self.CouldNotLock, _("Page could not get locked. Missing 'current' file?")
+                
+                # increment rev number of current(-locked) page
+                f = open(clfn)
+                revstr = f.read()
+                f.close()
+                rev = int(revstr)
+                if not was_deprecated:
+                    if self.do_revision_backup or rev == 0:
+                        rev += 1
+                revstr = '%08d' % rev
+                f = open(clfn, 'w')
+                f.write(revstr+'\n')
+                f.close()
+                
+                # save to page file
+                pagefile = os.path.join(revdir, revstr)
+                f = codecs.open(pagefile, 'wb', config.charset)
+                # Write the file using text/* mime type
+                f.write(self.encodeTextMimeType(text))
+                f.close()
+                os.chmod(pagefile, 0666 & config.umask)
+                mtime_usecs = wikiutil.timestamp2version(os.path.getmtime(pagefile))
+                # set in-memory content
+                self.set_raw_body(text)
+                
+                # reset page object
+                self.reset()
+                
+                # write the editlog entry
+                # for now simply make 2 logs, better would be some multilog stuff maybe
+                if self.do_revision_backup:
+                    # do not globally log edits with no revision backup (like /MoinEditorBackup pages)
+                    # if somebody edits a deprecated page, log it in global log, but not local log
+                    glog.add(self.request, mtime_usecs, rev, action, self.page_name, None, extra, comment)
+                if not was_deprecated and self.do_revision_backup:
+                    # if we did not create a new revision number, do not locally log it
+                    llog.add(self.request, mtime_usecs, rev, action, self.page_name, None, extra, comment)
+        finally:
+            if got_lock:
+                filesys.rename(clfn, cfn)
 
         # add event log entry
         elog = eventlog.EventLog(self.request)
@@ -854,7 +815,6 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
 
         return mtime_usecs, rev
 
-
     def saveText(self, newtext, rev, **kw):
         """ Save new text for a page.
 
@@ -875,8 +835,7 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
         #!!! rev check is not enough since internal operations use "0"
 
         # expand variables, unless it's a template or form page
-        if not (wikiutil.isTemplatePage(self.request, self.page_name) or
-                wikiutil.isFormPage(self.request, self.page_name)):
+        if not wikiutil.isTemplatePage(self.request, self.page_name): 
             newtext = self._expand_variables(newtext)
 
         msg = ""
@@ -890,6 +849,22 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
             msg = _('You cannot save empty pages.')
             raise self.EmptyPage, msg
         elif rev != 0 and rev != self.current_rev():
+            # check if we already saved that page
+            other = False
+            pagelog = self.getPagePath('edit-log', use_underlay=0, isfile=1)
+            for line in editlog.EditLog(self.request, pagelog).reverse():
+                if int(line.rev)==int(rev):
+                    break
+                if not line.is_from_current_user(self.request):
+                    other = True
+                next_line = line
+            if next_line.is_from_current_user(self.request):
+                saved_page = Page(self.request, self.page_name,
+                                  rev=int(next_line.rev))
+                if newtext==saved_page.get_raw_body():
+                    msg = _("You already saved this page!")
+                    return msg
+                
             msg = _("""Sorry, someone else saved the page while you edited it.
 
 Please do the following: Use the back button of your browser, and cut&paste
@@ -908,7 +883,7 @@ delete the changes of the other person, which is excessively rude!''
         elif newtext == self.get_raw_body():
             msg = _('You did not change the page content, not saved!')
             raise self.Unchanged, msg
-        elif self.cfg.acl_enabled:
+        else:
             from wikiacl import parseACL
             # Get current ACL and compare to new ACL from newtext. If
             # they are not the sames, the user must have admin
@@ -938,9 +913,17 @@ delete the changes of the other person, which is excessively rude!''
             self.clean_acl_cache()
   
             # send notification mails
-            if self.request.cfg.mail_smarthost:
+            if self.request.cfg.mail_enabled:
                 msg = msg + self._notifySubscribers(comment, trivial)
           
+            if self.request.cfg.lupy_search:
+                from MoinMoin import lupy
+                index = lupy.Index(self.request)
+                # When we have automatic index building, we can add to
+                # the queue even if the index is missing.
+                if index.exists():
+                    index.update_page(self)
+
         # remove lock (forcibly if we were allowed to break it by the UI)
         # !!! this is a little fishy, since the lock owner might not notice
         # we broke his lock ==> but revision checking during preview will
@@ -983,13 +966,13 @@ class PageLock:
                         pass
 
 
-    def aquire(self):
+    def acquire(self):
         """
         Begin an edit lock depending on the mode chosen in the config.
 
         @rtype: tuple
         @return: tuple is returned containing 2 values:
-              * a bool indicating successful aquiry
+              * a bool indicating successful acquiry
               * a string giving a reason for failure or an informational msg
         """
         if not self.locktype:
@@ -1039,15 +1022,13 @@ class PageLock:
                     ) % {'owner': owner, 'timestamp': timestamp, 'mins_valid': mins_valid}
             else:
                 # warn user about existing lock
-### WIKIMARKUP-TODO
+
                 result = 1, _(
-                    'This page was opened for editing or last previewed at %(timestamp)s by %(owner)s.[[BR]]\n'
-                    '\'\'\''
-                    'You should \'\'refrain from editing\'\' this page for at least another %(mins_valid)d minute(s),\n'
-                    'to avoid editing conflicts.'
-                    '\'\'\'[[BR]]\n'
-                    'To leave the editor, press the Cancel button.'
-                    ) % {'timestamp': timestamp, 'owner': owner, 'mins_valid': mins_valid}
+"""This page was opened for editing or last previewed at %(timestamp)s by %(owner)s.[[BR]]
+'''You should ''refrain from editing'' this page for at least another %(mins_valid)d minute(s),
+to avoid editing conflicts.'''[[BR]]
+To leave the editor, press the Cancel button.""") % {
+                    'timestamp': timestamp, 'owner': owner, 'mins_valid': mins_valid}
 
         return result
 
