@@ -16,17 +16,20 @@
        to view the content of the file
 
     To insert an attachment into the page, use the "attachment:" pseudo
-    schema.  
+    schema.
 
     @copyright: 2001 by Ken Sugino (sugino@mediaone.net)
     @copyright: 2001-2004 by Jrgen Hermann <jh@web.de>
+    @copyright: 2005 R. Bauer
+    @copyright: 2005 MoinMoin:AlexanderSchremmer
+    @copyright: 2005 DiegoOngaro at ETSZONE (diego@etszone.com)
     @license: GNU GPL, see COPYING for details.
 """
 
-import os, mimetypes, time, urllib
-from MoinMoin import config, user, util, wikiutil
+import os, mimetypes, time, zipfile
+from MoinMoin import config, user, util, wikiutil, packages
 from MoinMoin.Page import Page
-from MoinMoin.util import MoinMoinNoFooter, filesys, web
+from MoinMoin.util import MoinMoinNoFooter, filesys
 
 action_name = __name__.split('.')[-1]
 
@@ -62,6 +65,18 @@ def getAttachDir(request, pagename, create=0):
 
     return attach_dir
 
+def absoluteName(url, pagename):
+    """ Get (pagename, filename) of an attachment: link
+        @param url: PageName/filename.ext or filename.ext (unicode)
+        @param pagename: name of the currently processed page (unicode)
+        @rtype: tuple of unicode
+        @return: PageName, filename.ext
+    """
+    pieces = url.split(u'/')
+    if len(pieces) == 1:
+        return pagename, pieces[0]
+    else:
+        return u"/".join(pieces[:-1]), pieces[-1]
 
 def getAttachUrl(pagename, filename, request, addts=0, escaped=0):
     """ Get URL that points to attachment `filename` of page `pagename`.
@@ -81,12 +96,12 @@ def getAttachUrl(pagename, filename, request, addts=0, escaped=0):
 
         url = "%s/%s/attachments/%s%s" % (
             request.cfg.attachments['url'], wikiutil.quoteWikinameFS(pagename),
-            urllib.quote(filename.encode(config.charset)), timestamp)
+            wikiutil.url_quote(filename), timestamp)
     else:
         # send file via CGI
         url = "%s/%s?action=%s&do=get&target=%s" % (
             request.getScriptname(), wikiutil.quoteWikinameURL(pagename),
-            action_name, urllib.quote_plus(filename.encode(config.charset)))
+            action_name, wikiutil.url_quote_plus(filename))
     if escaped:
         url = wikiutil.escape(url)
     return url
@@ -111,8 +126,15 @@ def getIndicator(request, pagename):
     return attach_link
 
 
-def getFilename(request, pagename, name):
-    return os.path.join(getAttachDir(request, pagename), name).encode(config.charset)
+def getFilename(request, pagename, filename):
+    """ make complete pathfilename of file "name" attached to some page "pagename"
+        @param request: request object
+        @param pagename: name of page where the file is attached to (unicode)
+        @param filename: filename of attached file (unicode)
+        @rtype: string (in config.charset encoding)
+        @return: complete path/filename of attached file
+    """
+    return os.path.join(getAttachDir(request, pagename), filename).encode(config.charset)
 
 
 def info(pagename, request):
@@ -125,7 +147,7 @@ def info(pagename, request):
     if os.path.isdir(attach_dir):
         files = os.listdir(attach_dir)
     page = Page(request, pagename)
-    # TODO: remove escape=0 in 1.4
+    # TODO: remove escape=0 in 2.0
     link = page.url(request, {'action': 'AttachFile'}, escape=0)
     attach_info = _('There are <a href="%(link)s">%(count)s attachment(s)</a> stored for this page.', formatted=False) % {
         'count': len(files),
@@ -145,8 +167,7 @@ def _addLogEntry(request, action, pagename, filename):
     """
     from MoinMoin.logfile import editlog
     t = wikiutil.timestamp2version(time.time())
-    # urllib always return ascii
-    fname = unicode(urllib.quote(filename.encode(config.charset)))
+    fname = wikiutil.url_quote(filename, want_unicode=True)
 
     # TODO: for now we simply write 2 logs, maybe better use some multilog stuff
     # Write to global log
@@ -203,6 +224,8 @@ def _build_filelist(request, pagename, showheader, readonly):
         label_get = _("get")
         label_edit = _("edit")
         label_view = _("view")
+        label_unzip = _("unzip")
+        label_install = _("install")
 
         for file in files:
             fsize = float(os.stat(os.path.join(attach_dir,file).encode(config.charset))[6]) # in byte
@@ -210,7 +233,7 @@ def _build_filelist(request, pagename, showheader, readonly):
             baseurl = request.getScriptname()
             action = action_name
             urlpagename = wikiutil.quoteWikinameURL(pagename)
-            urlfile = urllib.quote_plus(file.encode(config.charset))
+            urlfile = wikiutil.url_quote_plus(file)
 
             base, ext = os.path.splitext(file)
             get_url = getAttachUrl(pagename, file, request, escaped=1)
@@ -218,8 +241,11 @@ def _build_filelist(request, pagename, showheader, readonly):
                         'urlfile': urlfile, 'label_del': label_del,
                         'base': base, 'label_edit': label_edit,
                         'label_view': label_view,
+                        'label_unzip': label_unzip,
+                        'label_install': label_install,
                         'get_url': get_url, 'label_get': label_get,
-                        'file': wikiutil.escape(file), 'fsize': fsize,
+                        'file': wikiutil.escape(file).replace(' ', '%20'),
+                        'fsize': fsize,
                         'pagename': pagename}
 
             del_link = ''
@@ -232,6 +258,15 @@ def _build_filelist(request, pagename, showheader, readonly):
             else:
                 viewlink = '<a href="%(baseurl)s/%(urlpagename)s?action=%(action)s&amp;do=view&amp;target=%(urlfile)s">%(label_view)s</a>' % parmdict
 
+            if (packages.ZipPackage(request, os.path.join(attach_dir, file).encode(config.charset)).isPackage() and
+                request.user.name in request.cfg.superuser):
+                viewlink += ' | <a href="%(baseurl)s/%(urlpagename)s?action=%(action)s&amp;do=install&amp;target=%(urlfile)s">%(label_install)s</a>' % parmdict
+            elif (zipfile.is_zipfile(os.path.join(attach_dir,file).encode(config.charset)) and
+                request.user.may.read(pagename) and request.user.may.delete(pagename)
+                and request.user.may.write(pagename)):
+                viewlink += ' | <a href="%(baseurl)s/%(urlpagename)s?action=%(action)s&amp;do=unzip&amp;target=%(urlfile)s">%(label_unzip)s</a>' % parmdict
+
+
             parmdict['viewlink'] = viewlink
             parmdict['del_link'] = del_link
             str = str + ('<li>[%(del_link)s'
@@ -257,6 +292,26 @@ def _get_files(request, pagename):
 def _get_filelist(request, pagename):
     return _build_filelist(request, pagename, 1, 0)
 
+def _subdir_exception(zf):
+    """
+    Checks for the existance of one common subdirectory shared among
+    all files in the zip file. If this is the case, returns a dict of
+    original names to modified names so that such files can be unpacked
+    as the user would expect.
+    """
+
+    b = zf.namelist()
+    if not '/' in b[0]:
+        return False #No directory
+    slashoffset = b[0].index('/')
+    directory = b[0][:slashoffset]
+    for origname in b:
+        if origname.rfind('/') != slashoffset or origname[:slashoffset] != directory:
+            return False #Multiple directories or different directory
+    names = {}
+    for origname in b:
+        names[origname] = origname[slashoffset+1:]
+    return names #Returns dict of {origname: safename}
 
 def error_msg(pagename, request, msg):
     Page(request, pagename).send_page(request, msg=msg)
@@ -275,7 +330,7 @@ def send_link_rel(request, pagename):
         for file in files:
             url = "%s/%s?action=%s&do=view&target=%s" % (
                 scriptName, pagename_quoted,
-                action_name, urllib.quote_plus(file.encode(config.charset)))
+                action_name, wikiutil.url_quote_plus(file))
 
             request.write(u'<link rel="Appendix" title="%s" href="%s">\n' % (
                 wikiutil.escape(file), wikiutil.escape(url)))
@@ -290,7 +345,7 @@ def send_hotdraw(pagename, request):
     drawpath = getAttachUrl(pagename, basename + '.draw', request, escaped=1)
     pngpath = getAttachUrl(pagename, basename + '.png', request, escaped=1)
     querystr = {'action': 'AttachFile', 'ts': now}
-    querystr = wikiutil.escape(web.makeQueryString(querystr))
+    querystr = wikiutil.escape(wikiutil.makeQueryString(querystr))
     pagelink = '%s/%s?%s' % (request.getScriptname(), wikiutil.quoteWikinameURL(pagename), querystr)
     helplink = Page(request, "HelpOnActions/AttachFile").url(request)
     savelink = Page(request, pagename).url(request) # XXX include target filename param here for twisted
@@ -388,7 +443,7 @@ def execute(pagename, request):
     _ = request.getText
 
     msg = None
-    if action_name in request.cfg.excluded_actions:
+    if action_name in request.cfg.actions_excluded:
         msg = _('File attachments are not allowed in this wiki!')
     elif request.form.has_key('filepath'):
         if request.user.may.write(pagename):
@@ -419,6 +474,16 @@ def execute(pagename, request):
             get_file(pagename, request)
         else:
             msg = _('You are not allowed to get attachments from this page.')
+    elif request.form['do'][0] == 'unzip':
+         if request.user.may.delete(pagename) and request.user.may.read(pagename) and request.user.may.write(pagename):
+            unzip_file(pagename, request)
+         else:
+            msg = _('You are not allowed to unzip attachments of this page.')
+    elif request.form['do'][0] == 'install':
+         if request.user.name in request.cfg.superuser:
+            install_package(pagename, request)
+         else:
+            msg = _('You are not allowed to install files.')
     elif request.form['do'][0] == 'view':
         if request.user.may.read(pagename):
             view_file(pagename, request)
@@ -441,7 +506,7 @@ def upload_form(pagename, request, msg=''):
     request.write('<div id="content">\n') # start content div
     send_uploadform(pagename, request)
     request.write('</div>\n') # end content div
-    wikiutil.send_footer(request, pagename, showpage=1)
+    wikiutil.send_footer(request, pagename)
 
 
 def do_upload(pagename, request):
@@ -587,6 +652,112 @@ def get_file(pagename, request):
 
     raise MoinMoinNoFooter
 
+def install_package(pagename, request):
+    _ = request.getText
+
+    target, targetpath = _access_file(pagename, request)
+    if not target:
+        return
+
+    package = packages.ZipPackage(request, targetpath)
+
+    if package.isPackage():
+        if package.installPackage():
+            msg=_("Attachment '%(filename)s' installed.") % {'filename': wikiutil.escape(target)}
+        else:
+            msg=_("Installation of '%(filename)s' failed.") % {'filename': wikiutil.escape(target)}
+        if package.msg != "":
+            msg += "<br><pre>" + wikiutil.escape(package.msg) + "</pre>"
+    else:
+        msg = _('The file %s is not a MoinMoin package file.' % wikiutil.escape(target))
+
+    upload_form(pagename, request, msg=msg)
+
+def unzip_file(pagename, request):
+    _ = request.getText
+    valid_pathname = lambda name: (name.find('/') == -1) and (name.find('\\') == -1)
+
+    filename, fpath = _access_file(pagename, request)
+    if not filename:
+        return # error msg already sent in _access_file
+
+    attachment_path = getAttachDir(request, pagename)
+    single_file_size = request.cfg.unzip_single_file_size
+    attachments_file_space = request.cfg.unzip_attachments_space
+    attachments_file_count = request.cfg.unzip_attachments_count
+
+    files = _get_files(request, pagename)
+
+    msg = ""
+    if files:
+        fsize = 0.0
+        fcount = 0
+        for file in files:
+            fsize += float(os.stat(getFilename(request, pagename, file))[6]) # in byte
+            fcount += 1
+
+        available_attachments_file_space = attachments_file_space - fsize
+        available_attachments_file_count = attachments_file_count - fcount
+        
+        if zipfile.is_zipfile(fpath):
+            zf = zipfile.ZipFile(fpath)
+            sum_size_over_all_valid_files = 0.0
+            count_valid_files = 0
+            namelist = _subdir_exception(zf)
+            if not namelist: #if it's not handled by _subdir_exception()
+                #Convert normal zf.namelist() to {origname:finalname} dict
+                namelist = {}
+                for name in zf.namelist():
+                    namelist[name] = name
+            for (origname, finalname) in namelist.iteritems():
+                if valid_pathname(finalname):
+                    sum_size_over_all_valid_files += zf.getinfo(origname).file_size
+                    count_valid_files += 1
+
+            if sum_size_over_all_valid_files > available_attachments_file_space:
+                msg=_("Attachment '%(filename)s' could not be unzipped because"
+                      " the resulting files would be too large (%(space)d kB"
+                      " missing).") % {
+                        'filename': filename,
+                        'space': (sum_size_over_all_valid_files -
+                              available_attachments_file_space) / 1000 }
+            elif count_valid_files > available_attachments_file_count:
+                msg=_("Attachment '%(filename)s' could not be unzipped because"
+                      " the resulting files would be too many (%(count)d "
+                      "missing).") % {
+                        'filename': filename,
+                        'count': (count_valid_files -
+                                  available_attachments_file_count) }
+            else:
+                valid_name = False
+                for (origname, finalname) in namelist.iteritems():
+                    if valid_pathname(finalname):
+                        zi = zf.getinfo(origname)
+                        if zi.file_size < single_file_size:
+                            new_file = getFilename(request, pagename, finalname)
+                            if not os.path.exists(new_file):
+                                outfile = open(new_file, 'wb')
+                                outfile.write(zf.read(origname))
+                                outfile.close()
+                                # it's not allowed to zip a zip file so it is dropped
+                                if zipfile.is_zipfile(new_file):
+                                    os.unlink(new_file)
+                                else:
+                                    valid_name = True
+                                    os.chmod(new_file, 0666 & config.umask)
+                                    _addLogEntry(request, 'ATTNEW', pagename, finalname)
+
+                if valid_name:
+                    msg=_("Attachment '%(filename)s' unzipped.") % {'filename': filename}
+                else:
+                    msg=_("Attachment '%(filename)s' not unzipped because the "
+                          "files are too big, .zip files only, exist already or "
+                          "reside in folders.") % {'filename': filename}
+        else:
+            msg = _('The file %(target)s is not a .zip file.' % target)
+
+    upload_form(pagename, request, msg=wikiutil.escape(msg))
+
 def send_viewfile(pagename, request):
     _ = request.getText
 
@@ -614,6 +785,21 @@ def send_viewfile(pagename, request):
             request.write("</pre>")
             return
 
+    package = packages.ZipPackage(request, fpath)
+    if package.isPackage():
+        request.write("<pre><b>%s</b>\n%s</pre>" % (_("Package script:"),wikiutil.escape(package.getScript())))
+        return
+
+    import zipfile
+    if zipfile.is_zipfile(fpath):
+        zf = zipfile.ZipFile(fpath, mode='r')
+        request.write("<pre>%-46s %19s %12s\n" % (_("File Name"), _("Modified")+" "*5, _("Size")))
+        for zinfo in zf.filelist:
+            date = "%d-%02d-%02d %02d:%02d:%02d" % zinfo.date_time
+            request.write(wikiutil.escape("%-46s %s %12d\n" % (zinfo.filename, date, zinfo.file_size)))
+        request.write("</pre>")
+        return
+
     request.write('<p>' + _("Unknown file type, cannot display this attachment inline.") + '</p>')
     request.write('<a href="%s">%s</a>' % (
         getAttachUrl(pagename, filename, request, escaped=1), wikiutil.escape(filename)))
