@@ -16,12 +16,12 @@ from MoinMoin.PageEditor import PageEditor
 def execute(pagename, request):
     _ = request.getText
     actname = __name__.split('.')[-1]
-    # Create a page editor that does not do edior backups, becuase
-    # delete generate "deleted" version, of the page.
+    # Create a page editor that does not do editor backups, because
+    # delete generates a "deleted" version of the page.
     page = PageEditor(request, pagename, do_editor_backup=0)
 
     # be extra paranoid in dangerous actions
-    if actname in request.cfg.excluded_actions \
+    if actname in request.cfg.actions_excluded \
             or not request.user.may.write(pagename) \
             or not request.user.may.delete(pagename):
         return page.send_page(request,
@@ -41,10 +41,11 @@ def execute(pagename, request):
                 msg = _('Please use the interactive user interface to delete pages!'))
 
         # Delete the page
-        page.deletePage(request.form.get('comment', [u''])[0])
+        comment = request.form.get('comment', [u''])[0]
+        comment = wikiutil.clean_comment(comment)
+        msg = page.deletePage(comment)
 
-        return page.send_page(request,
-                msg = _('Page "%s" was successfully deleted!') % (pagename,))
+        return page.send_page(request, msg=msg)
 
     # send deletion form
     ticket = wikiutil.createTicket()
@@ -52,7 +53,7 @@ def execute(pagename, request):
     button = _('Delete')
     comment_label = _("Optional reason for the deletion")
 
-    # TODO: this form suck, redesign like RenamePage
+    # TODO: this form sucks, redesign like RenamePage
     formhtml = '''
 <form method="post" action="">
 <strong>%(querytext)s</strong>
