@@ -8,7 +8,12 @@
 
 import os
 from MoinMoin import config
+from MoinMoin.util import filesys
 
+locking = 1
+if locking:
+    from MoinMoin.util import lock
+    
 class CacheEntry:
     def __init__(self, request, arena, key):
         """ init a cache entry
@@ -17,20 +22,19 @@ class CacheEntry:
                           page local cache area
             @param key: under which key we access the cache content
         """
+        self.request = request
         if isinstance(arena, str):
-            cache_dir = request.cfg.cache_dir
-            self.arena_dir = os.path.join(cache_dir, arena)
-            if not os.path.isdir(cache_dir):
-                os.mkdir(cache_dir, 0777 & config.umask)
-                os.chmod(cache_dir, 0777 & config.umask)
-            if not os.path.isdir(self.arena_dir):
-                os.mkdir(self.arena_dir, 0777 & config.umask)
-                os.chmod(self.arena_dir, 0777 & config.umask)
+            self.arena_dir = os.path.join(request.cfg.cache_dir, arena)
+            filesys.makeDirs(self.arena_dir)
         else: # arena is in fact a page object
             cache_dir = None
             self.arena_dir = arena.getPagePath('cache', check_create=1)
         self.key = key
-
+        if locking:
+            self.lock_dir = os.path.join(self.arena_dir, '__lock__')
+            self.rlock = lock.ReadLock(self.lock_dir, 60.0)
+            self.wlock = lock.WriteLock(self.lock_dir, 60.0)
+        
     def _filename(self):
         return os.path.join(self.arena_dir, self.key)
 
@@ -44,7 +48,8 @@ class CacheEntry:
             return 0
 
     def needsUpdate(self, filename, attachdir=None):
-        if not self.exists(): return 1
+        if not self.exists():
+            return 1
 
         try:
             ctime = os.path.getmtime(self._filename())
@@ -66,22 +71,36 @@ class CacheEntry:
 
     def copyto(self, filename):
         import shutil
-        shutil.copyfile(filename, self._filename())
-
-        try:
-            os.chmod(self._filename(), 0666 & config.umask)
-        except OSError:
-            pass
+        if not locking or locking and self.wlock.acquire(1.0):
+            try:
+                shutil.copyfile(filename, self._filename())
+                try:
+                    os.chmod(self._filename(), 0666 & config.umask)
+                except OSError:
+                    pass
+            finally:
+                if locking:
+                    self.wlock.release()
+        else:
+            request.log("Can't acquire write lock in %s", self.lock_dir)
 
     def update(self, content, encode=False):
         if encode:
             content = content.encode(config.charset)
-        open(self._filename(), 'wb').write(content)
-
-        try:
-            os.chmod(self._filename(), 0666 & config.umask)
-        except OSError:
-            pass
+        if not locking or locking and self.wlock.acquire(1.0):
+            try:
+                f = open(self._filename(), 'wb')
+                f.write(content)
+                f.close()
+                try:
+                    os.chmod(self._filename(), 0666 & config.umask)
+                except OSError:
+                    pass
+            finally:
+                if locking:
+                    self.wlock.release()
+        else:
+            request.log("Can't acquire write lock in %s", self.lock_dir)
 
     def remove(self):
         try:
@@ -90,7 +109,16 @@ class CacheEntry:
             pass
 
     def content(self, decode=False):
-        data = open(self._filename(), 'rb').read()
+        if not locking or locking and self.rlock.acquire(1.0):
+            try:
+                f = open(self._filename(), 'rb')
+                data = f.read()
+                f.close()
+            finally:
+                if locking:
+                    self.rlock.release()
+        else:
+            request.log("Can't acquire read lock in %s", self.lock_dir)
         if decode:
             data = data.decode(config.charset)
         return data
