@@ -23,14 +23,14 @@ class Formatter(FormatterBase):
 
     hardspace = ' '
 
-    format_tags = ['b', 'em', 'highlight', 'sup', 'sub', 'code', 'u']
+    format_tags = ['b', 'em', 'highlight', 'sup', 'sub', 'strike', 'code', 'u']
 
     unbreakables = ['h1', 'h2', 'h3', 'h4', 'h5', 'h6',
                     'p', 'ol', 'ul', 'li', 'pre', 'a',
                     'table', 'td', 'tr']
 
-    need_p = [] #format_tags[:]
-    need_p.extend(['ol', 'a', 'pagelink', 'interwiki', 'macro']) #XXX add more
+    need_p = [] # format_tags[:]
+    need_p.extend(['ol', 'a', 'pagelink', 'interwiki', 'macro']) # XXX add more
 
     no_p_after = ['h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'p', 'ol', 'ul', 'pre',
                   'small', 'big', 'table', 'td', 'tr', 'dt',
@@ -38,25 +38,25 @@ class Formatter(FormatterBase):
                   'sysmesg']
 
     close_on_open = {
-        'h1' : ['p'],
-        'li' : ['li'],
-        'p' : ['p'],
-        #'pre' : ['p'],
+        'h1': ['p'],
+        'li': ['li'],
+        'p': ['p'],
+        #'pre': ['p'],
         }
 
-    for i in xrange(2,7):
+    for i in xrange(2, 7):
         close_on_open['h%i' % i] = close_on_open['h1']
 
     close_on_open = {} # XXX
 
     close_on_close = {
-        'table' : ['td', 'tr'],
-        'td' : ['tr'],
-        'tr' : ['td'],
-        'ol' : ['li'],
-        'ul' : ['li'],
+        'table': ['td', 'tr'],
+        'td': ['tr'],
+        'tr': ['td'],
+        'ol': ['li'],
+        'ul': ['li'],
         }
-    close_on_close = {} #XXX
+    close_on_close = {} # XXX
 
     def __init__(self, request, **kw):
         self.request = request
@@ -92,9 +92,10 @@ class Formatter(FormatterBase):
 
     def _close_tag(self, tag):
         """ low level function: closes tag right now
-            must be the last opend tag!!!
+            must be the last opened tag!!!
         """
-        if tag=='p': self.in_p = 0 #XXX
+        if tag == 'p':
+            self.in_p = 0 # XXX
         if self.tag_stack[-1][0] != tag:
             raise ValueError, "<%s> expected <%s> given" % (self.tag_stack[-1][0], tag)
         self.position = self.position.parentNode
@@ -163,12 +164,13 @@ class Formatter(FormatterBase):
         return ''
 
     def _check_p(self, opening_tag=None):
-        if ((opening_tag is not None) and
-            (opening_tag not in self.need_p)):
+        if (opening_tag is not None) and (opening_tag not in self.need_p):
             return
         for tag in self.tag_stack:
-            if tag[0] in self.no_p_after: return
-        if self.in_p: return
+            if tag[0] in self.no_p_after:
+                return
+        if self.in_p:
+            return
         self.in_p = 1
         self._open_tag('p', type=str(opening_tag))
 
@@ -181,7 +183,7 @@ class Formatter(FormatterBase):
         return self._set_tag('sysmesg', on, **kw)
 
     def startDocument(self, pagename):
-        return ""
+        return ''
 
     def endDocument(self):
         #return self.document.documentElement.toxml()
@@ -190,18 +192,6 @@ class Formatter(FormatterBase):
     def lang(self, on, lang_name):
         return self._set_tag('lang', on, value=lang_name)
 
-    def rawHTML(self, markup):
-        """ This allows emitting pre-formatted HTML markup, and should be
-            used wisely (i.e. very seldom).
-
-            Using this event while generating content results in unwanted
-            effects, like loss of markup or insertion of CDATA sections
-            when output goes to XML formats.
-        """
-        ## XXX
-        self.text(markup)
-        return ''
-
     def pagelink(self, on, pagename='', page=None, **kw):
         apply(FormatterBase.pagelink, (self, pagename, page), kw)
         if not pagename and page is not None:
@@ -218,9 +208,9 @@ class Formatter(FormatterBase):
         # call the macro
         return self._add_tag('macro', name=name, args=(args or ''))
 
-    def processor(self, processor_name, lines, is_parser = 0):
+    def processor(self, processor_name, lines, is_parser=0):
         """ processor_name MUST be valid!
-            writes out the result insted of returning it!
+            writes out the result instead of returning it!
         """
         node = self.document.createElement('processor')
         node.setAttribute('name', processor_name)
@@ -230,8 +220,8 @@ class Formatter(FormatterBase):
                 self.text('\n'.join(lines)) +
                 self._set_tag('processor', False))
 
-    def dynamic_content(self, parser, callback, arg_list = [], arg_dict = {},
-                        returns_content = 1):
+    def dynamic_content(self, parser, callback, arg_list=[], arg_dict={},
+                        returns_content=1):
         content = parser[callback](*arg_list, **arg_dict)
         if returns_content:
             return content
@@ -242,7 +232,7 @@ class Formatter(FormatterBase):
         kw['href'] = str(url)
         if css:
             kw['class'] = str(css)
-        return self._set_tag('a', on,  **kw)
+        return self._set_tag('a', on, **kw)
 
     def attachment_link(self, on, url='', **kw):
         kw['href'] = url
@@ -263,44 +253,50 @@ class Formatter(FormatterBase):
         kw['type'] = 'inline'
         return self._add_tag('attachment', **kw)
 
-    def rule(self, size=0):
+    def rule(self, size=0, **kw):
         return self._add_tag('hr', size=str(size))
 
     def icon(self, type):
         return self._add_tag('icon', type=type)
 
-    def strong(self, on):
+    def smiley(self, type):
+        return self._add_tag('smiley', type=type)
+
+    def strong(self, on, **kw):
         return self._set_tag('b', on)
 
-    def emphasis(self, on):
+    def emphasis(self, on, **kw):
         return self._set_tag('em', on)
 
-    def highlight(self, on):
+    def highlight(self, on, **kw):
         return self._set_tag('highlight', on)
 
-    def number_list(self, on, type=None, start=None):
+    def number_list(self, on, type=None, start=None, **kw):
         return self._set_tag('ol', on, type=type, start=start)
 
-    def bullet_list(self, on):
+    def bullet_list(self, on, **kw):
         return self._set_tag('ul', on)
 
     def listitem(self, on, **kw):
         return self._set_tag('li', on)
 
-    def sup(self, on):
+    def sup(self, on, **kw):
         return self._set_tag('sup', on)
 
-    def sub(self, on):
+    def sub(self, on, **kw):
         return self._set_tag('sub', on)
 
-    def code(self, on):
+    def strike(self, on, **kw):
+        return self._set_tag('strike', on)
+
+    def code(self, on, **kw):
         return self._set_tag('code', on)
 
-    def preformatted(self, on):
+    def preformatted(self, on, **kw):
         self.in_pre = on != 0
         return self._set_tag('pre', on)
 
-    def paragraph(self, on):
+    def paragraph(self, on, **kw):
         FormatterBase.paragraph(self, on)
         return self._set_tag('p', on)
 
@@ -319,31 +315,28 @@ class Formatter(FormatterBase):
             result[str(name)] = value
         return result
 
-    def table(self, on, attrs={}):
+    def table(self, on, attrs={}, **kw):
         return self._set_tag('table', on, **self._check_attrs(attrs))
         
-    def table_row(self, on, attrs={}):
+    def table_row(self, on, attrs={}, **kw):
         return self._set_tag('tr', on, **self._check_attrs(attrs))
 
-
-    def table_cell(self, on, attrs={}):
+    def table_cell(self, on, attrs={}, **kw):
         return self._set_tag('td', on, **self._check_attrs(attrs))
 
     def anchordef(self, name):
         return self._add_tag('anchor', name=name)
 
-    def anchorlink(self, on, name, id = None):
-        kw = {}
-        if id: kw['id'] = str(id)
+    def anchorlink(self, on, name, **kw):
         return self.url(on, "#" + name, **kw)
 
-    def underline(self, on):
+    def underline(self, on, **kw):
         return self._set_tag('u', on)
 
-    def definition_list(self, on):
+    def definition_list(self, on, **kw):
         return self._set_tag('dl', on)
 
-    def definition_term(self, on, compact=0):
+    def definition_term(self, on, compact=0, **kw):
         # XXX may be not correct
         # self._langAttr() missing
         if compact and on:
@@ -351,33 +344,37 @@ class Formatter(FormatterBase):
         else:
             return self._set_tag('dt', on)            
 
-    def definition_desc(self, on):
+    def definition_desc(self, on, **kw):
         # self._langAttr() missing
         return self._set_tag('dd', on)
 
-    def image(self, **kw):
+    def image(self, src=None, **kw):
         """ Take HTML <IMG> tag attributes in `attr`.
 
             Attribute names have to be lowercase!
         """
+        if src:
+            kw['src'] = src
         return self._add_tag('img', **kw)
 
-    def escapedText(self, text):
+    def escapedText(self, text, **kw):
         return wikiutil.escape(text)
 
-    def small(self, on):
+    def small(self, on, **kw):
         return self._set_tag('small', on)
 
-    def big(self, on):
+    def big(self, on, **kw):
         return self._set_tag('big', on)
 
     def code_area(self, on, code_id, code_type='code', show=0, start=-1, step=-1):
-        kw = {'id' : code_id,
-              'type' : code_type,
-              'show' : show,
-              }
-        if start != -1: kw['start'] = start
-        if step != -1: kw['step'] = step
+        kw = {'id': code_id,
+              'type': code_type,
+              'show': show,
+             }
+        if start != -1:
+            kw['start'] = start
+        if step != -1:
+            kw['step'] = step
         return self._set_tag('codearea', on, **kw)
 
     def code_line(self, on):
@@ -385,3 +382,5 @@ class Formatter(FormatterBase):
 
     def code_token(self, on, tok_type):
         return self._set_tag('codetoken', on, type=tok_type)
+
+
