@@ -8,20 +8,21 @@
     values. Other supporting modules start with an underscore.
 
     Public attributes:
-    languages -- languages that MoinMoin knows about
-    NAME, ENCODING, DIRECTION, MAINTAINER -- named indexes
+        languages -- languages that MoinMoin knows about
+        NAME, ENCODING, DIRECTION, MAINTAINER -- named indexes
 
     Public functions:
-    filename(lang) -- return the filename of lang
-    loadLanguage(request, lang) -- load text dictionary for a specific language, returns (formatted, unformatted)
-    requestLanguage(request, usecache=1) -- return the request language
-    wikiLanguages() -- return the available wiki user languages
-    browserLanguages() -- return the browser accepted languages
-    getDirection(lang) -- return the lang direction either 'ltr' or 'rtl'
-    getText(str, request) -- return str translation
-    canRecode(input, output, strict) -- check recode ability
+        filename(lang) -- return the filename of lang
+        loadLanguage(request, lang) -- load text dictionary for a specific language, returns (formatted, unformatted)
+        requestLanguage(request, usecache=1) -- return the request language
+        wikiLanguages() -- return the available wiki user languages
+        browserLanguages() -- return the browser accepted languages
+        getDirection(lang) -- return the lang direction either 'ltr' or 'rtl'
+        getText(str, request) -- return str translation
+        canRecode(input, output, strict) -- check recode ability
     
     @copyright: 2001-2004 by Jrgen Hermann <jh@web.de>
+    @copyright: 2005 by Thomas Waldmann
     @license: GNU GPL, see COPYING for details.
 """
 
@@ -32,16 +33,10 @@ try:
 except ImportError:
     import pickle
 
-from MoinMoin import config
-from MoinMoin.Page import Page
-
 # Set pickle protocol, see http://docs.python.org/lib/node64.html
-try:
-    # Requires 2.3
-    PICKLE_PROTOCOL = pickle.HIGHEST_PROTOCOL
-except AttributeError:
-    # Use protocol 1, binary format compatible with all python versions
-    PICKLE_PROTOCOL = 1
+PICKLE_PROTOCOL = pickle.HIGHEST_PROTOCOL
+
+from MoinMoin.Page import Page
 
 NAME, ENAME, ENCODING, DIRECTION, MAINTAINER = range(0,5)
 # we do not generate this on the fly due to performance reasons -
@@ -70,6 +65,7 @@ def filename(lang):
     filename = filename.lower()
     return filename
 
+
 def formatMarkup(request, text, currentStack = []):
     """
     Formats the text passed according to wiki markup.
@@ -83,7 +79,6 @@ def formatMarkup(request, text, currentStack = []):
         pass
     currentStack.append(text)
 
-    from MoinMoin.Page import Page
     from MoinMoin.parser.wiki import Parser
     from MoinMoin.formatter.text_html import Formatter
     import StringIO
@@ -91,7 +86,7 @@ def formatMarkup(request, text, currentStack = []):
     origtext = text
     out = StringIO.StringIO()
     request.redirect(out)
-    parser = Parser(text, request)
+    parser = Parser(text, request, line_anchors=False)
     formatter = Formatter(request, terse = True)
     reqformatter = None
     if hasattr(request, 'formatter'):
@@ -109,7 +104,8 @@ def formatMarkup(request, text, currentStack = []):
     del currentStack[-1]
     text = text.strip()
     return text
-   
+
+
 def loadLanguage(request, lang):
     """
     Load text dictionary for a specific language.
@@ -133,14 +129,16 @@ def loadLanguage(request, lang):
     if not needsupdate:
         try:
             (uc_texts, uc_unformatted) = pickle.loads(cache.content())
-        except (IOError,ValueError,pickle.UnpicklingError): #bad pickle data, no pickle
+        except (IOError,ValueError,pickle.UnpicklingError): # bad pickle data, no pickle
             needsupdate = 1
 
     if needsupdate:    
         from MoinMoin.util import pysupport
         lang_module = "MoinMoin.i18n." + filename(lang)
-        texts = pysupport.importName(lang_module, "text")
-        if not texts:
+        try:
+            # Language module without text dict will raise AttributeError
+            texts = pysupport.importName(lang_module, "text")
+        except ImportError:
             return (None, None)
         meta = pysupport.importName(lang_module, "meta") 
         encoding = meta['encoding']
@@ -202,13 +200,16 @@ def requestLanguage(request):
     # Or try to return one of the user browser accepted languages, if it
     # is available on this wiki...
     available = wikiLanguages()
-    for lang in browserLanguages(request):
-        if available.has_key(lang):
-            return lang
+    if not request.cfg.language_ignore_browser:
+        for lang in browserLanguages(request):
+            if available.has_key(lang):
+                if request.http_accept_language:
+                    request.setHttpHeader('Vary: Accept-Language')
+                return lang
     
     # Or return the wiki default language...
-    if available.has_key(request.cfg.default_lang):
-        lang = request.cfg.default_lang
+    if available.has_key(request.cfg.language_default):
+        lang = request.cfg.language_default
 
     # If everything else fails, read the manual... or return 'en'
     else:
@@ -220,28 +221,9 @@ def requestLanguage(request):
 def wikiLanguages():
     """
     Return the available user languages in this wiki.
-
-    Since the wiki has only one charset set by the admin, and the user
-    interface files could have another charset, not all languages are
-    available on every wiki - unless we move to Unicode for all
-    languages translations. Even then, Python still can't recode some
-    charsets.
-
-    !!! Note: we use strict = 1 to choose only language that we can
-    recode strictly from the language encodings to the wiki
-    encoding. This preference could be left to the wiki admin instead.
-
-    Return a dict containing a subset of MoinMoin languages ISO codes.
+    As we do everything in unicode (or utf-8) now, everything is available.
     """
-    
-    available = {}
-    for lang in languages.keys():
-        encoding = languages[lang][ENCODING].lower()
-        if (encoding == config.charset or
-            canRecode(encoding, config.charset, strict=1)):
-            available[lang] = languages[lang]
-        
-    return available
+    return languages
 
 
 def browserLanguages(request):
@@ -284,7 +266,7 @@ def getDirection(lang):
     return ('ltr', 'rtl')[languages[lang][DIRECTION]]
 
 
-def getText(str, request, lang, formatted = True):
+def getText(str, request, lang, formatted=True):
     """
     Return a translation of text in the user's language.
     """
@@ -316,26 +298,23 @@ def getText(str, request, lang, formatted = True):
                     userdict = dicts.dict(dictpagename)
                     trans = userdict[str]
         except KeyError:
-            pass
+            # do not simply return trans with str, but recursively call
+            # to get english translation, maybe formatted
+            if lang != 'en':
+                trans = getText(str, request, 'en', formatted)
     return trans
 
 
-########################################################################
-# Encoding
-########################################################################
-
 def canRecode(input, output, strict=1):
-    """
-    Check if we can recode text from input to output.
+    """ Check if we can recode text from input to output.
 
-    Return 1 if we can probably recode from one charset to
-    another, or 0 if the charset are not known or compatible.
-
-    arguments:
-    input -- the input encoding codec name or alias
-    output -- the output encoding codec name or alias
-    strict -- Are you going to recode using errors='strict' or you can
-    get with 'ignore', 'replace' or other error levels?
+    @param input: the input encoding codec name or alias
+    @param output: the output encoding codec name or alias
+    @param strict: Are you going to recode using errors='strict' or you can
+                   get with 'ignore', 'replace' or other error levels?
+    @rtype: bool
+    @return: True if we can probably recode from one charset to another or
+             False if the charset are not known or compatible.
     """
 
     import codecs
@@ -350,14 +329,14 @@ def canRecode(input, output, strict=1):
         decoder = codecs.getdecoder(input)          
     except LookupError:
         # Unknown encoding - forget about it!
-        return 0
+        return False
     
     # We assume that any charset that Python knows about, can be recoded
     # into any Unicode charset. Because codecs have many aliases, we
     # check the codec name itself
     if encoder.__name__.startswith('utf'):
         # This is a unicode encoding, so input could be anything.
-        return 1
+        return True
 
     # We know both encodings but we don't know if we can actually recode
     # any text from one to another. For example, we can recode from
@@ -366,9 +345,9 @@ def canRecode(input, output, strict=1):
     # We choose our answer according to the strict keyword argument
     if strict:
         # We are not sure, so NO
-        return 0
+        return False
     else:
         # Probably you can recode using 'replace' or 'ignore' or other
         # encodings error level and be happy with the results, so YES
-        return 1
+        return True
 
