@@ -4,10 +4,10 @@
     prepend some processing instructions to a .po file to be able to put it
     onto moinmaster wiki, letting it get processed by gettext parser
 """
-import sys, xmlrpclib
+import sys, os, xmlrpclib
 sys.path.insert(0, '../..')
 
-excluded = ["en", "he"] # languages managed in tla repository, not in wiki
+excluded = ["en",] # languages managed in tla repository, not in wiki
 
 lang = sys.argv[1]
 lang = lang.replace('_', '-') # module names use _ instead of -
@@ -43,12 +43,13 @@ data = u"""\
 from MoinMoin.support.BasicAuthTransport import BasicAuthTransport
 
 user = "ThomasWaldmann" # must be a known Wiki account
-password = "wrong"
+password = os.environ.get("PASS", "")
 pagename = "MoinI18n/%s" % lang
 pagedata = data.encode('utf-8')
 
 authtrans = BasicAuthTransport(user, password)
 wiki = xmlrpclib.ServerProxy("http://moinmaster.wikiwikiweb.de/?action=xmlrpc2", transport=authtrans)
 
-wiki.putPage(pagename, pagedata)
+rc = wiki.putPage(pagename, pagedata)
+print "Page: %s rc=%s" % (pagename, rc)
 
