@@ -10,6 +10,7 @@ from MoinMoin import util, config, wikiutil
 from MoinMoin.util import web
 
 class EventLog(LogFile):
+    
     def __init__(self, request, filename=None, buffer_size=65536, **kw):
         if filename == None:
             rootpagename = kw.get('rootpagename', None)
@@ -20,7 +21,8 @@ class EventLog(LogFile):
                 filename = request.rootpage.getPagePath('event-log', isfile=1)
         LogFile.__init__(self, filename, buffer_size)
 
-    def add(self, request, eventtype, values=None, add_http_info=1, mtime_usecs=None):
+    def add(self, request, eventtype, values=None, add_http_info=1,
+            mtime_usecs=None):
         """ Write an event of type `eventtype, with optional key/value
         pairs appended (i.e. you have to pass a dict).
         """
@@ -40,8 +42,8 @@ class EventLog(LogFile):
                 if value:
                     # Save those http headers in UPPERcase
                     values[key.upper()] = value
-        # Encode values in a query string TODO: use more reaable format
-        values = web.makeQueryString(values)
+        # Encode values in a query string TODO: use more readable format
+        values = wikiutil.makeQueryString(values, want_unicode=True)
         self._add(u"%d\t%s\t%s\n" % (mtime_usecs, eventtype, values))
 
     def parser(self, line):
@@ -50,7 +52,7 @@ class EventLog(LogFile):
         except ValueError:
             # badly formatted line in file, skip it
             return None
-        return long(time_usecs), eventtype, web.parseQueryString(kvpairs)
+        return long(time_usecs), eventtype, wikiutil.parseQueryString(kvpairs)
                                                                         
     def set_filter(self, event_types = None):
         if event_types == None:
