@@ -59,13 +59,13 @@ def execute(macro, text, args_re=re.compile(_args_re_pattern), title_re=re.compi
         return ''
 
     # parse and check arguments
-    args = args_re.match(text)
+    args = text and args_re.match(text)
     if not args:
         return (_sysmsg % ('error', _('Invalid include arguments "%s"!')) % (text,))
 
     # prepare including page
     result = []
-    print_mode = macro.form.has_key('action') and macro.form['action'][0] == "print"
+    print_mode = macro.form.has_key('action') and macro.form['action'][0] in ("print", "format")
     this_page = macro.formatter.page
     if not hasattr(this_page, '_macroInclude_pagelist'):
         this_page._macroInclude_pagelist = {}
@@ -110,6 +110,8 @@ def execute(macro, text, args_re=re.compile(_args_re_pattern), title_re=re.compi
         fmt = macro.formatter.__class__(request, is_included=True)
         fmt._base_depth = macro.formatter._base_depth
         inc_page = Page(request, inc_name, formatter=fmt)
+        if not inc_page.exists():
+            continue
         inc_page._macroInclude_pagelist = this_page._macroInclude_pagelist
 
         # check for "from" and "to" arguments (allowing partial includes)
@@ -215,8 +217,9 @@ def execute(macro, text, args_re=re.compile(_args_re_pattern), title_re=re.compi
         strfile = StringIO.StringIO()
         request.redirect(strfile)
         try:
-            cid = request.makeUniqueID("Include_%s" % wikiutil.quoteWikinameFS(inc_page.page_name))
-            inc_page.send_page(request, content_only=1, content_id=cid)
+            cid = request.makeUniqueID("Include_%s" % wikiutil.quoteWikinameURL(inc_page.page_name))
+            inc_page.send_page(request, content_only=1, content_id=cid,
+                               omit_footnotes=True)
             result.append(strfile.getvalue())
         finally:
             request.redirect()
@@ -231,10 +234,10 @@ def execute(macro, text, args_re=re.compile(_args_re_pattern), title_re=re.compi
         # if no heading and not in print mode, then output a helper link
         if editlink and not (level or print_mode):
             result.extend([
-                '<div class="include-link">',
+                macro.formatter.div(1, css_class="include-link"),
                 inc_page.link_to(request, '[%s]' % (inc_name,), css_class="include-page-link"),
                 inc_page.link_to(request, '[%s]' % (_('edit'),), css_class="include-edit-link", querystr={'action': 'edit', 'backto': request._Include_backto}),
-                '</div>',
+                macro.formatter.div(0),
             ])
         # XXX page.link_to is wrong now, it escapes the edit_icon html as it escapes normal text
 
