@@ -9,13 +9,14 @@
 """
 
 import re, time
-from MoinMoin import util, wikiutil
+from MoinMoin import util, wikiutil, config
 from MoinMoin.Page import Page
 from MoinMoin.logfile import editlog
 
 _DAYS_SELECTION = [1, 2, 3, 7, 14, 30, 60, 90]
 _MAX_DAYS = 7
 _MAX_PAGENAME_LENGTH = 15 # 35
+_MAX_COMMENT_LENGTH = 20
 
 #############################################################################
 ### RecentChanges Macro
@@ -26,19 +27,22 @@ Dependencies = ["time"] # ["user", "pages", "pageparams", "bookmark"]
 def format_comment(request, line):
     comment = line.comment
     _ = request.getText
-    if line.action[:3] == 'ATT':
-        import urllib
-        filename = urllib.unquote(line.extra)
+    if line.action.startswith('ATT'):
+        filename = wikiutil.url_unquote(line.extra)
         if line.action == 'ATTNEW':
-            comment = _("Upload of attachment '%(filename)s'.") % {'filename': filename}
+            comment = _("Upload of attachment '%(filename)s'.") % {
+                'filename': filename}
         elif line.action == 'ATTDEL':
-            comment = _("Attachment '%(filename)s' deleted.") % {'filename': filename}
+            comment = _("Attachment '%(filename)s' deleted.") % {
+                'filename': filename}
         elif line.action == 'ATTDRW':
-            comment = _("Drawing '%(filename)s' saved.") % {'filename': filename}
+            comment = _("Drawing '%(filename)s' saved.") % {
+                'filename': filename}
     elif line.action.find('/REVERT') != -1:
         rev = int(line.extra)
         comment = _("Revert to revision %(rev)d.") % {'rev': rev}
-    return comment
+
+    return wikiutil.make_breakable(comment, _MAX_COMMENT_LENGTH)
 
 def format_page_edits(macro, lines, bookmark_usecs):
     request = macro.request
@@ -59,19 +63,21 @@ def format_page_edits(macro, lines, bookmark_usecs):
     elif is_new:
         # show "NEW" icon if page was created after the user's bookmark
         if hilite:
-            html_link = request.theme.make_icon('new')
+            img = request.theme.make_icon('new')
+            html_link = wikiutil.link_tag(request, wikiutil.quoteWikinameURL(pagename),
+                                          img, formatter=macro.formatter)
     elif hilite:
         # show "UPDATED" icon if page was edited after the user's bookmark
         img = request.theme.make_icon('updated')
         html_link = wikiutil.link_tag(request,
                                       wikiutil.quoteWikinameURL(pagename) + "?action=diff&date=%d" % bookmark_usecs,
-                                      img, formatter=macro.formatter, pretty_url=1)
+                                      img, formatter=macro.formatter)
     else:
         # show "DIFF" icon else
         img = request.theme.make_icon('diffrc')
         html_link = wikiutil.link_tag(request,
                                       wikiutil.quoteWikinameURL(line.pagename) + "?action=diff",
-                                      img, formatter=macro.formatter, pretty_url=1)
+                                      img, formatter=macro.formatter)
 
     # print name of page, with a link to it
     force_split = len(page.page_name) > _MAX_PAGENAME_LENGTH
@@ -83,15 +89,15 @@ def format_page_edits(macro, lines, bookmark_usecs):
     d['time_html'] = None
     if request.cfg.changed_time_fmt:
         tdiff = long(tnow - wikiutil.version2timestamp(long(line.ed_time_usecs))) / 60 # has to be long for py 2.2.x
-        if tdiff < 1440:
-            d['time_html'] = _("%(hours)dh %(mins)dm ago") % {
-                'hours': int(tdiff/60), 'mins': tdiff%60}
+        if tdiff < 100:
+            d['time_html'] = _("%(mins)dm ago") % {
+                'mins': tdiff}
         else:
             d['time_html'] = time.strftime(request.cfg.changed_time_fmt, line.time_tuple)
     
     # print editor name or IP
     d['editors'] = None
-    if request.cfg.show_hosts:
+    if request.cfg.show_names:
         if len(lines) > 1:
             counters = {}
             for idx in range(len(lines)):
@@ -120,7 +126,7 @@ def format_page_edits(macro, lines, bookmark_usecs):
     img = request.theme.make_icon('info')
     info_html = wikiutil.link_tag(request,
                                   wikiutil.quoteWikinameURL(line.pagename) + "?action=info",
-                                  img, formatter=macro.formatter, pretty_url=1)
+                                  img, formatter=macro.formatter)
     d['info_html'] = info_html
     
     return request.theme.recentchanges_entry(d)
