@@ -29,7 +29,7 @@ _args_re_pattern = r'^(?P<name>[^,]+)(%s(%s)?%s%s%s%s%s%s%s)?$' % (
     _arg_skipitems, _arg_titlesonly, _arg_editlink)
 
 # from Include, too, but with extra htext group around header text
-_title_re = r"^(?P<heading>\s*(?P<hmarker>=+)\s(?P<htext>.*)\s(?P=hmarker))$"
+_title_re = r"^(?P<heading>(?P<hmarker>=+)\s(?P<htext>.*)\s(?P=hmarker))$"
 
 class TableOfContents:
     """
@@ -75,7 +75,7 @@ class TableOfContents:
         for i in range(self.baseindent, self.indent):
             self.result.append(self.macro.formatter.listitem(0))
             self.result.append(self.macro.formatter.number_list(0))
-        return '\n'.join(self.result)
+        return ''.join(self.result)
 
     def process_lines(self, lines, pagename):
         for line in lines:
@@ -137,16 +137,19 @@ class TableOfContents:
         # Open Lists
         for i in range(0,newindent-self.indent):
             self.result.append(self.macro.formatter.number_list(1))
+            self.result.append(self.macro.formatter.listitem(1))
 
         # Add the heading
         unique_id = ''
         if self.titles[pntt] > 1:
             unique_id = '-%d' % (self.titles[pntt],)
 
+        # close last listitem if same level
         if self.indent == newindent:
             self.result.append(self.macro.formatter.listitem(0))
             
-        self.result.append(self.macro.formatter.listitem(1))
+        if self.indent >= newindent:
+            self.result.append(self.macro.formatter.listitem(1))
         self.result.append(self.macro.formatter.anchorlink(1,
             "head-" + sha.new(pntt.encode(config.charset)).hexdigest() + unique_id) +
                            self.macro.formatter.text(title_text) +
