@@ -1,8 +1,6 @@
 # -*- coding: iso-8859-1 -*-
 """
-    MoinMoin - Multiple config handler
-    and
-    MoinMoin - Configuration defaults class
+    MoinMoin - Multiple configuration handler and Configuration defaults class
 
     @copyright: 2000-2004 by Jrgen Hermann <jh@web.de>
     @license: GNU GPL, see COPYING for details.
@@ -10,73 +8,106 @@
 
 import re, os, sys
 from MoinMoin import error
+import MoinMoin.auth as authmodule
+
+_url_re_cache = None
+_farmconfig_mtime = None
+_config_cache = {}
 
 
-_url_re = None
-def url_re():
+def _importConfigModule(name):
+    """ Import and return configuration module and its modification time
+    
+    Handle all errors except ImportError, because missing file is not
+    always an error.
+    
+    @param name: module name
+    @rtype: tuple
+    @return: module, modification time
+    """
+    try:
+        module = __import__(name, globals(), {})
+        mtime = os.path.getmtime(module.__file__)
+    except ImportError:
+        raise
+    except IndentationError, err:
+        msg = 'IndentationError: %s\n' % str(err) + '''
+
+The configuration files are python modules. Therefore, whitespace is
+important. Make sure that you use only spaces, no tabs are allowed here!
+You have to use four spaces at the beginning of the line mostly.
+'''
+        raise error.ConfigurationError(msg)
+    except Exception, err:
+        msg = '%s: %s' % (err.__class__.__name__, str(err))
+        raise error.ConfigurationError(msg)
+    return module, mtime
+
+
+def _url_re():
     """ Return url matching regular expression
 
-    Using this regular expression, we find the config_module for each
-    url.
-    
-    Import wikis from 'farmconfig' on the first time, compile and cache url_re
-    regular expression, and return it.
-
-    Note: You must restart a long running process when you edit
-    farmconfig.py config file.
+    Import wikis list from farmconfig on the first call and compile a
+    regex. Later then return the cached regex.
 
     @rtype: compiled re object
     @return: url to wiki config  matching re
     """
-    global _url_re
-    if not _url_re:
+    global _url_re_cache, _farmconfig_mtime
+    if _url_re_cache is None:
         try:
-            farmconfig = __import__('farmconfig', globals(), {})
-            pattern = '|'.join([r'(?P<%s>%s)' % (name, regex)
-                                for name, regex in farmconfig.wikis])
-            _url_re = re.compile(pattern)
-        except (ImportError, AttributeError):
-            # It is not there, so we maybe have only one config. Fall back to
-            # old config file name and use it for all urls we get.
-            # Or we have a farmconfig file, but it does not contain a wikis
-            # attribute (because of typo or everything commented out as in
-            # sample config).
-            _url_re = re.compile(r'(?P<wikiconfig>.)')
-    return _url_re
+            farmconfig, _farmconfig_mtime = _importConfigModule('farmconfig')
+        except ImportError:
+            # Default to wikiconfig for all urls.
+            _farmconfig_mtime = 0
+            _url_re_cache = re.compile(r'(?P<wikiconfig>.)')
+        else:
+            try:
+                pattern = '|'.join([r'(?P<%s>%s)' % (name, regex)
+                                    for name, regex in farmconfig.wikis])
+                _url_re_cache = re.compile(pattern)
+            except AttributeError:
+                msg = """
+Missing required 'wikis' list in 'farmconfig.py'.
+
+If you run a single wiki you do not need farmconfig.py. Delete it and
+use wikiconfig.py.
+"""
+                raise error.ConfigurationError(msg)    
+    return _url_re_cache
 
 
-def getConfig(url):
-    """ Make and return config object, or raise an error
+def _makeConfig(name):
+    """ Create and return a config instance 
 
-    If the config file is not found or broken, either because of a typo
-    in farmconfig or deleted file or some other error, we raise a
-    ConfigurationError which is handled by our client.
-    
-    @param url: the url from request, possibly matching specific wiki
-    @rtype: DefaultConfig subclass instance
-    @return: config object for specific wiki
+    Timestamp config with either module mtime or farmconfig mtime. This
+    mtime can be used later to invalidate older caches.
+
+    @param name: module name
+    @rtype: DefaultConfig sub class instance
+    @return: new configuration instance
     """
-    match = url_re().match(url)
-    if match and match.groups():
-        # Get config module name from match
-        for name, value in match.groupdict().items():
-            if value: break
+    global _farmconfig_mtime
+    try:
+        module, mtime = _importConfigModule(name)
+        configClass = getattr(module, 'Config')
+        cfg = configClass(name)
+        cfg.cfg_mtime = max(mtime, _farmconfig_mtime)
+    except ImportError, err:
+        msg = 'ImportError: %s\n' % str(err) + '''
 
-        # FIXME: we should cache config objects by wiki url and return
-        # here a ready to use config, instead of creating new instance
-        # for each request.
-        
-        try:
-            module =  __import__(name, globals(), {})
-            Config = getattr(module, 'Config', None)
-            if Config:
-                # Config found, return config instance using name as
-                # site identifier (name must be unique of our url_re).
-                cfg = Config(name)
-                return cfg
-            else:
-                # Broken config file, probably old config from 1.2
-                msg = '''
+Check that the file is in the same directory as the server script. If
+it is not, you must add the path of the directory where the file is
+located to the python path in the server script. See the comments at
+the top of the server script.
+
+Check that the configuration file name is either "wikiconfig.py" or the
+module name specified in the wikis list in farmconfig.py. Note that the
+module name does not include the ".py" suffix.
+'''
+        raise error.ConfigurationError(msg)
+    except AttributeError:
+        msg = '''
 Could not find required "Config" class in "%(name)s.py". This might
 happen if you are trying to use a pre 1.3 configuration file, or made a
 syntax or spelling error.
@@ -84,60 +115,44 @@ syntax or spelling error.
 Please check your configuration file. As an example for correct syntax,
 use the wikiconfig.py file from the distribution.
 ''' % {'name': name}
+        raise error.ConfigurationError(msg)
+    return cfg
 
-        # We don't handle fatal errors here
-        except error.FatalError, err:
-            raise err
 
-        # These errors will not be big surprise:       
-        except ImportError, err:
-            msg = '''
-Import of configuration file "%(name)s.py" failed because of
-ImportError: %(err)s.
-
-Check that the file is in the same directory as the server script. If
-it is not, you must add the path of the directory where the file is located
-to the python path in the server script. See the comments at the top of
-the server script.
-
-Check that the configuration file name is either "wikiconfig.py" or the
-module name specified in the wikis list in farmconfig.py. Note that the module
-name does not include the ".py" suffix.
-''' % {'name': name, 'err': str(err)}
-
-        except IndentationError, err:
-            msg = '''
-Import of configuration file "%(name)s.py" failed because of
-IndentationError: %(err)s.
-
-The configuration files are python modules. Therefore, whitespace is
-important. Make sure that you use only spaces, no tabs are allowed here!
-You have to use four spaces at the beginning of the line mostly.
-''' % {'name': name, 'err': str(err)}
-
-        # But people can have many other errors. We hope that the python
-        # error message will help them.
-        except:
-            err = sys.exc_info()[1]
-            msg = '''
-Import of configuration file "%(name)s.py" failed because of %(class)s:
-%(err)s.
-
-We hope this error message make sense. If not, you are welcome to ask on
-the page http://moinmoin.wikiwikiweb.de/MoinMoinQuestions/ConfigFiles
-or the #moin channel on irc.freenode.net or on the mailing list.
-''' % {'name': name, 'class': err.__class__.__name__, 'err': str(err)}
-
-    else:
-        # URL did not match anything, probably error in farmconfig.wikis 
+def _getConfigName(url):
+    """ Return config name for url or raise """
+    match = _url_re().match(url)
+    if not (match and match.groups()):
         msg = '''
 Could not find a match for url: "%(url)s".
 
 Check your URL regular expressions in the "wikis" list in
 "farmconfig.py". 
 ''' % {'url': url}
+        raise error.ConfigurationError(msg)    
+    for name, value in match.groupdict().items():
+        if value: break
+    return name
 
-    raise error.ConfigurationError(msg)
+
+def getConfig(url):
+    """ Return cached config instance for url or create new one
+
+    If called by many threads in the same time multiple config
+    instances might be created. The first created item will be
+    returned, using dict.setdefault.
+
+    @param url: the url from request, possibly matching specific wiki
+    @rtype: DefaultConfig subclass instance
+    @return: config object for specific wiki
+    """
+    configName = _getConfigName(url)
+    try:
+        config = _config_cache[configName]
+    except KeyError:
+        config = _makeConfig(configName)
+        config = _config_cache.setdefault(configName, config)
+    return config
 
 
 # This is a way to mark some text for the gettext tools so that they don't
@@ -146,26 +161,32 @@ def _(text): return text
 
 
 class DefaultConfig:
-    """ default config values
-
-    FIXME: update according to MoinMoin:UpdateConfiguration
-    """    
-    acl_enabled = 0
+    """ default config values """
+    
     # All acl_right lines must use unicode!
     acl_rights_default = u"Trusted:read,write,delete,revert Known:read,write,delete,revert All:read,write"
     acl_rights_before = u""
     acl_rights_after = u""
     acl_rights_valid = ['read', 'write', 'delete', 'revert', 'admin']
     
-    allow_extended_names = 1
-    allow_numeric_entities = 1
-    allowed_actions = []
+    actions_excluded = [] # ['DeletePage', 'AttachFile', 'RenamePage']
     allow_xslt = 0
     attachments = None # {'dir': path, 'url': url-prefix}
-    auth_http_enabled = 0
-    auth_http_insecure = 0
-    bang_meta = 0
-    backtick_meta = 1
+    auth = [authmodule.moin_cookie]
+    
+    backup_compression = 'gz'
+    backup_users = []
+    backup_include = []
+    backup_exclude = [
+        r"(.+\.py(c|o)$)",
+        r"%(cache_dir)s",
+        r"%(/)spages%(/)s.+%(/)scache%(/)s[^%(/)s]+$" % {'/': os.sep},
+        r"%(/)s(edit-lock|event-log|\.DS_Store)$" % {'/': os.sep},
+        ]
+    backup_storage_dir = '/tmp'
+    backup_restore_target_dir = '/tmp'
+    
+    bang_meta = 1
     caching_formats = ['text_html']
     changed_time_fmt = '%H:%M'
     # chars_{upper,lower,digits,spaces} see MoinMoin/util/chartypes.py
@@ -173,15 +194,29 @@ class DefaultConfig:
     # chart_options = {'width = 720, 'height': 540}
     chart_options = None
     config_check_enabled = 0
+    cookie_domain = None # use '.domain.tld" for a farm with hosts in that domain
+    cookie_path = None   # use '/wikifarm" for a farm with pathes below that path
     cookie_lifetime = 12 # 12 hours from now
     data_dir = './data/'
     data_underlay_dir = './underlay/'
     date_fmt = '%Y-%m-%d'
     datetime_fmt = '%Y-%m-%d %H:%M:%S'
-    default_lang = 'en'
     default_markup = 'wiki'
+    docbook_html_dir = r"/usr/share/xml/docbook/stylesheet/nwalsh/html/" # correct for debian sarge
+    editor_default = 'text' # which editor is called when nothing is specified
+    editor_ui = 'freechoice' # which editor links are shown on user interface
+    editor_force = False
     edit_locking = 'warn 10' # None, 'warn <timeout mins>', 'lock <timeout mins>'
-    edit_rows = 30
+    edit_rows = 20
+                
+    hacks = {} # { 'feature1': value1, ... }
+               # Configuration for features still in development.
+               # For boolean stuff just use config like this:
+               #   hacks = { 'feature': True, ...}
+               # and in the code use:
+               #   if cfg.hacks.get('feature', False): <doit>
+               # A non-existing hack key should ever mean False, None, "", [] or {}!
+
     hosts_deny = []
     html_head = ''
     html_head_queries = '''<meta name="robots" content="noindex,nofollow">\n'''
@@ -190,10 +225,20 @@ class DefaultConfig:
     html_head_normal  = '''<meta name="robots" content="index,nofollow">\n'''
     html_pagetitle = None
 
+    interwiki_preferred = [] # list of wiki names to show at top of interwiki list
+
+    language_default = 'en'
+    language_ignore_browser = False # ignore browser settings, use language_default
+                                    # or user prefs
+
+    lupy_search = False # disabled until lupy is finished
+
     mail_login = None # or "user pwd" if you need to use SMTP AUTH
+    mail_sendmail = None # "/usr/sbin/sendmail -t -i" to not use SMTP, but sendmail
     mail_smarthost = None
-    mail_from = None
-    navi_bar = [ u'%(page_front_page)s', u'RecentChanges', u'FindPage', u'HelpContents', ]
+    mail_from = None # u'Jrgen Wiki <noreply@jhwiki.org>'
+    
+    navi_bar = [u'RecentChanges', u'FindPage', u'HelpContents', ]
     nonexist_qm = 0
 
     page_credits = [
@@ -207,13 +252,12 @@ class DefaultConfig:
     page_header1 = ''
     page_header2 = ''
     
-    page_front_page = u'FrontPage'
+    page_front_page = u'HelpOnLanguages' # this will make people choose a sane config
     page_local_spelling_words = u'LocalSpellingWords'
     page_category_regex = u'^Category[A-Z]'
-    page_dict_regex = u'[a-z]Dict$'
-    page_form_regex = u'[a-z]Form$'
-    page_group_regex = u'[a-z]Group$'
-    page_template_regex = u'[a-z]Template$'
+    page_dict_regex = u'[a-z0-9]Dict$'
+    page_group_regex = u'[a-z0-9]Group$'
+    page_template_regex = u'[a-z0-9]Template$'
 
     page_license_enabled = 0
     page_license_page = u'WikiLicense'
@@ -240,22 +284,44 @@ class DefaultConfig:
         'up':          ("%(q_page_parent_page)s", _("Up"), "up"),
         }
     refresh = None # (minimum_delay, type), e.g.: (2, 'internal')
+    rss_cache = 60 # suggested caching time for RecentChanges RSS, in seconds
     shared_intermap = None # can be string or list of strings (filenames)
     show_hosts = 1
-    show_section_numbers = 1
+    show_interwiki = 0
+    show_login = 1
+    show_names = True
+    show_section_numbers = 0
     show_timings = 0
     show_version = 0
     siteid = 'default'
+    stylesheets = [] # list of tuples (media, csshref) to insert after theme css, before user css
+    superuser = [] # list of unicode user names that have super powers :)
+    
+    surge_action_limits = { # allow max. <count> <action> requests per <dt> secs
+        # action: (count, dt)
+        'show': (20, 60),
+        'raw': (20, 40),  # some people use this for css
+        'AttachFile': (60, 60),
+        'diff': (30, 60),
+        'fullsearch': (5, 60),
+        'edit': (10, 120),
+        'rss_rc': (1, 60),
+        'default': (30, 60),
+    }
+    surge_lockout_time = 3600 # secs you get locked out when you ignore warnings
+    
     theme_default = 'modern'
     theme_force = False
     trail_size = 5
     tz_offset = 0.0 # default time zone offset in hours from UTC
-    
+    user_autocreate = False # do we auto-create user profiles
+    user_email_unique = True # do we check whether a user's email is unique?
+
     # a regex of HTTP_USER_AGENTS that should be excluded from logging
     # and receive a FORBIDDEN for anything except viewing a page
-    ua_spiders = ('archiver|crawler|curl|google|holmes|htdig|httrack|httpunit|jeeves|larbin|leech|'
-                  'linkbot|linkmap|linkwalk|mercator|mirror|nutbot|robot|scooter|'
-                  'search|sherlock|sitecheck|spider|wget')
+    ua_spiders = ('archiver|cfetch|crawler|curl|gigabot|google|holmes|htdig|httrack|httpunit|jeeves|larbin|leech|'
+                  'linkbot|linkmap|linkwalk|mercator|mirror|msnbot|nutbot|omniexplorer|puf|robot|scooter|'
+                  'search|sherlock|sitecheck|spider|teleport|voyager|wget')
 
     # Wiki identity
     sitename = u'Untitled Wiki'
@@ -264,7 +330,87 @@ class DefaultConfig:
     interwikiname = None
     
     url_mappings = {}
+
+    user_checkbox_fields = [
+        ('mailto_author', lambda _: _('Publish my email (not my wiki homepage) in author info')),
+        ('edit_on_doubleclick', lambda _: _('Open editor on double click')),
+        ('remember_last_visit', lambda _: _('Jump to last visited page instead of frontpage')),
+        ('show_nonexist_qm', lambda _: _('Show question mark for non-existing pagelinks')),
+        ('show_page_trail', lambda _: _('Show page trail')),
+        ('show_toolbar', lambda _: _('Show icon toolbar')),
+        ('show_topbottom', lambda _: _('Show top/bottom links in headings')),
+        ('show_fancy_diff', lambda _: _('Show fancy diffs')),
+        ('wikiname_add_spaces', lambda _: _('Add spaces to displayed wiki names')),
+        ('remember_me', lambda _: _('Remember login information')),
+        ('want_trivial', lambda _: _('Subscribe to trivial changes')),
+        
+        ('disabled', lambda _: _('Disable this account forever')),
+        # if an account is disabled, it may be used for looking up
+        # id -> username for page info and recent changes, but it
+        # is not usable for the user any more:
+    ]
+    user_checkbox_defaults = {'mailto_author':       0,
+                              'edit_on_doubleclick': 0,
+                              'remember_last_visit': 0,
+                              'show_nonexist_qm':    nonexist_qm,
+                              'show_page_trail':     1,
+                              'show_toolbar':        1,
+                              'show_topbottom':      0,
+                              'show_fancy_diff':     1,
+                              'wikiname_add_spaces': 0,
+                              'remember_me':         1,
+                              'want_trivial':        0,
+                             }
+    # don't let the user change those
+    # user_checkbox_disable = ['disabled', 'want_trivial']
+    user_checkbox_disable = []
     
+    # remove those checkboxes:
+    #user_checkbox_remove = ['edit_on_doubleclick', 'show_nonexist_qm', 'show_toolbar', 'show_topbottom',
+    #                        'show_fancy_diff', 'wikiname_add_spaces', 'remember_me', 'disabled',]
+    user_checkbox_remove = []
+    
+    user_form_fields = [
+        ('name', _('Name'), "text", "36", _("(Use Firstname''''''Lastname)")),
+        ('aliasname', _('Alias-Name'), "text", "36", ''),
+        ('password', _('Password'), "password", "36", ''),
+        ('password2', _('Password repeat'), "password", "36", _('(Only when changing passwords)')),
+        ('email', _('Email'), "text", "36", ''),
+        ('css_url', _('User CSS URL'), "text", "40", _('(Leave it empty for disabling user CSS)')),
+        ('edit_rows', _('Editor size'), "text", "3", ''),
+        ##('theme', _('Preferred theme'), [self._theme_select()])
+        ##('', _('Editor Preference'), [self._editor_default_select()])
+        ##('', _('Editor shown on UI'), [self._editor_ui_select()])
+        ##('', _('Time zone'), [self._tz_select()])
+        ##('', _('Date format'), [self._dtfmt_select()])
+        ##('', _('Preferred language'), [self._lang_select()])
+    ]
+    user_form_defaults = { # key: default
+        'name': '',
+        'aliasname': '',
+        'password': '',
+        'password2': '',
+        'email': '',
+        'css_url': '',
+        'edit_rows': "20",
+    }
+    # don't let the user change those, but show them:
+    #user_form_disable = ['name', 'aliasname', 'email',]
+    user_form_disable = []
+    
+    # remove those completely:
+    #user_form_remove = ['password', 'password2', 'css_url', 'logout', 'create', 'account_sendmail',]
+    user_form_remove = []
+    
+    # attributes we do NOT save to the userpref file
+    user_transient_fields =  ['id', 'valid', 'may', 'auth_username', 'trusted', 'password', 'password2', 'auth_method', 'auth_attribs']
+
+    user_homewiki = 'Self' # interwiki name for where user homepages are located
+
+    unzip_single_file_size = 2.0 * 1000**2
+    unzip_attachments_space = 200.0 * 1000**2
+    unzip_attachments_count = 51 # 1 zip file + 50 files contained in it
+
     xmlrpc_putpage_enabled = 0 # if 0, putpage will write to a test page only
     xmlrpc_putpage_trusted_only = 1 # if 1, you will need to be http auth authenticated
     
@@ -275,6 +421,15 @@ class DefaultConfig:
         self.siteid = siteid
         if self.config_check_enabled:
             self._config_check()
+
+        # define directories
+        self.moinmoin_dir = os.path.abspath(os.path.dirname(__file__))
+        data_dir = os.path.normpath(self.data_dir)
+        self.data_dir = data_dir
+        for dirname in ('user', 'cache', 'plugin'):
+            name = dirname + '_dir'
+            if not getattr(self, name, None):
+                setattr(self, name, os.path.join(data_dir, dirname))
             
         # Try to decode certain names which allow unicode
         self._decode()
@@ -284,9 +439,12 @@ class DefaultConfig:
 
         # Load plugin module
         self._loadPluginModule()
+
+        # Preparse user dicts
+        self._fillDicts()
         
         # Normalize values
-        self.default_lang = self.default_lang.lower()
+        self.language_default = self.language_default.lower()
 
         # Use site name as default name-logo
         if self.logo_string is None:
@@ -304,25 +462,17 @@ class DefaultConfig:
             except ImportError:
                 self.chart_options = None
         
-        # create excluded_actions with actions thats are not in
-        # allowed_actions
-        excluded_actions = ['DeletePage', 'AttachFile', 'RenamePage']
-        self.excluded_actions = [action for action in excluded_actions
-                                 if not action in self.allowed_actions]
-        
-        # define directories
-        self.moinmoin_dir = os.path.abspath(os.path.dirname(__file__))
-        data_dir = os.path.normpath(self.data_dir)
-        self.data_dir = data_dir
-        for dirname in ('user', 'cache', 'plugin'):
-            name = dirname + '_dir'
-            if not getattr(self, name, None):
-                setattr(self, name, os.path.join(data_dir, dirname))
-        
-        # post process navibar
+        # post process
         # we replace any string placeholders with config values
         # e.g u'%(page_front_page)s' % self
         self.navi_bar = [elem % self for elem in self.navi_bar]
+        self.backup_exclude = [elem % self for elem in self.backup_exclude]
+
+        # list to cache lupy searcher objects
+        self.lupy_searchers = []
+
+        # check if mail is possible and set flag:
+        self.mail_enabled = (self.mail_smarthost is not None or self.mail_sendmail is not None) and self.mail_from
 
     def _config_check(self):
         """ Check namespace and warn about unknown names
@@ -344,7 +494,6 @@ Unknown configuration options: %s.
 For more information, visit HelpOnConfiguration. Please check your
 configuration for typos before requesting support or reporting a bug.
 """ % ', '.join(unknown)
-            from MoinMoin import error
             raise error.ConfigurationError(msg)
 
     def _decode(self):
@@ -362,8 +511,6 @@ configuration for typos before requesting support or reporting a bug.
         config files.
         """
         charset = 'utf-8'
-
-        # TODO: add to translation in 1.4?
         message = u'''
 "%(name)s" configuration variable is a string, but should be
 unicode. Use %(name)s = u"value" syntax for unicode variables.
@@ -374,10 +521,10 @@ file. It should match the actual charset of the configuration file.
         
         decode_names = (
             'sitename', 'logo_string', 'navi_bar', 'page_front_page',
-            'page_category_regex', 'page_dict_regex', 'page_form_regex',
+            'page_category_regex', 'page_dict_regex', 
             'page_group_regex', 'page_template_regex', 'page_license_page',
             'page_local_spelling_words', 'acl_rights_default',
-            'acl_rights_before', 'acl_rights_after',
+            'acl_rights_before', 'acl_rights_after', 'mail_from'
             )
         
         for name in decode_names:
@@ -390,7 +537,7 @@ file. It should match the actual charset of the configuration file.
                     except UnicodeError:
                         raise error.ConfigurationError(message %
                                                        {'name': name})
-                # Look into lists and try to decode string inside them
+                # Look into lists and try to decode strings inside them
                 elif isinstance(attr, list):
                     for i in xrange(len(attr)):
                         item = attr[i]
@@ -418,11 +565,12 @@ file. It should match the actual charset of the configuration file.
             path_pages = os.path.join(path, "pages")
             if not (os.path.isdir(path_pages) and os.access(path_pages, mode)):
                 msg = '''
-"%(attr)s" does not exists at "%(path)s", or has incorrect ownership and
+%(attr)s "%(path)s" does not exists, or has incorrect ownership or
 permissions.
 
-Make sure the directory and the subdirectory pages are owned by the web server and are readable,
-writable and executable by the web server user and group.
+Make sure the directory and the subdirectory pages are owned by the web
+server and are readable, writable and executable by the web server user
+and group.
 
 It is recommended to use absolute paths and not relative paths. Check
 also the spelling of the directory name.
@@ -442,23 +590,15 @@ also the spelling of the directory name.
         import sys, imp
 
         name = self.siteid + '.plugin'
-        
-        # Get lock functions - require Python 2.3
-        try:
-            acquire_lock = imp.acquire_lock
-            release_lock = imp.release_lock
-        except AttributeError:
-            def acquire_lock(): pass
-            def release_lock(): pass
-
         try:
             # Lock other threads while we check and import
-            acquire_lock()
+            imp.acquire_lock()
             try:
                 # If the module is not loaded, try to load it
                 if not name in sys.modules:
                     # Find module on disk and try to load - slow!
-                    fp, path, info = imp.find_module('plugin', [self.data_dir])
+                    plugin_parent_dir = os.path.abspath(os.path.join(self.plugin_dir, '..'))
+                    fp, path, info = imp.find_module('plugin', [plugin_parent_dir])
                     try:
                         # Load the module and set in sys.modules             
                         module = imp.load_module(name, fp, path, info)
@@ -468,17 +608,28 @@ also the spelling of the directory name.
                         if fp:
                             fp.close()
             finally:
-                release_lock()
+                imp.release_lock()
         except ImportError, err:
             msg = '''
-Could not import plugin package from "%(path)s" because of ImportError:
+Could not import plugin package "%(path)s/plugin" because of ImportError:
 %(err)s.
 
 Make sure your data directory path is correct, check permissions, and
 that the data/plugin directory has an __init__.py file.
 ''' % {'path': self.data_dir, 'err': str(err)}
             raise error.ConfigurationError(msg)
-            
+
+    def _fillDicts(self):
+        """ fill config dicts
+
+        Fills in missing dict keys of derived user config by copying
+        them from this base class.
+        """
+        # user checkbox defaults
+        for key, value in DefaultConfig.user_checkbox_defaults.items():
+            if not self.user_checkbox_defaults.has_key(key):
+                self.user_checkbox_defaults[key] = value
+
     def __getitem__(self, item):
         """ Make it possible to access a config object like a dict """
         return getattr(self, item)
