@@ -6,7 +6,7 @@
         and by Alexander Schremmer <alex AT alexanderweb DOT de>
     @license: GNU GPL, see COPYING for details.
 
-    REQUIRES docutils 0.3.3 or later
+    REQUIRES docutils 0.3.10 or later (must be later than December 30th, 2005)
 """
 
 import re
@@ -14,11 +14,15 @@ import new
 import StringIO
 import __builtin__
 import sys
-import copy
+
+import types
+import os
 
 # docutils imports are below
 import MoinMoin.parser.wiki
 from MoinMoin.Page import Page
+from MoinMoin.action import AttachFile
+from MoinMoin import wikiutil
 
 Dependencies = [] # this parser just depends on the raw text
 
@@ -40,14 +44,40 @@ class dummyUrllib2:
     urlopen = staticmethod(urlopen)
 
 # # # All docutils imports must be contained below here
-import docutils
-from docutils.core import publish_parts
-from docutils.writers import html4css1
-from docutils.nodes import fully_normalize_name, reference
-from docutils.parsers import rst
-from docutils.parsers.rst import directives, roles
+try:
+    import docutils
+    from docutils.core import publish_parts
+    from docutils.writers import html4css1
+    from docutils.nodes import reference
+    from docutils.parsers import rst
+    from docutils.parsers.rst import directives, roles
 # # # All docutils imports must be contained above here
 
+    ErrorParser = None # used in the case of missing docutils
+    docutils.io.FileOutput = docutils.io.FileInput = dummyIO
+except ImportError:
+    # we need to workaround this totally broken plugin interface that does
+    # not allow us to raise exceptions
+    class ErrorParser:
+        caching = 0
+        Dependencies = Dependencies # copy dependencies from module-scope
+
+        def __init__(self, raw, request, **kw):
+            self.raw = raw
+            self.request = request
+    
+        def format(self, formatter):
+            _ = self.request.getText
+            from MoinMoin.parser import plain
+            self.request.write(formatter.sysmsg(1) +
+                               formatter.rawHTML(_('Rendering of reStructured text is not possible, ''please'' install docutils.')) +
+                               formatter.sysmsg(0))
+            plain.Parser(self.raw, self.request).format(formatter)
+    
+    # Create a pseudo docutils environment
+    docutils = html4css1 = dummyUrllib2()
+    html4css1.HTMLTranslator = html4css1.Writer = object
+
 def safe_import(name, globals = None, locals = None, fromlist = None):
     mod = __builtin__.__import__(name, globals, locals, fromlist)
     if mod:
@@ -63,9 +93,7 @@ for i in sys.modules.keys():
         sys.modules[i].open = dummyOpen
         sys.modules[i].urllib2 = dummyUrllib2
         sys.modules[i].__import__ = safe_import
-
-docutils.io.FileOutput = docutils.io.FileInput = dummyIO
-
+        
 # --- End of dummy-code --------------------------------------------------------
 
 def html_escape_unicode(node):
@@ -88,23 +116,23 @@ class MoinWriter(html4css1.Writer):
         """
             Normally an unknown reference would be an error in an reST document.
             However, this is how new documents are created in the wiki. This
-            passes on unknown references to eventually be handled by the
-            MoinMoin formatter.
+            passes on unknown references to eventually be handled by
+            MoinMoin.
         """
-        # TODO: Need to better document the attributes here.
-        if getattr(node, 'indirect_reference_name', None):
+        if hasattr(node, 'indirect_reference_name'):
             node['refuri'] = node.indirect_reference_name
-            return 1
-        elif 'id' in node.attributes:
-            # I'm pretty sure the first test should catch any targets or
-            # references with the "id" attribute. Therefore, if we get to here
-            # its probably an internal link that didn't work so we let it go
-            # through as an error.
-            return 0
-        node['refuri'] = node['refname']
+        elif (len(node['ids']) != 0):
+            # If the node has an id then it's probably an internal link. Let
+            # docutils generate an error.
+            return False
+        elif node.hasattr('name'):
+            node['refuri'] = node['name']
+        else:
+            node['refuri'] = node['refname']
         del node['refname']
+        node.resolved = 1
         self.nodes.append(node)
-        return 1
+        return True
 
     wiki_resolver.priority = 001
 
@@ -121,7 +149,13 @@ class MoinWriter(html4css1.Writer):
         self.wikiparser.formatter = self.formatter
         self.wikiparser.hilite_re = None
         self.nodes = []
-
+        # Make sure it's a supported docutils version.
+        required_version = (0, 3, 10)
+        current_version = tuple([int(i) for i in (docutils.__version__.split('.')+['0','0'])[:3]])
+        if current_version < required_version:
+            err = 'ERROR: The installed docutils version is %s;' % ('.'.join([str(i) for i in current_version]))
+            err += ' version %s or later is required.' % ('.'.join([str(i) for i in required_version]))
+            raise RuntimeError, err
 
     def translate(self):
         visitor = MoinTranslator(self.document,
@@ -133,7 +167,6 @@ class MoinWriter(html4css1.Writer):
         self.visitor = visitor
         self.output = html_escape_unicode(visitor.astext())
 
-
 class Parser:
     caching = 1
     Dependencies = Dependencies # copy dependencies from module-scope
@@ -147,28 +180,48 @@ class Parser:
         # Create our simple parser
         parser = MoinDirectives(self.request)
 
-        parts =  publish_parts(source = self.raw,
-                               writer = MoinWriter(formatter, self.request),
-                               settings_overrides = {'halt_level': 5,
-                                                     'traceback': True,
-                                                     'file_insertion_enabled': 0,
-                                                     'raw_enabled': 0,
-                                                     }
-                              )
+        parts = publish_parts(
+            source = self.raw,
+            writer = MoinWriter(formatter, self.request),
+            settings_overrides = {
+                'halt_level': 5,
+                'traceback': True,
+                'file_insertion_enabled': 0,
+                'raw_enabled': 0,
+                'stylesheet_path': '',
+            }
+        )
 
-        text = ''
+        html = []
         if parts['title']:
-            text += '<h2>' + parts['title'] + '</h2>'
+            html.append(formatter.rawHTML('<h1>%s</h1>' % parts['title']))
         # If there is only one subtitle then it is held in parts['subtitle'].
         # However, if there is more than one subtitle then this is empty and
         # fragment contains all of the subtitles.
         if parts['subtitle']:
-            text += '<h3>' + parts['subtitle'] + '</h3>'
+            html.append(formatter.rawHTML('<h2>%s</h2>' % parts['subtitle']))
         if parts['docinfo']:
-            text += parts['docinfo']
-        text += parts['fragment']
-        self.request.write(html_escape_unicode(text))
+            html.append(parts['docinfo'])
+        html.append(parts['fragment'])
+        self.request.write(html_escape_unicode('\n'.join(html)))
 
+class RawHTMLList(list):
+    """
+        RawHTMLList catches all html appended to internal HTMLTranslator lists.
+        It passes the HTML through the MoinMoin rawHTML formatter to strip 
+        markup when necessary. This is to support other formatting outputs
+        (such as ?action=format&mimetype=text/plain).
+    """
+    
+    def __init__(self, formatter):
+        self.formatter = formatter
+        
+    def append(self, text):
+        f = sys._getframe()
+        if f.f_back.f_code.co_filename.endswith('html4css1.py'):
+            if isinstance(text, types.StringType) or isinstance(text, types.UnicodeType):
+                text = self.formatter.rawHTML(text)
+        list.append(self, text)
 
 class MoinTranslator(html4css1.HTMLTranslator):
 
@@ -176,7 +229,7 @@ class MoinTranslator(html4css1.HTMLTranslator):
         html4css1.HTMLTranslator.__init__(self, document)
         self.formatter = formatter
         self.request = request
-        # MMG: Using our own writer when needed. Save the old one to restore
+        # Using our own writer when needed. Save the old one to restore
         # after the page has been processed by the html4css1 parser.
         self.original_write, self.request.write = self.request.write, self.capture_wiki_formatting
         self.wikiparser = parser
@@ -190,6 +243,20 @@ class MoinTranslator(html4css1.HTMLTranslator):
         # function (see visit_image for an example).
         self.wiki_text = ''
         self.setup_wiki_handlers()
+        
+        # Make all internal lists RawHTMLLists, see RawHTMLList class
+        # comment for more information.
+        for i in self.__dict__:
+            if isinstance(getattr(self, i), types.ListType):
+                setattr(self, i, RawHTMLList(formatter))
+
+    def depart_docinfo(self, node):
+        """
+            depart_docinfo assigns a new list to self.body, we need to re-make that
+            into a RawHTMLList.
+        """
+        html4css1.HTMLTranslator.depart_docinfo(self, node)
+        self.body = RawHTMLList(self.formatter)
 
     def capture_wiki_formatting(self, text):
         """
@@ -231,37 +298,6 @@ class MoinTranslator(html4css1.HTMLTranslator):
         self.request.write = self.original_write
         return html4css1.HTMLTranslator.astext(self)
 
-    def process_inline(self, node, uri_string):
-        """
-            Process the "inline:" link scheme. This can either ome from
-            visit_reference or from visit_image. The uri_string changes
-            depending on the caller. The uri is passed to MoinMoin to handle the
-            inline link. If it is an image, the src line is extracted and passed
-            to the html4css1 writer to allow the reST image attributes.
-            Otherwise, the html from MoinMoin is inserted into the reST document
-            and SkipNode is raised.
-        """
-        self.process_wiki_text(node[uri_string])
-        # Only pass the src and alt parts to the writer. The reST writer
-        # inserts its own tags so we don't need the MoinMoin html markup.
-        src = re.search('src="([^"]+)"', self.wiki_text)
-        if src:
-            node['uri'] = src.groups()[0]
-            if not 'alt' in node.attributes:
-                alt = re.search('alt="([^"]*)"', self.wiki_text)
-                if alt:
-                    node['alt'] = alt.groups()[0]
-        else:
-            # Image doesn't exist yet for the page so just use what's
-            # returned from MoinMoin verbatim
-            self.add_wiki_markup()
-
-    def process_wiki_target(self, target):
-        self.process_wiki_text(target)
-        # MMG: May need a call to fixup_wiki_formatting here but I
-        # don't think so.
-        self.add_wiki_markup()
-
     def fixup_wiki_formatting(self, text):
         replacement = {'<p>': '', '</p>': '', '\n': '', '> ': '>'}
         for src, dst in replacement.items():
@@ -284,53 +320,59 @@ class MoinTranslator(html4css1.HTMLTranslator):
             MoinMoin formatter and the resulting markup is inserted into the
             document in the place of the original link reference.
         """
-        moin_link_schemes = ('wiki:', 'attachment:', 'drawing:', '[[',
-                             'inline:')
-
         if 'refuri' in node.attributes:
-            target = None
             refuri = node['refuri']
+            prefix = ''
+            link = refuri
+            if ':' in refuri:
+                prefix, link = refuri.lstrip().split(':', 1)
+            
+            # First see if MoinMoin should handle completely. Exits through add_wiki_markup.
+            if ((refuri.startswith('[[') and refuri.endswith(']]')) or 
+                    (prefix == 'drawing') or
+                    (prefix == 'inline')):
+                self.process_wiki_text(refuri)
+                # Don't call fixup_wiki_formatting because who knows what
+                # MoinMoin is inserting. (exits through add_wiki_markup)
+                self.add_wiki_markup()
 
-            # MMG: Fix this line
-            if [scheme for scheme in moin_link_schemes if
-                    refuri.lstrip().startswith(scheme)]:
-                # For a macro, We want the actuall text from the user in target,
-                # not the fully normalized version that is contained in refuri.
-                if refuri.startswith('[['):
-                    target = node['name']
-                else:
-                    target = refuri
-            # TODO: Figure out the following two elif's and comment
-            # appropriately.
-            # The node should have a whitespace normalized name if the docutlis
-            # reStructuredText parser would normally fully normalize the name.
-            elif ('name' in node.attributes and
-                  fully_normalize_name(node['name']) == refuri):
-                target = ':%s:' % (node['name'])
-            # If its not a uri containing a ':' then its probably destined for
-            # wiki space.
-            elif ':' not in refuri:
-                target = ':%s:' % (refuri)
-
-            if target:
-                if target.startswith('inline:'):
-                    self.process_inline(node, 'refuri')
-                elif target.startswith('[[') and target.endswith(']]'):
-                    self.process_wiki_target(target)
-                else:
-                    # Not a macro or inline so hopefully its a link. Put the target in
-                    # brackets so that MoinMoin knows its a link. Extract the
-                    # href, if it exists, and let docutils handle it from there.
-                    # If there is no href just add whatever MoinMoin returned.
-                    node_text = node.astext().replace('\n', ' ')
-                    self.process_wiki_text('[%s %s]' % (target, node_text))
-                    href = re.search('href="([^"]+)"', self.wiki_text)
-                    if href:
-                        # dirty hack in order to undo the HTML entity quoting
-                        node['refuri'] = href.groups()[0].replace("&amp;", "&")
-                    else:
-                        self.wiki_text = self.fixup_wiki_formatting(self.wiki_text)
-                        self.add_wiki_markup()
+            # From here down, all links are handled by docutils (except 
+            # missing attachments), just fixup node['refuri'].
+            if prefix == 'attachment':
+                attach_file = AttachFile.getFilename(self.request, 
+                        self.request.page.page_name, link)
+                if not os.path.exists(attach_file):
+                    # Attachment doesn't exist, give to MoinMoin to insert
+                    # upload text.
+                    self.process_wiki_text(refuri)
+                    self.add_wiki_markup()
+                # Attachment exists, just get a link to it.
+                node['refuri'] = AttachFile.getAttachUrl(self.request.page.page_name, 
+                        link, self.request)
+                if not [i for i in node.children if i.__class__ == docutils.nodes.image]:
+                    node['classes'].append(prefix)                
+            elif prefix == 'wiki':
+                wikitag, wikiurl, wikitail, err = wikiutil.resolve_wiki(self.request, link)
+                wikiurl = wikiutil.mapURL(self.request, wikiurl)
+                node['refuri'] = wikiutil.join_wiki(wikiurl, wikitail)
+                # Only add additional class information if the reference does
+                # not have a child image (don't want to add additional markup
+                # for images with targets).
+                if not [i for i in node.children if i.__class__ == docutils.nodes.image]:
+                    node['classes'].append('interwiki')
+            elif prefix != '':
+                # Some link scheme (http, file, https, mailto, etc.), add class
+                # information if the reference doesn't have a child image (don't 
+                # want additional markup for images with targets). 
+                # Don't touch the refuri.
+                if not [i for i in node.children if i.__class__ == docutils.nodes.image]:
+                    node['classes'].append(prefix)
+            else:
+                # Default case - make a link to a wiki page.
+                page = MoinMoin.Page.Page(self.request, refuri)
+                node['refuri'] = page.url(self.request)
+                if not page.exists():
+                    node['classes'].append('nonexistent')
         html4css1.HTMLTranslator.visit_reference(self, node)
 
     def visit_image(self, node):
@@ -344,15 +386,28 @@ class MoinTranslator(html4css1.HTMLTranslator):
             TODO: Need to handle figures similarly.
         """
         uri = node['uri'].lstrip()
-        prefix = ''       # assume no prefix
+        prefix = ''          # assume no prefix
+        attach_name = uri
         if ':' in uri:
-            prefix = uri.split(':',1)[0]
+            prefix = uri.split(':', 1)[0]
+            attach_name = uri.split(':', 1)[1]
         # if prefix isn't URL, try to display in page
         if not prefix.lower() in ('file', 'http', 'https', 'ftp'):
-            # no prefix given, so fake "inline:"
-            if not prefix:
-                node['uri'] = 'inline:' + uri
-            self.process_inline(node, 'uri')
+            attach_file = AttachFile.getFilename(self.request, 
+                    self.request.page.page_name, attach_name)
+            if not os.path.exists(attach_file):
+                # Attachment doesn't exist, MoinMoin should process it
+                if prefix == '':
+                    prefix = 'inline:'
+                self.process_wiki_text(prefix + attach_name)
+                self.wiki_text = self.fixup_wiki_formatting(self.wiki_text)
+                self.add_wiki_markup()
+            # Attachment exists, get a link to it.
+            # create the url
+            node['uri'] = AttachFile.getAttachUrl(self.request.page.page_name, 
+                    attach_name, self.request, addts = 1)
+            if not node.hasattr('alt'):
+                node['alt'] = node.get('name', uri)
         html4css1.HTMLTranslator.visit_image(self, node)
 
     def create_wiki_functor(self, moin_func):
@@ -386,12 +441,15 @@ class MoinTranslator(html4css1.HTMLTranslator):
             # Blocks
             'literal_block': 'preformatted',
             # Simple Lists
-            'bullet_list': 'bullet_list',
+            # bullet-lists are handled completely by docutils because it uses
+            # the node context to decide when to make a compact list 
+            # (no <p> tags).
             'list_item': 'listitem',
             # Definition List
             'definition_list': 'definition_list',
             # Admonitions
-            'warning': 'highlight'}
+            'warning': 'highlight'
+        }
         for rest_func, moin_func in handlers.items():
             visit_func, depart_func = self.create_wiki_functor(moin_func)
             visit_func = new.instancemethod(visit_func, self, MoinTranslator)
@@ -495,3 +553,5 @@ class MoinDirectives:
 
     macro.content = True
 
+if ErrorParser: # fixup in case of missing docutils
+    Parser = ErrorParser
