@@ -32,16 +32,16 @@ class Parser:
     PARENT_PREFIX = wikiutil.PARENT_PREFIX
     attachment_schemas = ["attachment", "inline", "drawing"]
     punct_pattern = re.escape(u'''"\'}]|:,.)?!''')
-    url_pattern = (u'http|https|ftp|nntp|news|mailto|telnet|wiki|file|' +
+    url_pattern = (u'http|https|ftp|nntp|news|mailto|telnet|wiki|file|irc|' +
             u'|'.join(attachment_schemas) + 
             (config.url_schemas and u'|' + u'|'.join(config.url_schemas) or ''))
 
     # some common rules
-    word_rule = ur'(?:(?<![%(l)s])|^)%(parent)s(?:%(subpages)s(?:[%(u)s][%(l)s]+){2,})+(?![%(u)s%(l)s]+)' % {
+    word_rule = ur'(?:(?<![%(u)s%(l)s])|^)%(parent)s(?:%(subpages)s(?:[%(u)s][%(l)s]+){2,})+(?![%(u)s%(l)s]+)' % {
         'u': config.chars_upper,
         'l': config.chars_lower,
-        'subpages': config.allow_subpages and (wikiutil.CHILD_PREFIX + '?') or '',
-        'parent': config.allow_subpages and (ur'(?:%s)?' % re.escape(PARENT_PREFIX)) or '',
+        'subpages': wikiutil.CHILD_PREFIX + '?',
+        'parent': ur'(?:%s)?' % re.escape(PARENT_PREFIX),
     }
     url_rule = ur'%(url_guard)s(%(url)s)\:([^\s\<%(punct)s]|([%(punct)s][^\s\<%(punct)s]))+' % {
         'url_guard': u'(^|(?<!\w))',
@@ -53,7 +53,8 @@ class Parser:
     dl_rule = ur"^\s+.*?::\s"
 
     # the big, fat, ugly one ;)
-    formatting_rules = ur"""(?:(?P<emph_ibb>'''''(?=[^']+'''))
+    formatting_rules = ur"""(?P<ent_numeric>&#(\d{1,5}|x[0-9a-fA-F]+);)
+(?:(?P<emph_ibb>'''''(?=[^']+'''))
 (?P<emph_ibi>'''''(?=[^']+''))
 (?P<emph_ib_or_bi>'{5}(?=[^']))
 (?P<emph>'{2,3})
@@ -65,12 +66,15 @@ class Parser:
 (?P<pre>(\{\{\{ ?|\}\}\}))
 (?P<small>(\~- ?|-\~))
 (?P<big>(\~\+ ?|\+\~))
+(?P<strike>(--\(|\)--))
 (?P<rule>-{4,})
 (?P<comment>^\#\#.*$)
 (?P<macro>\[\[(%%(macronames)s)(?:\(.*?\))?\]\]))
 (?P<ol>%(ol_rule)s)
 (?P<dl>%(dl_rule)s)
-(?P<li>^\s+\*?)
+(?P<li>^\s+\*\s*)
+(?P<li_none>^\s+\.\s*)
+(?P<indent>^\s+)
 (?P<tableZ>\|\| $)
 (?P<table>(?:\|\|)+(?:<[^>]*?>)?(?!\|? $))
 (?P<heading>^\s*(?P<hmarker>=+)\s.*\s(?P=hmarker) $)
@@ -81,7 +85,11 @@ class Parser:
 (?P<email>[-\w._+]+\@[\w-]+(\.[\w-]+)+)
 (?P<smiley>(?<=\s)(%(smiley)s)(?=\s))
 (?P<smileyA>^(%(smiley)s)(?=\s))
-(?P<ent>[<>&])"""  % {
+(?P<ent_symbolic>&[a-zA-Z]+;)
+(?P<ent>[<>&])
+(?P<wikiname_bracket>\[".*?"\])
+(?P<tt_bt>`.*?`)"""  % {
+
         'url': url_pattern,
         'punct': punct_pattern,
         'ol_rule': ol_rule,
@@ -91,9 +99,11 @@ class Parser:
         'smiley': u'|'.join(map(re.escape, config.smileys.keys()))}
 
     # Don't start p before these 
-    no_new_p_before = ("heading rule table tableZ tr td ul ol dl dt dd li "
-                       "processor macro pre")
-    no_new_p_before = dict(zip(no_new_p_before.split(), [1] * len(no_new_p_before)))
+    no_new_p_before = ("heading rule table tableZ tr td "
+                       "ul ol dl dt dd li li_none indent "
+                       "macro processor pre")
+    no_new_p_before = no_new_p_before.split()
+    no_new_p_before = dict(zip(no_new_p_before, [1] * len(no_new_p_before)))
 
     def __init__(self, raw, request, **kw):
         self.raw = raw
@@ -101,12 +111,14 @@ class Parser:
         self.form = request.form
         self._ = request.getText
         self.cfg = request.cfg
-
+        self.line_anchors = kw.get('line_anchors', True)
         self.macro = None
+        self.start_line = kw.get('start_line', 0)
 
         self.is_em = 0
         self.is_b = 0
         self.is_u = 0
+        self.is_strike = 0
         self.lineno = 0
         self.in_li = 0
         self.in_dd = 0
@@ -149,19 +161,26 @@ class Parser:
         else:
             url, text = url_and_text
 
-        url = url[5:] # remove "wiki:"
+        # keep track of whether this is a self-reference, so links
+        # are always shown even the page doesn't exist.
+        is_self_reference = 0
+        url2 = url.lower()
+        if url2.startswith('wiki:self:'):
+            url = url[10:] # remove "wiki:self:"
+            is_self_reference = 1
+        elif url2.startswith('wiki:'):
+            url = url[5:] # remove "wiki:"
+           
+        tag, tail = wikiutil.split_wiki(url)
         if text is None:
-            tag, tail = wikiutil.split_wiki(url)
             if tag:
                 text = tail
             else:
                 text = url
                 url = ""
-        elif config.allow_subpages and url.startswith(wikiutil.CHILD_PREFIX):
-            # fancy link to subpage [wiki:/SubPage text]
-            return self._word_repl(url, text)
-        elif Page(self.request, url).exists():
-            # fancy link to local page [wiki:LocalPage text]
+        elif (url.startswith(wikiutil.CHILD_PREFIX) or # fancy link to subpage [wiki:/SubPage text]
+              is_self_reference or # [wiki:Self:LocalPage text] or [:LocalPage:text]
+              Page(self.request, url).exists()): # fancy link to local page [wiki:LocalPage text]
             return self._word_repl(url, text)
 
         wikitag, wikiurl, wikitail, wikitag_bad = wikiutil.resolve_wiki(self.request, url)
@@ -175,14 +194,13 @@ class Parser:
         if wikitag is None:
             return self._word_repl(wikitail)
               
-        return (self.formatter.interwikilink(1, wikitag, wikitail) + 
+        return (self.formatter.interwikilink(1, tag, tail) + 
                 self.formatter.text(text) +
                 self.formatter.interwikilink(0))
 
     def attachment(self, url_and_text, **kw):
         """ This gets called on attachment URLs.
         """
-        import urllib
         _ = self._
         if len(url_and_text) == 1:
             url = url_and_text[0]
@@ -193,123 +211,50 @@ class Parser:
         inline = url[0] == 'i'
         drawing = url[0] == 'd'
         url = url.split(":", 1)[1]
-        url = urllib.unquote(url)
+        url = wikiutil.url_unquote(url, want_unicode=True)
         text = text or url
 
-        pagename = self.formatter.page.page_name
-        parts = url.split('/')
-        if len(parts) > 1:
-            # get attachment from other page
-            pagename = '/'.join(parts[:-1])
-            url = parts[-1]
-
-        import urllib
         from MoinMoin.action import AttachFile
-        fname = wikiutil.taintfilename(url)
         if drawing:
-            drawing = fname
-            fname = fname + ".png"
-            url = url + ".png"
-            # fallback for old gif drawings (1.1 -> 1.2)
-            fpath = AttachFile.getFilename(self.request, pagename, fname)
-            if not os.path.exists(fpath):
-                gfname = fname[:-4] + ".gif"
-                gurl = url[:-4] + ".gif"
-                gfpath = AttachFile.getFilename(self.request, pagename, gfname)
-                if os.path.exists(gfpath):
-                    fname, url, fpath = gfname, gurl, gfpath
-        else:
-            fpath = AttachFile.getFilename(self.request, pagename, fname)
-
-        # check whether attachment exists, possibly point to upload form
-        if not os.path.exists(fpath):
-            if drawing:
-                linktext = _('Create new drawing "%(filename)s"')
-            else:
-                linktext = _('Upload new attachment "%(filename)s"')
-            return wikiutil.link_tag(self.request,
-                self.formatter.text('%s?action=AttachFile&rename=%s%s' % (
-                    wikiutil.quoteWikinameURL(pagename),
-                    urllib.quote_plus(fname.encode(config.charset)),
-                    drawing and ('&drawing=%s' % urllib.quote(drawing.encode(config.charset))) or '')),
-                linktext % {'filename': self.formatter.text(fname)})
+            return self.formatter.attachment_drawing(url, text)
 
         # check for image URL, and possibly return IMG tag
         # (images are always inlined, just like for other URLs)
         if not kw.get('pretty_url', 0) and wikiutil.isPicture(url):
-            if drawing:
-                # check for map file
-                mappath = AttachFile.getFilename(self.request, pagename, drawing + '.map')
-                edit_link = self.formatter.text('%s?action=AttachFile&rename=%s&drawing=%s' % (wikiutil.quoteWikinameURL(pagename), urllib.quote_plus(fname.encode(config.charset)), urllib.quote(drawing.encode(config.charset))))
-                if os.path.exists(mappath):
-                    # we have a image map. inline it and add a map ref
-                    # to the img tag
-                    try:
-                        map = open(mappath,'r').read()
-                    except IOError:
-                        pass
-                    except OSError:
-                        pass
-                    else:
-                        mapid = 'ImageMapOf'+drawing
-                        # replace MAPNAME
-                        map = map.replace('%MAPNAME%', mapid)
-                        # add alt and title tags to areas
-                        map = re.sub('href\s*=\s*"((?!%TWIKIDRAW%).+?)"',r'href="\1" alt="\1" title="\1"',map)
-                        # add in edit links plus alt and title attributes
-                        map = map.replace('%TWIKIDRAW%"', edit_link + '" alt="' + _('Edit drawing %(filename)s') % {'filename': self.formatter.text(fname)} + '" title="' + _('Edit drawing %(filename)s') % {'filename': self.formatter.text(fname)} + '"')
-                        # unxml, because 4.01 concrete will not validate />
-                        map = map.replace('/>','>')
-                        return map + self.formatter.image(alt=drawing,
-                                src=AttachFile.getAttachUrl(pagename, url, self.request, addts=1), usemap='#'+mapid, html_class="drawing")
-                else:
-                    return wikiutil.link_tag(self.request,
-                        edit_link,
-                        self.formatter.image(alt=url,
-                            src=AttachFile.getAttachUrl(pagename, url, self.request, addts=1), html_class="drawing"),
-                        attrs='title="%s"' % (_('Edit drawing %(filename)s') % {'filename': self.formatter.text(fname)}))
-            else:
-                return self.formatter.image(alt=url,
-                    src=AttachFile.getAttachUrl(pagename, url, self.request, addts=1))
-
-        # try to inline the attachment (parser know what they
-        # can handle)
-        base, ext = os.path.splitext(url)
+            return self.formatter.attachment_image(url)
+                
+        # inline the attachment
         if inline:
-            Parser = wikiutil.getParserForExtension(self.cfg, ext)
-            if Parser is not None:
-                content = file(fpath, 'r').read()
-                # Try to decode text. It might return junk, but we don't
-                # have enough information with attachments.
-                content = wikiutil.decodeUnknownInput(content)
-                colorizer = Parser(content, self.request)
-                colorizer.format(self.formatter)
+            return self.formatter.attachment_inlined(url, text)
 
-        url = AttachFile.getAttachUrl(pagename, url, self.request)
-
-        if not kw.get('pretty_url', 0) and wikiutil.isPicture(url):
-            return self.formatter.image(src=url)
-        else:
-            return (self.formatter.url(1, url) +
-                    self.formatter.text(text) +
-                    self.formatter.url(0))
+        return self.formatter.attachment_link(url, text)
 
     def _u_repl(self, word):
         """Handle underline."""
         self.is_u = not self.is_u
         return self.formatter.underline(self.is_u)
 
+    def _strike_repl(self, word):
+        """Handle strikethrough."""
+        # XXX we don't really enforce the correct sequence --( ... )-- here
+        self.is_strike = not self.is_strike
+        return self.formatter.strike(self.is_strike)
+
     def _small_repl(self, word):
         """Handle small."""
-        if word.strip() == '~-' and self.is_small: return word
-        if word.strip() == '-~' and not self.is_small: return word
+        if word.strip() == '~-' and self.is_small:
+            return self.formatter.text(word)
+        if word.strip() == '-~' and not self.is_small:
+            return self.formatter.text(word)
         self.is_small = not self.is_small
         return self.formatter.small(self.is_small)
 
     def _big_repl(self, word):
         """Handle big."""
-        if word.strip() == '~+' and self.is_big: return word
-        if word.strip() == '+~' and not self.is_big: return word
+        if word.strip() == '~+' and self.is_big:
+            return self.formatter.text(word)
+        if word.strip() == '+~' and not self.is_big:
+            return self.formatter.text(word)
         self.is_big = not self.is_big
         return self.formatter.big(self.is_big)
 
@@ -318,25 +263,29 @@ class Parser:
         ##print "#", self.is_b, self.is_em, "#"
         if len(word) == 3:
             self.is_b = not self.is_b
-            if self.is_em and self.is_b: self.is_b = 2
+            if self.is_em and self.is_b:
+                self.is_b = 2
             return self.formatter.strong(self.is_b)
         else:
             self.is_em = not self.is_em
-            if self.is_em and self.is_b: self.is_em = 2
+            if self.is_em and self.is_b:
+                self.is_em = 2
             return self.formatter.emphasis(self.is_em)
 
     def _emph_ibb_repl(self, word):
         """Handle mixed emphasis, i.e. ''''' followed by '''."""
         self.is_b = not self.is_b
         self.is_em = not self.is_em
-        if self.is_em and self.is_b: self.is_b = 2
+        if self.is_em and self.is_b:
+            self.is_b = 2
         return self.formatter.emphasis(self.is_em) + self.formatter.strong(self.is_b)
 
     def _emph_ibi_repl(self, word):
         """Handle mixed emphasis, i.e. ''''' followed by ''."""
         self.is_b = not self.is_b
         self.is_em = not self.is_em
-        if self.is_em and self.is_b: self.is_em = 2
+        if self.is_em and self.is_b:
+            self.is_em = 2
         return self.formatter.strong(self.is_b) + self.formatter.emphasis(self.is_em)
 
     def _emph_ib_or_bi_repl(self, word):
@@ -357,7 +306,6 @@ class Parser:
             self.formatter.text(word[1:-1]) + \
             self.formatter.sup(0)
 
-
     def _sub_repl(self, word):
         """Handle subscript."""
         return self.formatter.sub(1) + \
@@ -384,8 +332,9 @@ class Parser:
         # check for parent links
         # !!! should use wikiutil.AbsPageName here, but setting `text`
         # correctly prevents us from doing this for now
-        if config.allow_subpages and word.startswith(wikiutil.PARENT_PREFIX):
-            if not text: text = word
+        if word.startswith(wikiutil.PARENT_PREFIX):
+            if not text:
+                text = word
             word = '/'.join(filter(None, self.formatter.page.page_name.split('/')[:-1] + [word[wikiutil.PARENT_PREFIX_LEN:]]))
 
         if not text:
@@ -393,36 +342,49 @@ class Parser:
             if word == self.formatter.page.page_name:
                 return self.formatter.text(word)
             text = word
-        if config.allow_subpages and word.startswith(wikiutil.CHILD_PREFIX):
+        if word.startswith(wikiutil.CHILD_PREFIX):
             word = self.formatter.page.page_name + '/' + word[wikiutil.CHILD_PREFIX_LEN:]
-        return (self.formatter.pagelink(1, word) +
+
+        # handle anchors
+        parts = word.split("#", 1)
+        anchor = ""
+        if len(parts)==2:
+            word, anchor = parts
+            
+        return (self.formatter.pagelink(1, word, anchor=anchor) +
                 self.formatter.text(text) +
                 self.formatter.pagelink(0, word))
 
     def _notword_repl(self, word):
         """Handle !NotWikiNames."""
-        return self.formatter.text(word[1:])
+        return self.formatter.nowikiword(word[1:])
 
     def _interwiki_repl(self, word):
         """Handle InterWiki links."""
-        return self.interwiki(["wiki:" + word])
+        wikitag, wikiurl, wikitail, wikitag_bad = wikiutil.resolve_wiki(self.request, word)
+        if wikitag_bad:
+            return self.formatter.text(word)
+        else:
+            return self.interwiki(["wiki:" + word])
 
 
     def _url_repl(self, word):
         """Handle literal URLs including inline images."""
         scheme = word.split(":", 1)[0]
 
-        if scheme == "wiki": return self.interwiki([word])
+        if scheme == "wiki":
+            return self.interwiki([word])
         if scheme in self.attachment_schemas:
             return self.attachment([word])
 
         if wikiutil.isPicture(word):
+            word = wikiutil.mapURL(self.request, word)
             # Get image name http://here.com/dir/image.gif -> image
             name = word.split('/')[-1]
             name = ''.join(name.split('.')[:-1])
             return self.formatter.image(src=word, alt=name)
         else:
-            return (self.formatter.url(1, word, type='www') +
+            return (self.formatter.url(1, word, css=scheme) +
                     self.formatter.text(word) +
                     self.formatter.url(0))
 
@@ -433,7 +395,7 @@ class Parser:
         if wikiname:
             return self._word_repl(wikiname)
         else:
-            return word
+            return self.formatter.text(word)
 
 
     def _url_bracket_repl(self, word):
@@ -450,7 +412,8 @@ class Parser:
 
         # Traditional split on space
         words = word[1:-1].split(None, 1)
-        if len(words) == 1: words = words * 2
+        if len(words) == 1:
+            words = words * 2
 
         if words[0][0] == '#':
             # anchor link
@@ -459,24 +422,24 @@ class Parser:
                     self.formatter.url(0))
 
         scheme = words[0].split(":", 1)[0]
-        if scheme == "wiki": return self.interwiki(words, pretty_url=1)
+        if scheme == "wiki":
+            return self.interwiki(words, pretty_url=1)
         if scheme in self.attachment_schemas:
             return self.attachment(words, pretty_url=1)
 
         if wikiutil.isPicture(words[1]) and re.match(self.url_rule, words[1]):
-            return (self.formatter.url(1, words[0], 'external', unescaped=1) +
+            return (self.formatter.url(1, words[0], css='external', do_escape=0) +
                     self.formatter.image(title=words[0], alt=words[0], src=words[1]) +
                     self.formatter.url(0))
         else:
-            return (self.formatter.url(1, words[0], 'external',
-                                       type='www', unescaped=1) +
+            return (self.formatter.url(1, words[0], css=scheme, do_escape=0) +
                     self.formatter.text(words[1]) +
                     self.formatter.url(0))
 
 
     def _email_repl(self, word):
         """Handle email addresses (without a leading mailto:)."""
-        return (self.formatter.url(1, "mailto:" + word, type='mailto') +
+        return (self.formatter.url(1, "mailto:" + word, css='mailto') +
                 self.formatter.text(word) +
                 self.formatter.url(0))
 
@@ -488,33 +451,47 @@ class Parser:
         #        '<': '&lt;',
         #        '>': '&gt;'}[word]
 
-
     def _ent_numeric_repl(self, word):
-        """Handle numeric SGML entities."""
+        """Handle numeric (decimal and hexadecimal) SGML entities."""
         return self.formatter.rawHTML(word)
 
-
-    def _li_repl(self, match):
-        """Handle bullet lists."""
+    def _ent_symbolic_repl(self, word):
+        """Handle symbolic SGML entities."""
+        return self.formatter.rawHTML(word)
+    
+    def _indent_repl(self, match):
+        """Handle pure indentation (no - * 1. markup)."""
         result = []
-        indented_only = (match == (" " * len(match)))
-        if indented_only and self.in_li: return ''
-            
-        self._close_item(result)
-        #self.inhibit_p = 1
-        self.in_li = 1
-        css_class = ''
-        if self.line_was_empty and not self.first_list_item:
-            css_class = 'gap'
-        if indented_only:
-            result.append(self.formatter.listitem(1, css_class=css_class,
-                                             style="list-style-type:none"))
-        else:
-            result.append(self.formatter.listitem(1, css_class=css_class))
-        # Suspected p!
-        ## result.append(self.formatter.paragraph(1))
+        if not self.in_li:
+            self._close_item(result)
+            self.in_li = 1
+            css_class = None
+            if self.line_was_empty and not self.first_list_item:
+                css_class = 'gap'
+            result.append(self.formatter.listitem(1, css_class=css_class, style="list-style-type:none"))
         return ''.join(result)
 
+    def _li_none_repl(self, match):
+        """Handle type=none (" .") lists."""
+        result = []
+        self._close_item(result)
+        self.in_li = 1
+        css_class = None
+        if self.line_was_empty and not self.first_list_item:
+            css_class = 'gap'
+        result.append(self.formatter.listitem(1, css_class=css_class, style="list-style-type:none"))
+        return ''.join(result)
+
+    def _li_repl(self, match):
+        """Handle bullet (" *") lists."""
+        result = []
+        self._close_item(result)
+        self.in_li = 1
+        css_class = None
+        if self.line_was_empty and not self.first_list_item:
+            css_class = 'gap'
+        result.append(self.formatter.listitem(1, css_class=css_class))
+        return ''.join(result)
 
     def _ol_repl(self, match):
         """Handle numbered lists."""
@@ -529,7 +506,7 @@ class Parser:
         self.in_dd = 1
         result.extend([
             self.formatter.definition_term(1),
-            self.formatter.text(match[1:-3]),
+            self.formatter.text(match[1:-3].lstrip(' ')),
             self.formatter.definition_term(0),
             self.formatter.definition_desc(1),
             ## CHANGE: no automatic paragraph
@@ -548,7 +525,6 @@ class Parser:
         open = []   # don't make one out of these two statements!
         close = []
 
-
         if self._indent_level() != new_level and self.in_table:
             close.append(self.formatter.table(0))
             self.in_table = 0
@@ -558,10 +534,11 @@ class Parser:
         #        self.inhibit_p = 1
     
         # Close lists while char-wise indent is greater than the current one
-        while ((self._indent_level() > new_level) or
-               ( new_level and
-                (self._indent_level() == new_level) and
-                (self.list_types[-1]) != list_type)):
+        #while ((self._indent_level() > new_level) or
+        #       ( new_level and
+        #        (self._indent_level() == new_level) and
+        #        (self.list_types[-1]) != list_type)):
+        while self._indent_level() > new_level:
             self._close_item(close)
             if self.list_types[-1] == 'ol':
                 tag = self.formatter.number_list(0)
@@ -587,7 +564,6 @@ class Parser:
                 
         # Open new list, if necessary
         if self._indent_level() < new_level:
-                    
             self.list_indents.append(new_level)
             self.list_types.append(list_type)
 
@@ -607,6 +583,7 @@ class Parser:
             ##self.inhibit_p = 1
             self.in_li = 0
             self.in_dd = 0
+            
         # If list level changes, close an open table
         if self.in_table and (open or close):
             close[0:0] = [self.formatter.table(0)]
@@ -623,7 +600,7 @@ class Parser:
         result = []
         #result.append("<!-- _undent start -->\n")
         self._close_item(result)
-        for type in self.list_types:
+        for type in self.list_types[::-1]:
             if type == 'ol':
                 result.append(self.formatter.number_list(0))
             elif type == 'dl':
@@ -645,8 +622,8 @@ class Parser:
 
     def _tt_bt_repl(self, word):
         """Handle backticked inline code."""
-        if len(word) == 2: return ""
-        return self.formatter.code(1) + \
+        # if len(word) == 2: return "" // removed for FCK editor
+        return self.formatter.code(1, css="backtick") + \
             self.formatter.text(word[1:-1]) + \
             self.formatter.code(0)
 
@@ -718,7 +695,7 @@ class Parser:
             else:
                 msg = None
             #print "key: %s\nattrs: %s" % (key, str(attrs))
-            return msg
+            return self.formatter.rawHTML(msg)
 
         # scan attributes
         attr, msg = wikiutil.parseAttributes(self.request, attrdef, '>', table_extension)
@@ -736,7 +713,7 @@ class Parser:
             result += self.formatter.table_cell(0) + self.formatter.table_row(0)
             return result
         else:
-            return word
+            return self.formatter.text(word)
 
     def _table_repl(self, word):
         """Handle table cell separator."""
@@ -767,7 +744,7 @@ class Parser:
             result.append(self.formatter.table_cell(1, attrs) + attrerr)         
             return ''.join(result) 
         else:
-            return word
+            return self.formatter.text(word)
 
 
     def _heading_repl(self, word):
@@ -779,7 +756,7 @@ class Parser:
         h = word.strip()
         level = 1
         while h[level:level+1] == '=':
-            level = level+1
+            level += 1
         depth = min(5,level)
 
         # this is needed for Included pages
@@ -801,7 +778,8 @@ class Parser:
     
     def _processor_repl(self, word):
         """Handle processed code displays."""
-        if word[:3] == '{{{': word = word[3:]
+        if word[:3] == '{{{':
+            word = word[3:]
 
         self.processor = None
         self.processor_name = None
@@ -814,16 +792,9 @@ class Parser:
             self.in_pre = 3
             return self._closeP() + self.formatter.preformatted(1)
         elif s_word[:2] == '#!':
-            # first try to find a processor for this (will go away in 1.4)
+            # First try to find a processor for this (will go away in 2.0)
             processor_name = s_word[2:].split()[0]
-            self.processor = wikiutil.importPlugin(
-                self.request.cfg, "processor", processor_name, "process")
-            # now look for a parser with that name
-            if self.processor is None:
-                self.processor = wikiutil.importPlugin(
-                    self.request.cfg, "parser", processor_name, "Parser")
-                if self.processor:
-                    self.processor_is_parser = 1
+            self.setProcessor(processor_name)
 
         if self.processor:
             self.processor_name = processor_name
@@ -849,7 +820,7 @@ class Parser:
             self.in_pre = 0
             self.inhibit_p = 0
             return self.formatter.preformatted(self.in_pre)
-        return word
+        return self.formatter.text(word)
 
 
     def _smiley_repl(self, word):
@@ -860,7 +831,7 @@ class Parser:
 
 
     def _comment_repl(self, word):
-        return ''
+        return self.formatter.comment(word)
 
     def _closeP(self):
         if self.formatter.in_p:
@@ -870,7 +841,7 @@ class Parser:
     def _macro_repl(self, word):
         """Handle macros ([[macroname]])."""
         macro_name = word[2:-2]
-        #self.inhibit_p = 1 # fixes UserPreferences, but makes new trouble!
+        self.inhibit_p = 1 # 0 fixes UserPreferences, but makes new trouble!
 
         # check for arguments
         args = None
@@ -901,7 +872,7 @@ class Parser:
                 ###result.append(u'<span class="info">[add text before match: <tt>"%s"</tt>]</span>' % line[lastpos:match.start()])
                 
                 if not (self.inhibit_p or self.in_pre or self.formatter.in_p):
-                    result.append(self.formatter.paragraph(1))
+                    result.append(self.formatter.paragraph(1, css_class="line879"))
                 result.append(self.formatter.text(line[lastpos:match.start()]))
             
             # Replace match with markup
@@ -913,7 +884,7 @@ class Parser:
         # No match: Add paragraph with the text of the line
         if not (self.in_pre or self.inhibit_p or
                 self.formatter.in_p) and lastpos < len(line):
-            result.append(self.formatter.paragraph(1))
+            result.append(self.formatter.paragraph(1, css_class="line886"))
         result.append(self.formatter.text(line[lastpos:]))
         return u''.join(result)
 
@@ -930,7 +901,7 @@ class Parser:
                     # Open p for certain types
                     if not (self.inhibit_p or self.formatter.in_p
                             or self.in_pre or (type in self.no_new_p_before)):
-                        result.append(self.formatter.paragraph(1))
+                        result.append(self.formatter.paragraph(1, css_class="line903"))
                     
                     # Get replace method and replece hit
                     replace = getattr(self, '_' + type + '_repl')
@@ -954,18 +925,11 @@ class Parser:
 
         # prepare regex patterns
         rules = self.formatting_rules.replace('\n', '|')
-        if self.cfg.allow_extended_names:
-            rules = rules + ur'|(?P<wikiname_bracket>\[".*?"\])'
         if self.cfg.bang_meta:
             rules = ur'(?P<notword>!%(word_rule)s)|%(rules)s' % {
                 'word_rule': self.word_rule,
                 'rules': rules,
             }
-        if self.cfg.backtick_meta:
-            rules = rules + ur'|(?P<tt_bt>`.*?`)'
-        if self.cfg.allow_numeric_entities:
-            rules = ur'(?P<ent_numeric>&#\d{1,5};)|' + rules
-
         self.request.clock.start('compile_huge_and_ugly')        
         scan_re = re.compile(rules, re.UNICODE)
         number_re = re.compile(self.ol_rule, re.UNICODE)
@@ -978,19 +942,36 @@ class Parser:
         rawtext = self.raw.expandtabs()
 
         # go through the lines
-        self.lineno = 0
+        self.lineno = self.start_line
         self.lines = eol_re.split(rawtext)
         self.line_is_empty = 0
 
+        self.in_processing_instructions = 1
+
         # Main loop
         for line in self.lines:
-            self.lineno = self.lineno + 1
+            self.lineno += 1
+            if self.line_anchors:
+                self.request.write(self.formatter.line_anchordef(self.lineno))
             self.table_rowstart = 1
             self.line_was_empty = self.line_is_empty
             self.line_is_empty = 0
             self.first_list_item = 0
             self.inhibit_p = 0
 
+            # ignore processing instructions
+            if self.in_processing_instructions:
+                found = False
+                for pi in ("##", "#format", "#refresh", "#redirect", "#deprecated",
+                           "#pragma", "#form", "#acl", "#language"):
+                    if line.lower().startswith(pi):
+                        self.request.write(self.formatter.comment(line))
+                        found = True
+                        break
+                if not found:
+                    self.in_processing_instructions = 0
+                else:
+                    continue # do not parse this line
             if self.in_pre:
                 # TODO: move this into function
                 # still looking for processing instructions
@@ -1001,15 +982,8 @@ class Parser:
                     processor_name = ''
                     if (line.strip()[:2] == "#!"):
                         processor_name = line.strip()[2:].split()[0]
-                        self.processor = wikiutil.importPlugin(
-                            self.request.cfg, "processor", processor_name, "process")
-                                                               
-                        # now look for a parser with that name
-                        if self.processor is None:
-                            self.processor = wikiutil.importPlugin(
-                                self.request.cfg, "parser", processor_name, "Parser") 
-                            if self.processor:
-                                self.processor_is_parser = 1
+                        self.setProcessor(processor_name)
+
                     if self.processor:
                         self.in_pre = 2
                         self.colorize_lines = [line]
@@ -1040,11 +1014,13 @@ class Parser:
                     self.processor = None
 
                     # send rest of line through regex machinery
-                    line = line[endpos+3:]                    
+                    line = line[endpos+3:]
+                    if not line.strip(): # just in the case "}}} " when we only have blanks left...
+                        continue
             else:
                 # we don't have \n as whitespace any more
                 # This is the space between lines we join to one paragraph
-                line = line + ' '
+                line += ' '
                 
                 # Paragraph break on empty lines
                 if not line.strip():
@@ -1053,7 +1029,7 @@ class Parser:
                         self.in_table = 0
                     # CHANGE: removed check for not self.list_types
                     # p should close on every empty line
-                    if (self.formatter.in_p):
+                    if self.formatter.in_p:
                         self.request.write(self.formatter.paragraph(0))
                     self.line_is_empty = 1
                     continue
@@ -1082,8 +1058,7 @@ class Parser:
                             indtype = "dl"
 
                 # output proper indentation tags
-                self.request.write(self._indent_to(indlen, indtype, numtype,
-                                                   numstart))
+                self.request.write(self._indent_to(indlen, indtype, numtype, numstart))
 
                 # Table mode
                 # TODO: move into function?                
@@ -1091,8 +1066,7 @@ class Parser:
                     and line[-3:] == "|| " and len(line) >= 5 + indlen):
                     # Start table
                     if self.list_types and not self.in_li:
-                        self.request.write(self.formatter.listitem
-                                           (1, style="list-style-type:none"))
+                        self.request.write(self.formatter.listitem(1, style="list-style-type:none"))
                         ## CHANGE: no automatic p on li
                         ##self.request.write(self.formatter.paragraph(1))
                         self.in_li = 1
@@ -1106,7 +1080,7 @@ class Parser:
                     self.in_table = True # self.lineno
                 elif (self.in_table and not
                       # intra-table comments should not break a table
-                      (line[:2]=="##" or  
+                      (line[:2] == "##" or  
                        line[indlen:indlen + 2] == "||" and
                        line[-3:] == "|| " and
                        len(line) >= 5 + indlen)):
@@ -1119,7 +1093,7 @@ class Parser:
             formatted_line = self.scan(scan_re, line)
             self.request.write(formatted_line)
 
-            if self.in_pre:
+            if self.in_pre == 3:
                 self.request.write(self.formatter.linebreak())
 
         # Close code displays, paragraphs, tables and open lists
@@ -1128,4 +1102,22 @@ class Parser:
         if self.formatter.in_p: self.request.write(self.formatter.paragraph(0))
         if self.in_table: self.request.write(self.formatter.table(0))
 
+    # --------------------------------------------------------------------
+    # Private helpers
+    
+    def setProcessor(self, name):
+        """ Set processer to either processor or parser named 'name' """
+        cfg = self.request.cfg
+        try:
+            self.processor = wikiutil.importPlugin(cfg, "processor", name,
+                                                   "process")
+            self.processor_is_parser = 0
+        except wikiutil.PluginMissingError:
+            try:
+                self.processor = wikiutil.importPlugin(cfg, "parser", name,
+                                                   "Parser")
+                self.processor_is_parser = 1
+            except wikiutil.PluginMissingError:
+                self.processor = None
+
 
