@@ -1,19 +1,26 @@
 #!/usr/bin/env python
 # -*- coding: iso-8859-1 -*-
 """
-    MoinMoin - Dump a MoinMoin wiki to static pages
+MoinMoin - Dump a MoinMoin wiki to static pages
 
-    @copyright: 2002-2004 by Jrgen Hermann <jh@web.de>
-    @license: GNU GPL, see COPYING for details.
+You must run this script as owner of the wiki files, usually this is the
+web server user.
+
+@copyright: 2002-2004 by Jrgen Hermann <jh@web.de>
+@copyright: 2005 Thomas Waldmann
+@license: GNU GPL, see COPYING for details.
 """
 
-__version__ = "20040329"
+import sys, os, time, StringIO, codecs, shutil, re, errno
 
-# use this if your moin installation is not in sys.path:
-import sys
-sys.path.insert(0, '../..') # path to MoinMoin
+# Insert the path to MoinMoin in the start of the path
+sys.path.insert(0, os.path.join(os.path.dirname(sys.argv[0]), 
+                                os.pardir, os.pardir))
 
-logo_html = '<img src="moinmoin.png">'
+from MoinMoin import config, wikiutil, Page
+from MoinMoin.scripts import _util
+from MoinMoin.request import RequestCLI
+from MoinMoin.action import AttachFile
 
 url_prefix = "."
 HTML_SUFFIX = ".html"
@@ -39,35 +46,51 @@ page_template = u'''<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://ww
 </tr>
 </table>
 <hr>
+<div id="page">
+<h1 id="title">%(pagename)s</h1>
 %(pagehtml)s
+</div>
 <hr>
 %(timestamp)s
 </body>
 </html>
 '''
 
-import os, time, StringIO, codecs, shutil
-from MoinMoin import config, wikiutil, Page
-from MoinMoin.scripts import _util
-from MoinMoin.request import RequestCLI
+def _attachment(request, pagename, filename, outputdir):
+    source_dir = AttachFile.getAttachDir(request, pagename)
+    source_file = os.path.join(source_dir, filename)
+    dest_dir = os.path.join(outputdir, "attachments", wikiutil.quoteWikinameFS(pagename))
+    dest_file = os.path.join(dest_dir, filename)
+    dest_url = "attachments/%s/%s" % (wikiutil.quoteWikinameFS(pagename), filename)
+    if os.access(source_file, os.R_OK):
+        if not os.access(dest_dir, os.F_OK):
+            try:
+                os.makedirs(dest_dir)
+            except:
+                _util.fatal("Cannot create attachment directory '%s'" % dest_dir)
+        elif not os.path.isdir(dest_dir):
+            _util.fatal("'%s' is not a directory" % dest_dir)
+
+        shutil.copyfile(source_file, dest_file)
+        _util.log('Writing "%s"...' % dest_url)
+        return dest_url
+    else:
+        return ""
+  
 
 class MoinDump(_util.Script):
+    
     def __init__(self):
         _util.Script.__init__(self, __name__, "[options] <target-directory>")
-
-        # --config=DIR            
         self.parser.add_option(
-            "--config", metavar="DIR", dest="configdir",
-            help="Path to wikiconfig.py (or its directory)"
+            "--config-dir", metavar="DIR", dest="config_dir",
+            help=("Path to the directory containing the wiki "
+                  "configuration files. [default: current directory]")
         )
-
-        # --wiki=URL             
         self.parser.add_option(
-            "--wiki", metavar="WIKIURL", dest="wiki_url",
-            help="URL of wiki to dump (e.g. moinmaster.wikiwikiweb.de)"
+            "--wiki-url", metavar="WIKIURL", dest="wiki_url",
+            help="URL of wiki e.g. localhost/mywiki/ [default: CLI]"
         )
-        
-        # --page=NAME             
         self.parser.add_option(
             "--page", metavar="NAME", dest="page",
             help="Dump a single page (with possibly broken links)"
@@ -83,64 +106,65 @@ class MoinDump(_util.Script):
         # Prepare output directory
         outputdir = self.args[0]
         outputdir = os.path.abspath(outputdir)
-        if not os.path.isdir(outputdir):
-            try:
-                os.mkdir(outputdir)
-                _util.log("Created output directory '%s'!" % outputdir)
-            except OSError:
+        try:
+            os.mkdir(outputdir)
+            _util.log("Created output directory '%s'!" % outputdir)
+        except OSError, err:
+            if err.errno != errno.EEXIST:
                 _util.fatal("Cannot create output directory '%s'!" % outputdir)
 
-        # Load the configuration
-        configdir = self.options.configdir
-        if configdir:
-            if os.path.isfile(configdir): configdir = os.path.dirname(configdir)
-            if not os.path.isdir(configdir):
-                _util.fatal("Bad path given to --config parameter")
-            configdir = os.path.abspath(configdir)
-            sys.path[0:0] = [configdir]
-            os.chdir(configdir)
+        # Insert config dir or the current directory to the start of the path.
+        config_dir = self.options.config_dir
+        if config_dir and os.path.isfile(config_dir):
+            config_dir = os.path.dirname(config_dir)
+        if config_dir and not os.path.isdir(config_dir):
+            _util.fatal("bad path given to --config-dir option")
+        sys.path.insert(0, os.path.abspath(config_dir or os.curdir))
 
-        # Dump the wiki
-        request = RequestCLI(self.options.wiki_url)
-        request.form = request.args = request.setup_args()
+        # Create request 
+        if self.options.wiki_url:
+            request = RequestCLI(self.options.wiki_url)
+        else:
+            request = RequestCLI()
 
         # fix url_prefix so we get relative paths in output html
+        original_url_prefix = request.cfg.url_prefix
         request.cfg.url_prefix = url_prefix
 
         if self.options.page:
             pages = [self.options.page]
         else:
             # Get all existing pages in the wiki
-            pages = list(request.rootpage.getPageList(user=''))
-        pages.sort()
+            pages = request.rootpage.getPageList(user='')
+            pages.sort()
 
         wikiutil.quoteWikinameURL = lambda pagename, qfn=wikiutil.quoteWikinameFS: (qfn(pagename) + HTML_SUFFIX)
 
+        AttachFile.getAttachUrl = lambda pagename, filename, request, addts=0, escaped=0: (_attachment(request, pagename, filename, outputdir))
+
         errfile = os.path.join(outputdir, 'error.log')
         errlog = open(errfile, 'w')
         errcnt = 0
 
-        page_front_page = wikiutil.getSysPage(request, 'FrontPage').page_name
+        page_front_page = wikiutil.getSysPage(request, request.cfg.page_front_page).page_name
         page_title_index = wikiutil.getSysPage(request, 'TitleIndex').page_name
         page_word_index = wikiutil.getSysPage(request, 'WordIndex').page_name
         
         navibar_html = ''
         for p in [page_front_page, page_title_index, page_word_index]:
-            navibar_html += '&nbsp;[<a href="%s">%s</a>]' % (wikiutil.quoteWikinameFS(p), wikiutil.escape(p))
+            navibar_html += '&nbsp;[<a href="%s">%s</a>]' % (wikiutil.quoteWikinameURL(p), wikiutil.escape(p))
 
         for pagename in pages:
-            file = wikiutil.quoteWikinameURL(pagename) # we have the same name in URL and FS
+            # we have the same name in URL and FS
+            file = wikiutil.quoteWikinameURL(pagename) 
             _util.log('Writing "%s"...' % file)
             try:
                 pagehtml = ''
                 page = Page.Page(request, pagename)
+                request.page = page
                 try:
                     request.reset()
-                    out = StringIO.StringIO()
-                    request.redirect(out)
-                    page.send_page(request, count_hit=0, content_only=1)
-                    pagehtml = out.getvalue()
-                    request.redirect()
+                    pagehtml = request.redirectedOutput(page.send_page, request, count_hit=0, content_only=1)
                 except:
                     errcnt = errcnt + 1
                     print >>sys.stderr, "*** Caught exception while writing page!"
@@ -149,6 +173,7 @@ class MoinDump(_util.Script):
                     import traceback
                     traceback.print_exc(None, errlog)
             finally:
+                logo_html = re.sub(original_url_prefix + "/?", "", request.cfg.logo_string)
                 timestamp = time.strftime("%Y-%m-%d %H:%M")
                 filepath = os.path.join(outputdir, file)
                 fileout = codecs.open(filepath, 'w', config.charset)
@@ -176,9 +201,11 @@ class MoinDump(_util.Script):
         if errcnt:
             print >>sys.stderr, "*** %d error(s) occurred, see '%s'!" % (errcnt, errfile)
 
+
 def run():
     MoinDump().run()
 
+
 if __name__ == "__main__":
     run()
 
