@@ -20,14 +20,14 @@ srcwiki = xmlrpclib.ServerProxy("http://moinmaster.wikiwikiweb.de/?action=xmlrpc
 dsttrans = BasicAuthTransport(user,password)
 dstwiki = xmlrpclib.ServerProxy("http://devel.linuxwiki.org/moin--main/__xmlrpc/?action=xmlrpc2", transport=dsttrans)
 
-def rawGetPage(srcurl, pagename, encoding='iso8859-1'):
+def rawGetPage(srcurl, pagename, encoding='iso-8859-1'):
     url = srcurl % wikiutil.quoteWikinameFS(pagename.encode(encoding))
     pagedata = urllib.urlopen(url).read()
     return unicode(pagedata, encoding).encode('utf-8')
 
 def transferpage(srcurlformat, dstwiki, pagename):
     pagedata = srcwiki.getPage(pagename)
-    #pagedata = rawGetPage(srcurlformat, pagename, 'iso8859-1')
+    #pagedata = rawGetPage(srcurlformat, pagename, 'iso-8859-1')
     rc = dstwiki.putPage(pagename, pagedata)
     print "Transferred %s. Len = %d, rc = %s" % (pagename.encode('ascii','replace'), len(pagedata), str(rc))
 
