@@ -28,9 +28,6 @@
 
     Significant contributions to this module by R. Church <rc@ghostbitch.org>
 
-    TODO: sends 'expires' header but browsers seems to ignore it,
-    requesting the css and images on each request.
-
     @copyright: 2001-2004 MoinMoin:JrgenHermann
     @copyright: 2005      MoinMoin:AlexanderSchremmer
     @copyright: 2005      MoinMoin:NirSoffer
@@ -38,19 +35,18 @@
 """
 
 # Imports
-import os, sys, time, urllib, socket, errno, shutil
+import os, sys, time, socket, errno, shutil
 import BaseHTTPServer, SimpleHTTPServer, SocketServer
-from email.Utils import formatdate
 
 # MoinMoin imports
-from MoinMoin import version
+from MoinMoin import version, wikiutil
 from MoinMoin.server import Config, switchUID
 from MoinMoin.request import RequestStandAlone
+from MoinMoin.util.datetime import formathttpdate
 
 # Server globals
 httpd = None
 config = None
-moin_requests_done = 0
 
 
 class SimpleServer(BaseHTTPServer.HTTPServer):
@@ -315,7 +311,13 @@ class MoinRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
     
     def serve_static_file(self):
         """ Serve files from the htdocs directory """
+        # 1 week expiry for static files
         self.expires = self.staticExpire
+        path = self.path.split("?", 1)
+        if len(path) > 1:
+            self.path = path[0]
+            # XXXXXX params
+            
         try: 
             SimpleHTTPServer.SimpleHTTPRequestHandler.do_GET(self)
         except socket.error, err:
@@ -325,30 +327,16 @@ class MoinRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
     
     def serve_moin(self):
         """ Serve a request using moin """
-        global moin_requests_done
-
         # don't make an Expires header for wiki pages
         self.expires = 0
-        
-        # Run request, use optional profiling
-        if config.memoryProfile:
-            config.memoryProfile.addRequest()
+
         try:
-            if config.hotshotProfile and moin_requests_done > 0:
-                # Don't profile the first request, its not interesting
-                # for long running process, and its very expensive.
-                runcall = config.hotshotProfile.runcall
-                req = runcall(RequestStandAlone, self)
-                runcall(req.run)
-            else:
-                req = RequestStandAlone(self)
-                req.run()
+            req = RequestStandAlone(self, properties=config.properties)
+            req.run()
         except socket.error, err:
             # Ignore certain errors
             if err.args[0] not in [errno.EPIPE, errno.ECONNABORTED]:
                 raise
-        # Count moin requests
-        moin_requests_done += 1
         
     def translate_path(self, uri):
         """ Translate a /-separated PATH to the local filename syntax.
@@ -356,7 +344,7 @@ class MoinRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
         Components that mean special things to the local file system
         (e.g. drive or directory names) are ignored.
         """
-        file = urllib.unquote(uri)
+        file = wikiutil.url_unquote(uri, want_unicode=False)
         file.replace('\\', '/')
         words = file.split('/')
         words = filter(None, words)
@@ -383,7 +371,7 @@ class MoinRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
         if self.expires:
             now = time.time()
             expires = now + self.expires
-            self.send_header('Expires', formatdate(expires))
+            self.send_header('Expires', formathttpdate(expires))
         SimpleHTTPServer.SimpleHTTPRequestHandler.end_headers(self)
         
     def copyfile(self, source, outputfile):
@@ -394,8 +382,29 @@ class MoinRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
         faster than the default 16K buffer.
         """
         shutil.copyfileobj(source, outputfile, length=self.bufferSize)
-        
 
+
+def memoryProfileDecorator(func, profile):
+    """ Return a profiled function """
+    def profiledFunction(*args, **kw):
+        profile.addRequest()
+        return func(*args, **kw)
+    return profiledFunction
+
+
+def hotshotProfileDecorator(func, profile):
+    """ Return a profiled function """
+    profile.moin_requests_done = 0
+    def profiledFunction(*args, **kw):
+        profile.moin_requests_done += 1
+        if profile.moin_requests_done == 1:
+            # Don't profile first request, its not interesting
+            return func(*args, **kw)
+        return profile.runcall(func, *args, **kw)
+    
+    return profiledFunction
+
+        
 def quit(signo, stackframe):
     """ Signal handler for aborting signals """
     global httpd
@@ -410,7 +419,7 @@ def registerSignalHandlers(func):
         import signal
         signal.signal(signal.SIGABRT, func)
         signal.signal(signal.SIGINT, func)
-        signal.signal(signal.SIGTERM, func)        
+        signal.signal(signal.SIGTERM, func)
     except ImportError:
         pass
 
@@ -439,10 +448,14 @@ def makeServer(config):
     _config.use_threads = serverClass.use_threads    
     return serverClass(config)
 
+# ------------------------------------------------------------------------
+# Public interface
 
 class StandaloneConfig(Config):
     """ Standalone server default config """
 
+    name = 'moin'
+    properties = {}
     docs = '/usr/share/moin/htdocs'
     user = 'www-data'
     group = 'www-data'
@@ -469,21 +482,35 @@ def run(configClass):
     
     @param configClass: config class
     """    
-    # set globals (only on first import, save from reloads!)
-    global httpd
-    global config
+    # Run only once!
+    global httpd, config
+    if httpd is not None:
+        raise RuntimeError("You can run only one server per process!")
 
     config = configClass()    
-    if config.logPath:
-        sys.stderr = file(config.logPath, 'at')
+     
+    # Install hotshot profiled serve_moin method. To compare with other
+    # servers, we profile the part that create and run the request.
+    if config.hotshotProfile:
+        import hotshot
+        config.hotshotProfile = hotshot.Profile(config.hotshotProfile)
+        MoinRequestHandler.serve_moin =  hotshotProfileDecorator(
+            MoinRequestHandler.serve_moin, config.hotshotProfile)
+    
+    # Install a memory profiled serve_moin method
     if config.memoryProfile:
         config.memoryProfile.sample()
+        MoinRequestHandler.serve_moin = memoryProfileDecorator(
+            MoinRequestHandler.serve_moin, config.memoryProfile)
+    
+    if config.logPath:
+        sys.stderr = file(config.logPath, 'at')
     registerSignalHandlers(quit)
-    httpd = makeServer(config)
-
-    # If running as root, switch user and group id
-    if os.getuid() == 0 and os.name == 'posix':
-        switchUID(config.uid, config.gid)        
-
+    httpd = makeServer(config)    
+    
+    # Run as a safe user (posix only)
+    if os.name == 'posix' and os.getuid() == 0:
+        switchUID(config.uid, config.gid)
+    
     httpd.serve_forever()
-
+        
