@@ -4,6 +4,10 @@
 
     This macro creates a hitcount chart from the data in "event.log".
 
+    TODO: refactor to use a class, this code is ugly. A lot of code
+    here is duplicated in stats.useragents. Maybe both can use same
+    base class, maybe some parts are useful to other code.
+
     @copyright: 2002-2004 by Jrgen Hermann <jh@web.de>
     @license: GNU GPL, see COPYING for details.
 """
@@ -13,19 +17,16 @@ _debug = 0
 from MoinMoin import caching, config, wikiutil
 from MoinMoin.Page import Page
 from MoinMoin.util import MoinMoinNoFooter, datetime
-from MoinMoin.logfile import eventlog
 from MoinMoin.formatter.text_html import Formatter
+from MoinMoin.logfile import eventlog, logfile
 
 
 def linkto(pagename, request, params=''):
-    from MoinMoin.util import web
     _ = request.getText
 
     if not request.cfg.chart_options:
         request.formatter = Formatter(request)
-        return (request.formatter.sysmsg(1) +
-                request.formatter.text(_('Charts are not available!')) +
-                request.formatter.sysmsg(0))
+        return text(pagename, request, params)
 
     if _debug:
         return draw(pagename, request)
@@ -34,12 +35,12 @@ def linkto(pagename, request, params=''):
 
     # Create escaped query string from dict and params
     querystr = {'action': 'chart', 'type': 'hitcounts'}
-    querystr = web.makeQueryString(querystr)
+    querystr = wikiutil.makeQueryString(querystr)
     querystr = wikiutil.escape(querystr)
     if params:
         querystr += '&amp;' + params
     
-    # TODO: remove escape=0 in 1.4
+    # TODO: remove escape=0 in 2.0
     data = {'url': page.url(request, querystr, escape=0)}
     data.update(request.cfg.chart_options)
     result = ('<img src="%(url)s" width="%(width)d" height="%(height)d"'
@@ -48,6 +49,142 @@ def linkto(pagename, request, params=''):
     return result
 
 
+def get_data(pagename, request, filterpage=None):
+    
+    # Get results from cache
+    if filterpage:
+        arena = Page(request, pagename)
+    else:
+        arena = 'charts'
+    
+    cache_days, cache_views, cache_edits = [], [], []
+    cache_date = 0
+    cache = caching.CacheEntry(request, arena, 'hitcounts')
+    if cache.exists():
+        try:
+            cache_date, cache_days, cache_views, cache_edits = eval(cache.content())
+        except:
+            cache.remove() # cache gone bad
+
+    # Get new results from the log
+    log = eventlog.EventLog(request)
+    try:
+        new_date = log.date()
+    except logfile.LogMissing:
+        new_date = None
+        
+    # prepare data
+    days = []
+    views = []
+    edits = []
+    ratchet_day = None
+    ratchet_time = None
+    if new_date is not None:
+        log.set_filter(['VIEWPAGE', 'SAVEPAGE'])
+        for event in log.reverse():
+            #print ">>>", wikiutil.escape(repr(event)), "<br>"
+    
+            if event[0] <=  cache_date:
+                break
+            # XXX Bug: event[2].get('pagename') -> u'Aktuelle%C4nderungen' 8(
+            eventpage = event[2].get('pagename','')
+            if filterpage and eventpage != filterpage:
+                continue
+            time_tuple = request.user.getTime(wikiutil.version2timestamp(event[0]))
+            day = tuple(time_tuple[0:3])
+            if day != ratchet_day:
+                # new day
+                while ratchet_time:
+                    ratchet_time -= 86400
+                    rday = tuple(request.user.getTime(ratchet_time)[0:3])
+                    if rday <= day: break
+                    days.append(request.user.getFormattedDate(ratchet_time))
+                    views.append(0)
+                    edits.append(0)
+                days.append(request.user.getFormattedDate(wikiutil.version2timestamp(event[0])))
+                views.append(0)
+                edits.append(0)
+                ratchet_day = day
+                ratchet_time = wikiutil.version2timestamp(event[0])
+            if event[1] == 'VIEWPAGE':
+                views[-1] = views[-1] + 1
+            elif event[1] == 'SAVEPAGE':
+                edits[-1] = edits[-1] + 1
+    
+        days.reverse()
+        views.reverse()
+        edits.reverse()
+
+    # merge the day on the end of the cache
+    if cache_days and days and days[0] == cache_days[-1]:
+        cache_edits[-1] += edits[0]
+        cache_views[-1] += views[0]
+        days, views, edits = days[1:], views[1:], edits[1:]
+
+    # Update and save the cache
+    cache_days.extend(days)
+    cache_views.extend(views)
+    cache_edits.extend(edits)
+    if new_date is not None:
+        cache.update("(%r, %r, %r, %r)" % 
+                     (new_date, cache_days, cache_views, cache_edits))
+
+    return cache_days, cache_views, cache_edits
+
+
+def text(pagename, request, params=''):
+    from MoinMoin.util.dataset import TupleDataset, Column
+    from MoinMoin.widget.browser import DataBrowserWidget
+    _ = request.getText
+
+    # check params
+    filterpage = None
+    if params.startswith('page='):
+        params = params[len('page='):]        
+        filterpage = wikiutil.decodeUserInput(params)
+    
+    if request and request.form and request.form.has_key('page'):
+        filterpage = request.form['page'][0]
+
+    days, views, edits = get_data(pagename, request, filterpage)
+
+    hits = TupleDataset()
+    hits.columns = [Column('day', label=_("Date"), align='left'),
+                    Column('views', label=_("Views/day") , align='right'),
+                    Column('edits', label=_("Edits/day") , align='right')
+                    ]
+
+    maxentries = 30
+
+    if maxentries < len(days):
+        step = float(len(days))/ maxentries
+    else:
+        step = 1
+        
+    sv = 0.0
+    se = 0.0
+    sd = 0.0
+    cnt = 0
+
+    for i in xrange(len(days)-1,-1,-1):
+        d,v,e = days[i], views[i], edits[i]
+        # sum up views and edits to step days 
+        sd += 1
+        cnt += 1
+        sv += v
+        se += e
+        if cnt >= step:
+            cnt -= step
+            hits.addRow((d, "%.1f" % (sv/sd), "%.1f" % (se/sd)))
+            sv = 0.0
+            se = 0.0
+            sd = 0.0
+
+    table = DataBrowserWidget(request)
+    table.setData(hits)
+    return table.toHTML()
+
+
 def draw(pagename, request):
     import shutil, cStringIO
     from MoinMoin.stats.chart import Chart, ChartData, Color
@@ -59,89 +196,7 @@ def draw(pagename, request):
     if request and request.form and request.form.has_key('page'):
         filterpage = request.form['page'][0]
 
-
-    # get results from cache
-    if filterpage:
-        arena = Page(request, pagename)
-        key = 'hitcounts'
-    else:
-        arena = 'charts'
-        key = 'hitcounts'
-    
-    cache = caching.CacheEntry(request, arena, key)
-    if cache.exists():
-        try:
-            cache_date, cache_days, cache_views, cache_edits = eval(cache.content())
-        except:
-            cache_days, cache_views, cache_edits = [], [], []
-            cache_date = 0
-    else:
-        cache_days, cache_views, cache_edits = [], [], []
-        cache_date = 0
-
-    logfile = eventlog.EventLog(request)
-    logfile.set_filter(['VIEWPAGE', 'SAVEPAGE'])
-    new_date = logfile.date()
-
-    # prepare data
-    days = []
-    views = []
-    edits = []
-    ratchet_day = None
-    ratchet_time = None
-    for event in logfile.reverse():
-        #print ">>>", wikiutil.escape(repr(event)), "<br>"
-
-        if event[0] <=  cache_date:
-            break
-        # XXX Bug: event[2].get('pagename') -> u'Aktuelle%C4nderungen' 8(
-        eventpage = event[2].get('pagename','')
-        if filterpage and eventpage != filterpage:
-            continue
-        time_tuple = request.user.getTime(wikiutil.version2timestamp(event[0]))
-        day = tuple(time_tuple[0:3])
-        if day != ratchet_day:
-            # new day
-            while ratchet_time:
-                ratchet_time -= 86400
-                rday = tuple(request.user.getTime(ratchet_time)[0:3])
-                if rday <= day: break
-                days.append(request.user.getFormattedDate(ratchet_time))
-                views.append(0)
-                edits.append(0)
-            days.append(request.user.getFormattedDate(wikiutil.version2timestamp(event[0])))
-            views.append(0)
-            edits.append(0)
-            ratchet_day = day
-            ratchet_time = wikiutil.version2timestamp(event[0])
-        if event[1] == 'VIEWPAGE':
-            views[-1] = views[-1] + 1
-        elif event[1] == 'SAVEPAGE':
-            edits[-1] = edits[-1] + 1
-
-    # give us a chance to develop this
-    if _debug:
-        return "labels = %s<br>views = %s<br>edits = %s<br>" % \
-            tuple(map(wikiutil.escape, map(repr, [days, views, edits])))
-
-    days.reverse()
-    views.reverse()
-    edits.reverse()
-
-    # merge the day on the end of the cache
-    if cache_days and days and days[0] == cache_days[-1]:
-        cache_edits[-1] += edits[0]
-        cache_views[-1] += views[0]
-        days, views, edits = days[1:], views[1:], edits[1:]
-
-    cache_days.extend(days)
-    cache_views.extend(views)
-    cache_edits.extend(edits)
-
-    days, views, edits = cache_days, cache_views, cache_edits
-
-    # save to cache
-    cache.update("(%r, %r, %r, %r)" % (new_date, days, views, edits))
+    days, views, edits = get_data(pagename, request, filterpage)
 
     import math
     
