@@ -3,17 +3,17 @@
 """
     MoinMoin - Version Information
 
-    @copyright: 2000-2004 by Jrgen Hermann <jh@web.de>
+    @copyright: 2000-2006 by Jrgen Hermann <jh@web.de>
     @license: GNU GPL, see COPYING for details.
 """
 
 try:
     from patchlevel import patchlevel
 except:
-    patchlevel = '1.3.4 release'
+    patchlevel = 'release'
 
 project = "MoinMoin"
-release  = '1.3.4'
+release  = '1.5.2'
 revision = patchlevel
 
 if __name__ == '__main__':
