@@ -19,7 +19,7 @@
     @license: GNU GPL, see COPYING for details.
 """
 
-import os, re, time, urllib
+import os, re, time
 from MoinMoin import config, util, wikiutil
 from MoinMoin.Page import Page
 from MoinMoin.util import MoinMoinNoFooter, pysupport
@@ -106,7 +106,7 @@ def do_diff(pagename, request):
     request.setContentLanguage(request.lang)
 
     request.http_headers()
-    wikiutil.send_title(request, _('Diff for "%s"') % (pagename,), pagename=pagename)
+    wikiutil.send_title(request, _('Diff for "%s"') % (pagename,), pagename=pagename, allow_doubleclick=1)
   
     if (rev1>0 and rev2>0 and rev1>rev2) or (rev1==0 and rev2>0):
         rev1,rev2 = rev2,rev1
@@ -190,7 +190,7 @@ def do_diff(pagename, request):
             request.write('</pre>')
 
     request.write('</div>\n') # end content div
-    wikiutil.send_footer(request, pagename, showpage=1)
+    wikiutil.send_footer(request, pagename)
 
 
 def do_info(pagename, request):
@@ -256,10 +256,8 @@ def do_info(pagename, request):
             Column('rev', label='#', align='right'),
             Column('mtime', label=_('Date'), align='right'),
             Column('size',  label=_('Size'), align='right'),
-            Column('diff', label='<input type="submit"           value="%s">' % (_("Diff"))),
-            # TODO: translate to English
-            # entfernt, nicht 4.01 compliant:          href="%s"   % page.url(request)
-            Column('editor', label=_('Editor'), hidden=not request.cfg.show_hosts),
+            Column('diff', label='<input type="submit" value="%s">' % (_("Diff"))),
+            Column('editor', label=_('Editor'), hidden=not request.cfg.show_names),
             Column('comment', label=_('Comment')),
             Column('action', label=_('Action')),
             ]
@@ -318,7 +316,8 @@ def do_info(pagename, request):
             else: # ATT*
                 rev = '-'
                 diff = '-'
-                filename = line.extra
+                
+                filename = wikiutil.url_unquote(line.extra)
                 comment = "%s: %s %s" % (line.action, filename, line.comment)
                 size = 0
                 if line.action != 'ATTDEL':
@@ -362,7 +361,6 @@ def do_info(pagename, request):
 
         # print version history
         from MoinMoin.widget.browser import DataBrowserWidget
-        from MoinMoin.formatter.text_html import Formatter
 
         request.write('<h2>%s</h2>\n' % _('Revision History'))
 
@@ -370,14 +368,13 @@ def do_info(pagename, request):
             request.write(_('No log entries found.'))
             return
 
-        # TODO: this form activate revert, which should use post, but
+        # TODO: this form activates revert, which should use POST, but
         # other actions should use get. Maybe we should put the revert
         # into the page view itself, and not in this form.
         request.write('<form method="GET" action="">\n')
         request.write('<div id="page-history">\n')
         request.write('<input type="hidden" name="action" value="diff">\n')
 
-        request.formatter = Formatter(request)
         history_table = DataBrowserWidget(request)
         history_table.setData(history)
         history_table.render()
@@ -395,7 +392,7 @@ def do_info(pagename, request):
     # This action uses page or wiki language TODO: currently
     # page.language is broken and not available now, when we fix it,
     # this will be automatically fixed.
-    lang = page.language or request.cfg.default_lang
+    lang = page.language or request.cfg.language_default
     request.setContentLanguage(lang)
     
     wikiutil.send_title(request, _('Info for "%s"') % (title,), pagename=pagename)
@@ -415,14 +412,14 @@ def do_info(pagename, request):
     
     if show_hitcounts:
         from MoinMoin.stats import hitcounts
-        request.write(hitcounts.linkto(pagename, request, 'page=' + urllib.quote_plus(pagename.encode(config.charset))))
+        request.write(hitcounts.linkto(pagename, request, 'page=' + wikiutil.url_quote_plus(pagename)))
     elif show_general:
         general(page, pagename, request)
     else:
         history(page, pagename, request)
         
     request.write('</div>\n') # end content div
-    wikiutil.send_footer(request, pagename, showpage=1)
+    wikiutil.send_footer(request, pagename)
 
 
 def do_recall(pagename, request):
@@ -460,6 +457,7 @@ def do_show(pagename, request):
             rev = 0
         Page(request, pagename, rev=rev).send_page(request, count_hit=1)
     else:
+        request.cacheable = 1
         Page(request, pagename).send_page(request, count_hit=1)
 
 
@@ -472,10 +470,10 @@ def do_refresh(pagename, request):
         arena = Page(request, pagename)
     key = request.form.get('key', ['text_html'])[0]
 
-    # Rmove cache entry (if exists), and send the page
+    # Remove cache entry (if exists), and send the page
     from MoinMoin import caching
-    cache = caching.CacheEntry(request, arena, key)
-    cache.remove()
+    caching.CacheEntry(request, arena, key).remove()
+    caching.CacheEntry(request, arena, "pagelinks").remove()
     do_show(pagename, request)
 
 
@@ -491,16 +489,6 @@ def do_content(pagename, request):
     raise MoinMoinNoFooter
 
 
-def do_edit(pagename, request):
-    if not request.user.may.write(pagename):
-        _ = request.getText
-        Page(request, pagename).send_page(request,
-            msg = _('You are not allowed to edit this page.'))
-        return
-    from MoinMoin.PageEditor import PageEditor
-    PageEditor(request, pagename).sendEditor()
-
-
 def do_revert(pagename, request):
     from MoinMoin.PageEditor import PageEditor
     _ = request.getText
@@ -524,9 +512,7 @@ def do_revert(pagename, request):
     pg.send_page(request, msg=savemsg)
     return None
 
-def do_savepage(pagename, request):
-    from MoinMoin.PageEditor import PageEditor
-
+def do_edit(pagename, request):
     _ = request.getText
 
     if not request.user.may.write(pagename):
@@ -534,18 +520,70 @@ def do_savepage(pagename, request):
             msg = _('You are not allowed to edit this page.'))
         return
 
-    pg = PageEditor(request, pagename)
-    savetext = request.form.get('savetext', [u''])[0]
+    valideditors = ['text', 'gui',]
+    editor = ''
+    if request.user.valid:
+        editor = request.user.editor_default
+    if editor not in valideditors:
+        editor = request.cfg.editor_default
+    
+    editorparam = request.form.get('editor', [editor])[0]
+    if editorparam == "guipossible":
+        lasteditor = editor
+    elif editorparam == "textonly":
+        editor = lasteditor = 'text'
+    else:
+        editor = lasteditor = editorparam
+
+    if request.cfg.editor_force:
+        editor = request.cfg.editor_default
+
+    # if it is still nothing valid, we just use the text editor
+    if editor not in valideditors:
+        editor = 'text'
+            
+    savetext = request.form.get('savetext', [None])[0]
     rev = int(request.form.get('rev', ['0'])[0])
     comment = request.form.get('comment', [u''])[0]
     category = request.form.get('category', [None])[0]
     rstrip = int(request.form.get('rstrip', ['0'])[0])
     trivial = int(request.form.get('trivial', ['0'])[0])
 
+    if request.form.has_key('button_switch'):
+        if editor == 'text':
+            editor = 'gui'
+        else: # 'gui'
+            editor = 'text'
+
+    # load right editor class
+    if editor == 'gui':
+        from MoinMoin.PageGraphicalEditor import PageGraphicalEditor
+        pg = PageGraphicalEditor(request, pagename)
+    else: # 'text'
+        from MoinMoin.PageEditor import PageEditor
+        pg = PageEditor(request, pagename)
+
+    # is invoked without savetext start editing
+    if savetext is None:
+        pg.sendEditor()
+        return
+
+    # convert input from Graphical editor
+    if lasteditor == 'gui':
+        from MoinMoin.converter.text_html_text_x_moin import convert
+        savetext = convert(request, pagename, savetext) # XXX error handling
+
     # IMPORTANT: normalize text from the form. This should be done in
     # one place before we manipulate the text.
     savetext = pg.normalizeText(savetext, stripspaces=rstrip)
 
+    comment = wikiutil.clean_comment(comment)
+
+    # Edit was canceled
+    if request.form.has_key('button_cancel'):
+        pg.sendCancel(savetext or "", rev)
+        return
+
     # Add category
 
     # TODO: this code does not work with extended links, and is doing
@@ -553,7 +591,7 @@ def do_savepage(pagename, request):
     # a full interface for categories (add, delete) or just add them by
     # markup.
     
-    if category:
+    if category and category != _('<No addition>', formatted=False): # opera 8.5 needs this
         # strip trailing whitespace
         savetext = savetext.rstrip()
 
@@ -577,34 +615,20 @@ def do_savepage(pagename, request):
             savetext += ' '
         savetext += category + u'\n' # Should end with newline!
 
-    # Clean comment - replace CR, LF, TAB by whitespace, delete control chars
-    # TODO: move this to config, create on first call then return cached.
-    remap_chars = {
-        ord(u'\t'): u' ',
-        ord(u'\r'): u' ',
-        ord(u'\n'): u' ',
-    }
-    control_chars = u'\x00\x01\x02\x03\x04\x05\x06\x07\x08\x0b\x0c\x0e\x0f' \
-                    '\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f'
-    for c in control_chars:
-        remap_chars[c] = None
-    comment = comment.translate(remap_chars)
-
     # Preview, spellcheck or spellcheck add new words
     if (request.form.has_key('button_preview') or
         request.form.has_key('button_spellcheck') or
         request.form.has_key('button_newwords')):
         pg.sendEditor(preview=savetext, comment=comment)
-
-    # Edit was canceled
-    elif request.form.has_key('button_cancel'):
-        pg.sendCancel(savetext, rev)
+    
+    # Preview with mode switch
+    elif request.form.has_key('button_switch'):
+        pg.sendEditor(preview=savetext, comment=comment, staytop=1)
     
     # Save new text
     else:
         try:
-            savemsg = pg.saveText(savetext, rev, trivial=trivial,
-                                  comment=comment)
+            savemsg = pg.saveText(savetext, rev, trivial=trivial, comment=comment)
         except pg.EditConflict, msg:
             # Handle conflict and send editor
 
@@ -638,46 +662,63 @@ Have a look at the diff of %(difflink)s to see what has been changed.""") % {
         pg.send_page(request, msg=savemsg)
         
 
+def do_quicklink(pagename, request):
+    """ Add the current wiki page to the user quicklinks 
+    
+    TODO: what if add or remove quicklink fail? display an error message?
+    """
+    _ = request.getText
+    msg = None
+
+    if not request.user.valid:
+        msg = _("You must login to add a quicklink.")    
+    elif request.user.isQuickLinkedTo([pagename]):
+        if request.user.removeQuicklink(pagename):
+            msg = _('Your quicklink to this page has been removed.')            
+    else:
+        if request.user.addQuicklink(pagename):
+            msg = _('A quicklink to this page has been added for you.')
+
+    Page(request, pagename).send_page(request, msg=msg)
+
+
 def do_subscribe(pagename, request):
-    """ Add the current wiki page to the subscribed_page property in
-        current user profile.
+    """ Subscribe or unsubscribe the user to pagename
+    
+    TODO: what if subscribe failed? no message is displayed.
     """
     _ = request.getText
     cfg = request.cfg
+    msg = None
 
     if not request.user.may.read(pagename):
         msg = _("You are not allowed to subscribe to a page you can't read.")
 
-    # check config
-    elif not cfg.mail_smarthost:
-        msg = _('''This wiki is not enabled for mail processing. '''
-                '''Contact the owner of the wiki, who can either enable email, or remove the "Subscribe" icon.''')
+    # Check if mail is enabled
+    elif not cfg.mail_enabled:
+        msg = _("This wiki is not enabled for mail processing.")
 
-    # check whether the user has a profile
+    # Suggest visitors to login
     elif not request.user.valid:
-        msg = _('''You didn't create a user profile yet. '''
-                '''Select UserPreferences in the upper right corner to create a profile.''')
+        msg = _("You must log in to use subscribtions.")
 
-    # check whether the user has an email address
+    # Suggest users without email to add their email address
     elif not request.user.email:
-        msg = _('''You didn't enter an email address in your profile. '''
-                '''Select your name (UserPreferences) in the upper right corner and enter a valid email address.''')
+        msg = _("Add your email address in your UserPreferences to use subscriptions.")
 
-    # check whether already subscribed
     elif request.user.isSubscribedTo([pagename]):
-        if request.user.subscribePage(pagename,remove=True):
-            request.user.save()
+        # Try to unsubscribe
+        if request.user.unsubscribe(pagename):
             msg = _('Your subscribtion to this page has been removed.')
         else:
             msg = _("Can't remove regular expression subscription!") + u' ' + \
-                  _('To unsubscribe, go to your profile and delete this page from the subscription list.')
+                  _("Edit the subscription regular expressions in your "
+                    "UserPreferences.")
             
-    # subscribe to current page
     else:
-        if request.user.subscribePage(pagename):
-            request.user.save()
-        msg = _('You have been subscribed to this page.') + u' ' + \
-              _('To unsubscribe, go to your profile and delete this page from the subscription list.')
+        # Try to subscribe
+        if request.user.subscribe(pagename):
+            msg = _('You have been subscribed to this page.')
 
     Page(request, pagename).send_page(request, msg=msg)
 
@@ -706,12 +747,7 @@ def do_bookmark(pagename, request):
     Page(request, pagename).send_page(request)
   
 
-def do_formtest(pagename, request):
-    # test a user defined form
-    from MoinMoin import wikiform
-    wikiform.do_formtest(pagename, request)
-
-
+# Commented out by Florian Festi, cf. bug report about it
 # def do_macro(pagename, request):
 #     """ Execute a helper action within a macro.
 #     """
@@ -768,27 +804,48 @@ def do_format(pagename, request):
         mimetype = u"text/plain"
 
     # try to load the formatter
-    Formatter = wikiutil.importPlugin(request.cfg, "formatter",
-        mimetype.translate({ord(u'/'): u'_', ord(u'.'): u'_'}), "Formatter")
-    if Formatter is None:
+    formatterName = mimetype.translate({ord(u'/'): u'_', ord(u'.'): u'_'})
+    try:
+        Formatter = wikiutil.importPlugin(request.cfg, "formatter",
+                                          formatterName, "Formatter")
+    except wikiutil.PluginMissingError:
         # default to plain text formatter
-        del Formatter
         mimetype = "text/plain"
-        from formatter.text_plain import Formatter
+        from MoinMoin.formatter.text_plain import Formatter
 
     if "xml" in mimetype:
         mimetype = "text/xml"
     request.http_headers(["Content-Type: %s; charset=%s" % (mimetype, config.charset)])
 
-    Page(request, pagename, formatter = Formatter(request)).send_page(request)
+    Page(request, pagename, formatter=Formatter(request)).send_page(request)
     raise MoinMoinNoFooter
 
 
 def do_chart(pagename, request):
-    if request.user.may.read(pagename) and request.cfg.chart_options:
-        chart_type = request.form['type'][0]
-        func = pysupport.importName("MoinMoin.stats." + chart_type, "draw")
-        func(pagename, request)
+    """ Show page charts 
+    
+    """
+    _ = request.getText
+    if not request.user.may.read(pagename):
+        msg = _("You are not allowed to view this page.")
+        return request.page.send_page(request, msg=msg)
+    
+    if not request.cfg.chart_options:
+        msg = _("Charts are not available!")
+        return request.page.send_page(request, msg=msg)
+    
+    chart_type = request.form.get('type', [''])[0].strip()
+    if not chart_type:
+        msg = _('You need to provide a chart type!')
+        return request.page.send_page(request, msg=msg)
+    
+    try:
+        func = pysupport.importName("MoinMoin.stats." + chart_type, 'draw')
+    except (ImportError, AttributeError):
+        msg = _('Bad chart type "%s"!') % chart_type
+        return request.page.send_page(request, msg=msg)
+    
+    func(pagename, request)
     raise MoinMoinNoFooter
 
 
@@ -800,45 +857,6 @@ def do_dumpform(pagename, request):
     raise MoinMoinNoFooter
 
 
-def do_export(pagename, request):
-    import shutil, StringIO
-    from MoinMoin.wikixml import wikiexport
-
-    # Protect this with ACLs, when ready!
-
-    # get parameters
-    compression = request.form.get('compression', None)
-
-    # prepare output stream
-    fileid = time.strftime("%Y-%m-%d", request.user.getTime(time.time()))
-    filename = "wiki-export-%s.xml" % fileid 
-    outbuff = StringIO.StringIO()
-    mimetype, out = 'text/xml', outbuff
-    if compression == "gzip":
-        import gzip
-        mimetype, out = 'application/x-gzip', gzip.GzipFile(
-            filename, "wb", 9, outbuff)
-        filename = filename + '.gz'
-
-    # create export document
-    export = wikiexport.WikiExport(out, public=1)
-    export.run()
-
-    # send http headers
-    headers = [
-        "Content-Type: %s" % mimetype,
-        "Content-Length: %d" % len(outbuff.getvalue()),
-    ]
-    if mimetype != 'text/xml':
-        headers.append("Content-Disposition: attachment; filename=%s" % filename)
-    request.http_headers(headers)
-
-    # copy the body
-    outbuff.seek(0)
-    shutil.copyfileobj(outbuff, request, 8192)
-    raise MoinMoinNoFooter
-
-
 def do_test(pagename, request):
     from MoinMoin.wikitest import runTest
     request.http_headers(["Content-type: text/plain;charset=%s" % config.charset])
@@ -861,15 +879,17 @@ def getPlugins(request):
 
 def getHandler(request, action, identifier="execute"):
     # check for excluded actions
-    if action in request.cfg.excluded_actions:
+    if action in request.cfg.actions_excluded:
         return None
 
-    handler = wikiutil.importPlugin(request.cfg, "action", action, identifier)
-    if handler is None:
+    from MoinMoin.formatter.text_html import Formatter
+    request.formatter = Formatter(request)
+
+    try:
+        handler = wikiutil.importPlugin(request.cfg, "action", action,
+                                        identifier)
+    except wikiutil.PluginMissingError:
         handler = globals().get('do_' + action)
         
     return handler
 
-    
- 
-
