@@ -1,12 +1,15 @@
 # -*- coding: iso-8859-1 -*-
 """
-    MoinMoin Dictionary / Group Functions
+    MoinMoin - Dictionary / Group Functions
 
     @copyright: 2003 by Thomas Waldmann, http://linuxwiki.de/ThomasWaldmann
     @copyright: 2003 by Gustavo Niemeyer, http://moin.conectiva.com.br/GustavoNiemeyer
     @license: GNU GPL, see COPYING for details.
 """
-import re, time, os, copy
+import re, time, os
+
+#import copy #broken, see comments at top of this file:
+from MoinMoin.support import copy
 
 # cPickle can encode normal and Unicode strings
 # see http://docs.python.org/lib/node66.html
@@ -16,15 +19,11 @@ except ImportError:
     import pickle
 
 # Set pickle protocol, see http://docs.python.org/lib/node64.html
-try:
-    # Requires 2.3
-    PICKLE_PROTOCOL = pickle.HIGHEST_PROTOCOL
-except AttributeError:
-    # Use protocol 1, binary format compatible with all python versions
-    PICKLE_PROTOCOL = 1
+PICKLE_PROTOCOL = pickle.HIGHEST_PROTOCOL
  
 from MoinMoin import config, caching, wikiutil, Page
 from MoinMoin.logfile.editlog import EditLog
+from MoinMoin.logfile import logfile
 
 # Version of the internal data structure which is pickled
 # Please increment if you have changed the structure
@@ -46,11 +45,7 @@ class DictBase:
         """
         self.name = name
 
-        # Lazy compile class regex on first use. All instances share the
-        # same regex, compiled once when the first instance is created.
-        if isinstance(self.__class__.regex, (str, unicode)):
-            self.__class__.regex = re.compile(self.__class__.regex,
-                                              re.MULTILINE | re.UNICODE)
+        self.regex = re.compile(self.regex, re.MULTILINE | re.UNICODE)
 
         # Get text from page named 'name'
         p = Page.Page(request, name)
@@ -90,7 +85,7 @@ class Dict(DictBase):
        any text ignored      
     '''
     # Key:: Value - ignore all but key:: value pairs, strip whitespace
-    regex = r'^\s(?P<key>.+?)::\s(?P<val>.*?)\s*$'
+    regex = r'^ (?P<key>.+?):: (?P<val>.*?) *$'
 
     def initFromText(self, text):
         """ Create dict from keys and values in text
@@ -101,7 +96,6 @@ class Dict(DictBase):
         for match in self.regex.finditer(text):
             key, val = match.groups()
             self._dict[key] = val
-    
 
 class Group(DictBase):
     ''' Group of users, of pages, of whatever
@@ -121,7 +115,7 @@ class Group(DictBase):
     '''
     # * Member - ignore all but first level list items, strip whitespace
     # Strip free links markup if exists
-    regex = r'^\s\*\s(?:\[\")?(?P<member>.+?)(?:\"\])?\s*$'
+    regex = r'^ \* +(?:\[\")?(?P<member>.+?)(?:\"\])? *$'
 
     def initFromText(self, text):
         """ Create dict from group members in text
@@ -305,7 +299,7 @@ class GroupDict(DictDict):
         now = wikiutil.timestamp2version(int(time.time()))
         try:
             lastchange = EditLog(self.request).date()
-        except OSError: #editlog was not created yet
+        except logfile.LogMissing:
             lastchange = 0
             dump = 1
 
@@ -331,7 +325,6 @@ class GroupDict(DictDict):
         if lastchange < self.namespace_timestamp and dump==0:
             return
 
-        # TODO: Should be compiled only once, and cached in cfg
         isdict = re.compile(self.cfg.page_dict_regex, re.UNICODE).search
         isgroup = re.compile(self.cfg.page_group_regex, re.UNICODE).search
 
@@ -346,8 +339,10 @@ class GroupDict(DictDict):
 
             # remove old entries when dict or group page have been deleted,
             # add entries when pages have been added
-            olddictdict = self.dictdict
-            oldgroupdict = self.groupdict
+            # use copies because the dicts are shared via cfg.DICTS_DATA
+            #  and must not be modified
+            olddictdict = self.dictdict.copy()
+            oldgroupdict = self.groupdict.copy()
             self.dictdict = {}
             self.groupdict = {}
 
@@ -376,7 +371,7 @@ class GroupDict(DictDict):
             self.namespace_timestamp = now
 
         # check if groups / dicts have been modified on disk
-        for pagename in self.dictdict:
+        for pagename in self.dictdict.keys():
             if Page.Page(self.request, pagename).mtime_usecs() >= self.pageupdate_timestamp:
                 if isdict(pagename):
                     self.adddict(self.request, pagename)
