@@ -6,12 +6,11 @@
     @license: GNU GPL, see COPYING for details.
 """
     
-import os, re, difflib
+import os, re, difflib, urllib, cgi
 
 from MoinMoin import util, version, config
 from MoinMoin.util import pysupport
 
-
 # Exceptions
 class InvalidFileNameError(Exception):
     """ Called when we find an invalid file name """ 
@@ -92,47 +91,126 @@ def decodeUserInput(s, charsets=[config.charset]):
             return s.decode(charset)
         except UnicodeError:
             pass
-    raise UnicodeError('The string "%s" cannot be decoded.' % s)
+    raise UnicodeError('The string %r cannot be decoded.' % s)
+
+
+# this is a thin wrapper around urllib (urllib only handles str, not unicode)
+# with py <= 2.4.1, it would give incorrect results with unicode
+# with py == 2.4.2, it crashes with unicode, if it contains non-ASCII chars
+def url_quote(s, safe='/', want_unicode=False):
+    """
+    Wrapper around urllib.quote doing the encoding/decoding as usually wanted:
+    
+    @param s: the string to quote (can be str or unicode, if it is unicode,
+              config.charset is used to encode it before calling urllib)
+    @param safe: just passed through to urllib
+    @param want_unicode: for the less usual case that you want to get back
+                         unicode and not str, set this to True
+                         Default is False.
+    """
+    if isinstance(s, unicode):
+        s = s.encode(config.charset)
+    elif not isinstance(s, str):
+        s = str(s)
+    s = urllib.quote(s, safe)
+    if want_unicode:
+        s = s.decode(config.charset) # ascii would also work
+    return s
+
+def url_quote_plus(s, safe='/', want_unicode=False):
+    """
+    Wrapper around urllib.quote_plus doing the encoding/decoding as usually wanted:
+    
+    @param s: the string to quote (can be str or unicode, if it is unicode,
+              config.charset is used to encode it before calling urllib)
+    @param safe: just passed through to urllib
+    @param want_unicode: for the less usual case that you want to get back
+                         unicode and not str, set this to True
+                         Default is False.
+    """
+    if isinstance(s, unicode):
+        s = s.encode(config.charset)
+    elif not isinstance(s, str):
+        s = str(s)
+    s = urllib.quote_plus(s, safe)
+    if want_unicode:
+        s = s.decode(config.charset) # ascii would also work
+    return s
+
+def url_unquote(s, want_unicode=True):
+    """
+    Wrapper around urllib.unquote doing the encoding/decoding as usually wanted:
+    
+    @param s: the string to unquote (can be str or unicode, if it is unicode,
+              config.charset is used to encode it before calling urllib)
+    @param want_unicode: for the less usual case that you want to get back
+                         str and not unicode, set this to False.
+                         Default is True.
+    """
+    if isinstance(s, unicode):
+        s = s.encode(config.charset) # ascii would also work
+    s = urllib.unquote(s)
+    if want_unicode:
+        s = s.decode(config.charset)
+    return s
+
+def parseQueryString(qstr, want_unicode=True):
+    """ Parse a querystring "key=value&..." into a dict.
+    """
+    is_unicode = isinstance(qstr, unicode)
+    if is_unicode:
+        qstr = qstr.encode(config.charset)
+    values = {}
+    for key, value in cgi.parse_qs(qstr).items():
+        if len(value) < 2:
+            v = ''.join(value)
+            if want_unicode:
+                try:
+                    v = unicode(v, config.charset)
+                except UnicodeDecodeError:
+                    v = unicode(v, 'iso-8859-1', 'replace')
+            values[key] = v
+    return values
+
+def makeQueryString(qstr=None, want_unicode=False, **kw):
+    """ Make a querystring from arguments.
+        
+    kw arguments overide values in qstr.
+
+    If a string is passed in, it's returned verbatim and
+    keyword parameters are ignored.
+
+    @param qstr: dict to format as query string, using either ascii or unicode
+    @param kw: same as dict when using keywords, using ascii or unicode
+    @rtype: string
+    @return: query string ready to use in a url
+    """
+    if qstr is None:
+        qstr = {}
+    if isinstance(qstr, type({})):
+        qstr.update(kw)
+        items = ['%s=%s' % (url_quote_plus(key, want_unicode=want_unicode), url_quote_plus(value, want_unicode=want_unicode)) for key, value in qstr.items()]
+        qstr = '&'.join(items)
+    return qstr
 
 
 # FIXME: better name would be quoteURL, as this is useful for any
 # string, not only wiki names.
 def quoteWikinameURL(pagename, charset=config.charset):
-    """
-    Return a simple encoding of filename in plain ascii.
+    """ Return a url encoding of filename in plain ascii
 
-    Warning: will raise UnicodeError if pagename can not be encoded
-    using charset. The default config.charset, 'utf-8', can encode any
-    character.
+    Use urllib.quote to quote any character that is not always safe. 
 
-    FIXME: isn't it better to use here urllib.quote instead of duplicating
-    the code? If we use cgi, urllib is already available. if we do
-    duplicate the code from urllib, why we don't use also fast_quote as
-    urllib does?
-
-    FIXME: If we don't use urllib.quote, maybe this is cleaner and faster
-    to use re, in the same way quoteWikinameFS works.
-    
-    @param pagename: the original pagename, maybe containing non-ascii chars
+    @param pagename: the original pagename (unicode)
+    @charset: url text encoding, 'utf-8' recommended. Other charsert
+              might not be able to encode the page name and raise
+              UnicodeError. (default config.charset ('utf-8')).
     @rtype: string
-    @return: the quoted filename, all special chars encoded in (xx)
+    @return: the quoted filename, all unsafe characters encoded
     """
-    safe = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789/+-_,\'*'
-    pagename = pagename.replace(u' ', u'_') # " " -> "_"
-    filename = pagename.encode(charset)
-    
-    res = list(filename)
-    c = None
-    for i in range(len(res)):
-        prev = c
-        c = res[i]
-        if c not in safe:
-            # FIXME: why are we doing this?
-            if c == '.' and prev == '/':
-                res[i] = '(%02x)' % ord(c)
-            else:
-                res[i] = '%%%02x' % ord(c)
-    return ''.join(res)
+    pagename = pagename.replace(u' ', u'_')
+    pagename = pagename.encode(charset)
+    return urllib.quote(pagename)
 
 
 def escape(s, quote=0):
@@ -162,6 +240,35 @@ def escape(s, quote=0):
         s = s.replace('"', "&quot;")
     return s
 
+def clean_comment(comment):
+    """ Clean comment - replace CR, LF, TAB by whitespace, delete control chars
+        TODO: move this to config, create on first call then return cached.
+    """
+    remap_chars = {
+        ord(u'\t'): u' ',
+        ord(u'\r'): u' ',
+        ord(u'\n'): u' ',
+    }
+    control_chars = u'\x00\x01\x02\x03\x04\x05\x06\x07\x08\x0b\x0c\x0e\x0f' \
+                    '\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f'
+    for c in control_chars:
+        remap_chars[c] = None
+    comment = comment.translate(remap_chars)
+    return comment
+
+def make_breakable(text, maxlen):
+    """ make a text breakable by inserting spaces into nonbreakable parts
+    """
+    text = text.split(" ")
+    newtext = []
+    for part in text:
+        if len(part) > maxlen:
+            while part:
+                newtext.append(part[:maxlen])
+                part = part[maxlen:]
+        else:
+            newtext.append(part)
+    return " ".join(newtext)
 
 ########################################################################
 ### Storage
@@ -334,7 +441,7 @@ def resolve_wiki(request, wikiurl):
     @param request: the request object
     @param wikiurl: the InterWiki:PageName link
     @rtype: tuple
-    @return: (wikitag, wikiurl, wikitail)
+    @return: (wikitag, wikiurl, wikitail, err)
     """
     # load map (once, and only on demand)
     try:
@@ -400,8 +507,7 @@ def isSystemPage(request, pagename):
     @return: true if page is a system page
     """
     return (request.dicts.has_member('SystemPagesGroup', pagename) or
-        isTemplatePage(request, pagename) or
-        isFormPage(request, pagename))
+        isTemplatePage(request, pagename))
 
 
 def isTemplatePage(request, pagename):
@@ -415,16 +521,6 @@ def isTemplatePage(request, pagename):
     return filter.search(pagename) is not None
 
 
-def isFormPage(request, pagename):
-    """ Is this a form page?
-    
-    @param pagename: the page name
-    @rtype: bool
-    @return: true if page is a form page
-    """
-    filter = re.compile(request.cfg.page_form_regex, re.UNICODE)
-    return filter.search(pagename) is not None
-
 def isGroupPage(request, pagename):
     """ Is this a name of group page?
 
@@ -500,16 +596,17 @@ def getFrontPage(request):
 
     @param request: current request
     @rtype: Page object
-    @return localized FrontPage
+    @return localized page_front_page, if there is a translation
     """
     return getSysPage(request, request.cfg.page_front_page)
     
 
-# FIXME - move to user class?
 def getHomePage(request, username=None):
     """
     Get a user's homepage, or return None for anon users and
     those who have not created a homepage.
+
+    DEPRECATED - try to use getInterwikiHomePage (see below)
     
     @param request: the request object
     @param username: the user's name
@@ -531,6 +628,34 @@ def getHomePage(request, username=None):
     return None
 
 
+def getInterwikiHomePage(request, username=None):
+    """
+    Get a user's homepage.
+    
+    cfg.user_homewiki influences behaviour of this:
+    'Self' does mean we store user homepage in THIS wiki.
+    When set to our own interwikiname, it behaves like with 'Self'.
+    
+    'SomeOtherWiki' means we store user homepages in another wiki.
+    
+    @param request: the request object
+    @param username: the user's name
+    @rtype: tuple (or None for anon users)
+    @return: (wikiname, pagename)
+    """
+    # default to current user
+    if username is None and request.user.valid:
+        username = request.user.name
+    if not username:
+        return None # anon user
+
+    homewiki = request.cfg.user_homewiki
+    if homewiki == request.cfg.interwikiname:
+        homewiki = 'Self'
+
+    return homewiki, username
+
+
 def AbsPageName(request, context, pagename):
     """
     Return the absolute pagename for a (possibly) relative pagename.
@@ -540,17 +665,16 @@ def AbsPageName(request, context, pagename):
     @rtype: string
     @return: the absolute page name
     """
-    if config.allow_subpages:
-        if pagename.startswith(PARENT_PREFIX):
-            pagename = '/'.join(filter(None, context.split('/')[:-1] + [pagename[PARENT_PREFIX_LEN:]]))
-        elif pagename.startswith(CHILD_PREFIX):
-            pagename = context + '/' + pagename[CHILD_PREFIX_LEN:]
-
+    if pagename.startswith(PARENT_PREFIX):
+        pagename = '/'.join(filter(None, context.split('/')[:-1] + [pagename[PARENT_PREFIX_LEN:]]))
+    elif pagename.startswith(CHILD_PREFIX):
+        pagename = context + '/' + pagename[CHILD_PREFIX_LEN:]
     return pagename
 
 def pagelinkmarkup(pagename):
     """ return markup that can be used as link to page <pagename> """
-    if isStrictWikiname(pagename): # XXX TODO re.match(Parser.word_rule, pagename)
+    from MoinMoin.parser.wiki import Parser
+    if re.match(Parser.word_rule + "$", pagename):
         return pagename
     else:
         return u'["%s"]' % pagename
@@ -559,72 +683,77 @@ def pagelinkmarkup(pagename):
 ### Plugins
 #############################################################################
 
+class PluginError(Exception):
+    """ Base class for plugin errors """
+
+class PluginMissingError(PluginError):
+    """ Raised when a plugin is not found """
+
+class PluginAttributeError(PluginError):
+    """ Raised when plugin does not contain an attribtue """
+
+
 def importPlugin(cfg, kind, name, function="execute"):
     """ Import wiki or builtin plugin
     
-    Returns an object from a plugin module or None if module or
-    'function' is not found.
+    Returns function from a plugin module name. If name can not be
+    imported, raise PluginMissingError. If function is missing, raise
+    PluginAttributeError.
 
     kind may be one of 'action', 'formatter', 'macro', 'processor',
     'parser' or any other directory that exist in MoinMoin or
     data/plugin
 
     Wiki plugins will always override builtin plugins. If you want
-    specific plugin, use either importWikiPlugin or importName directly.
+    specific plugin, use either importWikiPlugin or importBuiltinPlugin
+    directly.
     
     @param cfg: wiki config instance
     @param kind: what kind of module we want to import
     @param name: the name of the module
     @param function: the function name
-    @rtype: callable
+    @rtype: any object
     @return: "function" of module "name" of kind "kind", or None
     """
-    # Try to import from the wiki
-    plugin = importWikiPlugin(cfg, kind, name, function)
-    if plugin is None:
-        # Try to get the plugin from MoinMoin
-        modulename = 'MoinMoin.%s.%s' % (kind, name)
-        plugin = pysupport.importName(modulename, function)
-        
-    return plugin
+    try:
+        return importWikiPlugin(cfg, kind, name, function)
+    except PluginMissingError:
+        return importBuiltinPlugin(kind, name, function)
 
 
-# Here we cache our plugins until we have a wiki object.
-# WARNING: do not access directly in threaded enviromnent!
-_wiki_plugins = {}
-
 def importWikiPlugin(cfg, kind, name, function):
     """ Import plugin from the wiki data directory
     
-    We try to import only ONCE - then cache the plugin, even if we got
-    None. This way we prevent expensive import of existing plugins for
-    each call to a plugin.
-
-    @param cfg: wiki config instance
-    @param kind: what kind of module we want to import
-    @param name: the name of the module
-    @param function: the function name
-    @rtype: callable
-    @return: "function" of module "name" of kind "kind", or None
+    See importPlugin docstring.
     """
-    global _wiki_plugins
+    if not name in wikiPlugins(kind, cfg):
+        raise PluginMissingError
+    moduleName = '%s.plugin.%s.%s' % (cfg.siteid, kind, name)
+    return importNameFromPlugin(moduleName, function)
 
-    # Wiki plugins are located under 'wikiconfigname.plugin' module.
-    modulename = '%s.plugin.%s.%s' % (cfg.siteid, kind, name)
-    key = (modulename, function)
+
+def importBuiltinPlugin(kind, name, function):
+    """ Import builtin plugin from MoinMoin package 
+    
+    See importPlugin docstring.
+    """
+    if not name in builtinPlugins(kind):
+        raise PluginMissingError
+    moduleName = 'MoinMoin.%s.%s' % (kind, name)
+    return importNameFromPlugin(moduleName, function)
+
+
+def importNameFromPlugin(moduleName, name):
+    """ Return name from plugin module 
+    
+    Raise PluginAttributeError if name does not exists.
+    """
+    module = __import__(moduleName, globals(), {}, [name])
     try:
-        # Try cache first - fast!
-        plugin = _wiki_plugins[key]
-    except KeyError:
-        # Try to import from disk and cache result - slow!
-        plugin = pysupport.importName(modulename, function)
-        _wiki_plugins[key] = plugin
+        return getattr(module, name)
+    except AttributeError:
+        raise PluginAttributeError
 
-    return plugin
-
-# If we use threads, make this function thread safe
-if config.use_threads:
-    importWikiPlugin = pysupport.makeThreadSafe(importWikiPlugin)
 
 def builtinPlugins(kind):
     """ Gets a list of modules in MoinMoin.'kind'
@@ -634,12 +763,14 @@ def builtinPlugins(kind):
     @return: module names
     """
     modulename = "MoinMoin." + kind
-    plugins = pysupport.importName(modulename, "modules")
-    return plugins or []
+    return pysupport.importName(modulename, "modules")
 
 
 def wikiPlugins(kind, cfg):
     """ Gets a list of modules in data/plugin/'kind'
+ 
+    Require valid plugin directory. e.g missing 'parser' directory or
+    missing '__init__.py' file will raise errors.
     
     @param kind: what kind of modules we look for
     @rtype: list
@@ -647,8 +778,7 @@ def wikiPlugins(kind, cfg):
     """
     # Wiki plugins are located in wikiconfig.plugin module
     modulename = '%s.plugin.%s' % (cfg.siteid, kind)
-    plugins = pysupport.importName(modulename, "modules")
-    return plugins or []
+    return pysupport.importName(modulename, "modules")
 
 
 def getPlugins(kind, cfg):
@@ -686,24 +816,21 @@ def getParserForExtension(cfg, extension):
     @rtype: class, None
     @returns: the parser class or None
     """
-    ##
-    ## this is not a caching until cfg is persistent between requests!!!
-    ##
     if not hasattr(cfg, '_EXT_TO_PARSER'):
         import types
         etp, etd = {}, None
         for pname in getPlugins('parser', cfg):
-            Parser = importPlugin(cfg, 'parser', pname, 'Parser')
-            if Parser is not None:
-                if hasattr(Parser, 'extensions'):
-                    exts = Parser.extensions
-                    if type(exts) == types.ListType:
-                        for ext in Parser.extensions:
-                            etp[ext] = Parser
-                    elif str(exts) == '*':
-                        etd = Parser
-        # Cache in cfg for current request - this is not thread safe
-        # when cfg will be cached per wiki in long running process.
+            try:
+                Parser = importPlugin(cfg, 'parser', pname, 'Parser')
+            except PluginMissingError:
+                continue
+            if hasattr(Parser, 'extensions'):
+                exts = Parser.extensions
+                if type(exts) == types.ListType:
+                    for ext in Parser.extensions:
+                        etp[ext] = Parser
+                elif str(exts) == '*':
+                    etd = Parser
         cfg._EXT_TO_PARSER = etp
         cfg._EXT_TO_PARSER_DEFAULT = etd
         
@@ -795,10 +922,9 @@ def taintfilename(basename):
     @rtype: string
     @return: (safer) filename
     """
-    basename = basename.replace(os.pardir, '_')
-    basename = basename.replace(':', '_')
-    basename = basename.replace('/', '_')
-    basename = basename.replace('\\', '_')
+    for x in (os.pardir, ':', '/', '\\', '<', '>'):
+        basename = basename.replace(x, '_')
+
     return basename
 
 
@@ -835,7 +961,7 @@ def getUnicodeIndexGroup(name):
     if u'\uAC00' <= c <= u'\uD7AF': # Hangul Syllables
         return unichr(0xac00 + (int(ord(c) - 0xac00) / 588) * 588)
     else:
-        return c
+        return c.upper() # we put lower and upper case words into the same index group
 
 
 def isStrictWikiname(name, word_re=re.compile(ur"^(?:[%(u)s][%(l)s]+){2,}$" % {'u':config.chars_upper, 'l':config.chars_lower})):
@@ -858,7 +984,7 @@ def isPicture(url):
     @return: true if url points to a picture
     """
     extpos = url.rfind(".")
-    return extpos > 0 and url[extpos:].lower() in ['.gif', '.jpg', '.jpeg', '.png']
+    return extpos > 0 and url[extpos:].lower() in ['.gif', '.jpg', '.jpeg', '.png', '.bmp', '.ico', ]
 
 
 def link_tag(request, params, text=None, formatter=None, on=None, **kw):
@@ -886,7 +1012,7 @@ def link_tag(request, params, text=None, formatter=None, on=None, **kw):
                 formatter.url(0))
     if on != None and not on:
         return '</a>'
-        
+    
     attrs = ''
     if kw.has_key('attrs'):
         attrs += ' ' + kw['attrs']
@@ -1010,6 +1136,7 @@ def send_title(request, text, **keywords):
     @keyword pagename: 'PageName'
     @keyword page: the page instance that called us.
     @keyword print_mode: 1 (or 0)
+    @keyword editor_mode: 1 (or 0)
     @keyword media: css media type, defaults to 'screen'
     @keyword allow_doubleclick: 1 (or 0)
     @keyword html_head: additional <head> code
@@ -1033,11 +1160,12 @@ def send_title(request, text, **keywords):
     page_front_page = getFrontPage(request).page_name
     page_help_contents = getSysPage(request, 'HelpContents').page_name
     page_title_index = getSysPage(request, 'TitleIndex').page_name
+    page_site_navigation = getSysPage(request, 'SiteNavigation').page_name
     page_word_index = getSysPage(request, 'WordIndex').page_name
     page_user_prefs = getSysPage(request, 'UserPreferences').page_name
     page_help_formatting = getSysPage(request, 'HelpOnFormatting').page_name
     page_find_page = getSysPage(request, 'FindPage').page_name
-    page_home_page = getattr(getHomePage(request), 'page_name', None)
+    home_page = getInterwikiHomePage(request) # XXX sorry theme API change!!! Either None or tuple (wikiname,pagename) now.
     page_parent_page = getattr(page.getParentPage(), 'page_name', None)
     
     # Prepare the HTML <head> element
@@ -1064,7 +1192,10 @@ def send_title(request, text, **keywords):
     # for the original, English pages as well as for (the possibly
     # modified) frontpage:
     elif pagename in [page_front_page, request.cfg.page_front_page,
-                      page_title_index, ]:
+                      page_title_index, 'TitleIndex',
+                      page_find_page, 'FindPage',
+                      page_site_navigation, 'SiteNavigation',
+                      'RecentChanges',]:
         user_head.append(request.cfg.html_head_index)
     # if it is a normal page, index it, but do not follow the links, because
     # there are a lot of illegal links (like actions) or duplicates:
@@ -1085,13 +1216,14 @@ def send_title(request, text, **keywords):
 %s
 """ % (
         ''.join(user_head),
-        keywords.get('html_head', ''),
         request.theme.html_head({
+            'page': page,
             'title': escape(text),
             'sitename': escape(request.cfg.html_pagetitle or request.cfg.sitename),
             'print_mode': keywords.get('print_mode', False),
             'media': keywords.get('media', 'screen'),
-        })
+        }),
+        keywords.get('html_head', ''),
     ))
 
     # Links
@@ -1136,7 +1268,7 @@ def send_title(request, text, **keywords):
         '<link rel="Glossary" href="%s/%s">\n' % (scriptname, quoteWikinameURL(page_word_index)),
         '<link rel="Help" href="%s/%s">\n' % (scriptname, quoteWikinameURL(page_help_formatting)),
                   ])
-
+    
     output.append("</head>\n")
     request.write(''.join(output))
     output = []
@@ -1153,10 +1285,10 @@ def send_title(request, text, **keywords):
         not keywords.get('print_mode', 0) and
         request.user.edit_on_doubleclick):
         if request.user.may.write(pagename): # separating this gains speed
-            querystr = escape(util.web.makeQueryString({'action': 'edit'}))
-            # TODO: remove escape=0 in 1.4
+            querystr = escape(makeQueryString({'action': 'edit'}))
+            # TODO: remove escape=0 in 2.0
             url = page.url(request, querystr, escape=0)
-            bodyattr.append(''' ondblclick="location.href='%s'"''' % url)
+            bodyattr.append(''' ondblclick="location.href='%s'" ''' % url)
 
     # Set body to the user interface language and direction
     bodyattr.append(' %s' % request.theme.ui_lang_attr())
@@ -1175,6 +1307,7 @@ def send_title(request, text, **keywords):
         d = {'title_text': text, 'title_link': None, 'page': page,}
         request.themedict = d
         output.append(theme.startPage())
+        output.append(theme.interwiki(d))      
         output.append(theme.title(d))      
 
     # In standard mode, emit theme.header
@@ -1193,7 +1326,7 @@ def send_title(request, text, **keywords):
             'page_name': pagename or '',
             'page_find_page': page_find_page,
             'page_front_page': page_front_page,
-            'page_home_page': page_home_page,
+            'home_page': home_page,
             'page_help_contents': page_help_contents,
             'page_help_formatting': page_help_formatting,
             'page_parent_page': page_parent_page,
@@ -1224,7 +1357,10 @@ def send_title(request, text, **keywords):
         request.themedict = d
 
         # now call the theming code to do the rendering
-        output.append(theme.header(d))
+        if keywords.get('editor_mode', 0):
+            output.append(theme.editorheader(d))
+        else:
+            output.append(theme.header(d))
     
     # emit it
     request.write(''.join(output))
@@ -1238,8 +1374,6 @@ def send_footer(request, pagename, **keywords):
 
     @param request: the request object
     @param pagename: WikiName of the page
-    @keyword editable: true, when page is editable (default: true)
-    @keyword showpage: true, when link back to page is wanted (default: false)
     @keyword print_mode: true, when page is displayed in Print mode
     """
     d = request.themedict
@@ -1250,8 +1384,6 @@ def send_footer(request, pagename, **keywords):
         request.write(theme.pageinfo(d['page']))
         request.write(theme.endPage())
     else:
-        # This is used only by classic now, kill soon
-        d['footer_fragments'] = request._footer_fragments
         request.write(theme.footer(d, **keywords))
 
     
